package hu.bme.mit.mobilegen.componentmodel.ui.builder;

import static com.google.common.collect.Maps.uniqueIndex;
import static org.eclipse.xtext.ui.util.ResourceUtil.getContainer;
import hu.bme.mit.mobilegen.componentmodel.componentmodel.Componentmodel;
import hu.bme.mit.mobilegen.componentmodel.componentmodel.ComponentmodelFactory;
import hu.bme.mit.mobilegen.componentmodel.componentmodel.Content;
import hu.bme.mit.mobilegen.componentmodel.componentmodel.Element;
import hu.bme.mit.mobilegen.componentmodel.componentmodel.impl.ComponentmodelFactoryImpl;
import hu.bme.mit.mobilegen.componentmodel.generator.GeneratorConstants;
import hu.bme.mit.mobilegen.componentmodel.generator.GeneratorData;
import hu.bme.mit.mobilegen.componentmodel.generator.IComponentmodelGenerator;
import hu.bme.mit.mobilegen.componentmodel.helper.Utility;
import hu.bme.mit.mobilegen.componentmodel.helper.WrappedModel;
import hu.bme.mit.mobilegen.componentmodel.logger.Log;
import hu.bme.mit.mobilegen.componentmodel.preprocessor.PreProcessor;

import java.io.File;
import java.io.IOException;
import java.util.Collections;
import java.util.Date;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.SubMonitor;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.emf.common.util.TreeIterator;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.xtext.EcoreUtil2;
import org.eclipse.xtext.builder.EclipseResourceFileSystemAccess2;
import org.eclipse.xtext.generator.OutputConfiguration;
import org.eclipse.xtext.ui.resource.IResourceSetProvider;
import org.eclipse.xtext.util.CancelIndicator;

import com.google.common.base.Function;
import com.google.common.collect.Iterators;
import com.google.inject.Inject;
import com.google.inject.Provider;

/**
 * 
 * Componentmodel Builder
 * 
 */
public class ComponentmodelBuilder {

	@Inject
	private Provider<EclipseResourceFileSystemAccess2> fileSystemAccessProvider;

	@Inject
	private IComponentmodelGenerator generator;

	@Inject
	private IResourceSetProvider resourceSetProvider;

	@Inject
	private ComponentmodelOutputConfigurations outputConfigurations;

	private IProject developerProject;
	private IProject targetProject;

	private Date startTime;
	private Date endTime;

	private IResource[] srcResources;
	private ResourceSet rs;

	private EclipseResourceFileSystemAccess2 access;

	private Map<String, OutputConfiguration> outputConfig;

	/**
	 * 
	 * Generating Componentmodel from Project content
	 * @param: Project
	 * 
	 */
	public void generateComponentModel(IProject project) {
		startTime = new Date();
		Log.raw(Utility.formatTime(startTime) + " - STARTING COMPONENTMODEL GENERATION");

		developerProject = project;

		rs = resourceSetProvider.get(developerProject);

		if (!checkDeveloperProject(developerProject)) {
			return;
		}

		if (!parseConfiguration()) {
			return;
		}

		if (!searchModelFiles()) {
			return;
		}

		if (!configureTargetProject()) {
			return;
		}

		if (!processModel()) {
			return;
		}

		endTime = new Date();

		Log.separator();
		Log.raw(Utility.formatTime(endTime) + " - GENERATION PROCESS ENDED SUCCESSFULLY");
	}
	
	/**
	 * 
	 * Exporting Componentmodel to XMI file
	 * @param: Project
	 * 
	 */
	public void exportComponentModelToXMI(IProject project) {
		startTime = new Date();
		Log.raw(Utility.formatTime(startTime) + " - STARTING COMPONENTMODEL EXPORTING TO XMI");

		developerProject = project;

		rs = resourceSetProvider.get(developerProject);

		if (!checkDeveloperProject(developerProject)) {
			return;
		}

		if (!searchModelFiles()) {
			return;
		}

		if (!exportModelToXMI()) {
			return;
		}

		endTime = new Date();

		Log.separator();
		Log.raw(Utility.formatTime(endTime) + " - EXPORTING PROCESS ENDED SUCCESSFULLY");
	}

	/**
	 * 
	 * Generating Componentmodel from XMI resource
	 * @param: XMI Resource file
	 * 
	 */
	public void generateComponentmodelFromXMI(IFile file) {
		startTime = new Date();
		Log.raw(Utility.formatTime(startTime) + " - STARTING COMPONENTMODEL GENERATION");

		developerProject = file.getProject();

		rs = new ResourceSetImpl();

		if (!loadXMIResource(file)) {
			return;
		}

		if (!parseConfiguration()) {
			return;
		}

		if (!configureTargetProject()) {
			return;
		}

		if (!processModel()) {
			return;
		}

		endTime = new Date();

		Log.separator();
		Log.raw(Utility.formatTime(endTime) + " - GENERATION PROCESS ENDED SUCCESSFULLY");
	}

	/**
	 * 
	 * Checking developer project
	 * @param: Developer Project
	 * 
	 */
	private boolean checkDeveloperProject(IProject developerProject) {
		Log.header("STARTING TO CHECK COMPONENTMODEL PROJECT");

		Log.info("Checking developer project");
		IFolder src = developerProject.getFolder(BuilderConstants.SRC_FOLDER);

		if (!src.exists()) {
			Log.error("src folder not found in Modeling project - " + developerProject.getFullPath().toString());
			Log.terminated();
			return false;
		}

		try {
			srcResources = src.members();
		} catch (CoreException e) {
			Log.error("Componentmodel files not found in project - " + developerProject.getFullPath().toString());
			Log.terminated();
			return false;
		}

		Log.info("Checking project ended successfully");
		Log.separator();

		return true;
	}

	/**
	 * 
	 * Parsing configuration file
	 * 
	 */
	private boolean parseConfiguration() {
		if (!ConfigurationParser.getConfigurations(developerProject)) {
			Log.error("Error in parsing config file");
			Log.terminated();
			return false;
		}

		Log.info("Parsing configuration file ended successfully");
		Log.separator();
		return true;
	}

	/**
	 * 
	 * Searching Componentmodel files in src folder
	 * 
	 */
	private boolean searchModelFiles() {
		Log.header("STARTING SEARCH FOR COMPONENTMODEL FILES");

		int resources = 0;
		for (IResource r : srcResources) {
			if (r instanceof IFile) {
				if (((IFile) r).getFileExtension().equals(FileExtensions.COMPONENTMODEL_EXTENSION)) {
					Log.info("Found componentmodel resource - " + r.getFullPath().toString());
					IFile file = (IFile) r;
					Resource resource = rs.getResource(URI.createURI(file.getFullPath().toString()), true);

					final SubMonitor progress = SubMonitor.convert(new NullProgressMonitor(), 100);
					CancelIndicator cancelMonitor = new CancelIndicator() {
						public boolean isCanceled() {
							return progress.isCanceled();
						}
					};

					EcoreUtil2.resolveLazyCrossReferences(resource, cancelMonitor);

					resources++;
				}
			}
		}

		if (resources == 0) {
			Log.error("No componentmodel files found in project - " + developerProject.getFullPath().toString());
			Log.terminated();
			return false;
		}

		Log.info("Componentmodel files collected");
		Log.separator();

		return true;
	}

	/**
	 * 
	 * Configuring target project
	 * 
	 */
	private boolean configureTargetProject() {
		Log.header("SARTING TO CONFIGURE TARGET PROJECT - " + GeneratorData.getProjectName());

		access = fileSystemAccessProvider.get();
		targetProject = ResourcesPlugin.getWorkspace().getRoot().getProject(GeneratorData.getProjectName());

		if (!targetProject.exists()) {
			Log.error(GeneratorData.getProjectName() + " project does not exists in this Workspace");
			Log.terminated();
			return false;
		}

		Log.info("Target project is: " + GeneratorData.getProjectName());

		GeneratorData.setProject(targetProject);
		access.setProject(targetProject);

		outputConfig = getOutputConfigurations(targetProject);

		access.setOutputConfigurations(outputConfig);
		access.setMonitor(new NullProgressMonitor());

		Log.info("Setting up configurations succedded");
		Log.separator();

		return true;
	}

	/**
	 * 
	 * Exporting Componentmodel to XMI file
	 * 
	 */
	private boolean exportModelToXMI() {
		Log.info("Exporting componentmodel to xmi file");

		ResourceSet resourceSet = new ResourceSetImpl();
		EcoreUtil2.clone(resourceSet, rs);

		URI fileURI = URI.createFileURI(new File(developerProject.getLocation() + "\\" + developerProject.getName() + "_componentmodel.xmi").getAbsolutePath());
		Resource resource = resourceSet.createResource(fileURI);

		TreeIterator<Notifier> content = resourceSet.getAllContents();
		Iterator<Element> elements = Iterators.<Element> filter(content, Element.class);

		ComponentmodelFactory factory = new ComponentmodelFactoryImpl();
		Componentmodel cm = factory.createComponentmodel();
		Content c = factory.createContent();
		cm.setContent(c);

		while (elements.hasNext()) {
			Element e = elements.next();
			c.getElement().add(e);
		}
		
		resource.getContents().add(cm);

		try {
			resource.save(Collections.EMPTY_MAP);
			developerProject.refreshLocal(IResource.DEPTH_INFINITE, null);
			Log.info("XMI File generated - File location: " + fileURI.toFileString());
		} catch (IOException | CoreException e) {
			e.printStackTrace();
			Log.warning("Exporting to XMI Failed");
			return true;
		}

		return true;
	}

	/**
	 * 
	 * Processing and Generating Componentmodel
	 * 
	 */
	private boolean processModel() {
		try {

			Log.header("STARTING CODE GENERATION PROCESS");

			WrappedModel wm = PreProcessor.preProcessModel(rs);

			if (!cleanOutput(outputConfig, targetProject)) {
				Log.terminated();
				return false;
			}

			Log.separator();
			Log.header("STARTING GENERATION");

			generator.generate(wm, access);

			targetProject.refreshLocal(IResource.DEPTH_INFINITE, null);

			return true;
		} catch (RuntimeException | CoreException e) {
			Log.error("Error occured during generation process");
			Log.terminated();
			return false;
		}

	}

	/**
	 * 
	 * Getting output configurations for project
	 * @param: Project
	 * 
	 */
	private Map<String, OutputConfiguration> getOutputConfigurations(IProject project) {
		Log.info("Setting up output configurations");

		Set<OutputConfiguration> configurations = outputConfigurations.getOutputConfigurations(project);
		return uniqueIndex(configurations, new Function<OutputConfiguration, String>() {
			public String apply(OutputConfiguration from) {
				return from.getName();
			}
		});
	}

	/**
	 * 
	 * Cleaning target project's output folders
	 * @param: Output configurations
	 * @param: Project
	 * 
	 */
	private boolean cleanOutput(Map<String, OutputConfiguration> outputConfigurations, IProject project) {
		Log.info("Start cleaning target project - " + GeneratorConstants.SRC_GEN);

		if (!cleanOutputDirectory(project, outputConfigurations.get(GeneratorConstants.SRC_GEN_ACT).getOutputDirectory())) {
			return false;
		}
		if (!cleanOutputDirectory(project, outputConfigurations.get(GeneratorConstants.SRC_GEN_FRAG).getOutputDirectory())) {
			return false;
		}
		if (!cleanOutputDirectory(project, outputConfigurations.get(GeneratorConstants.SRC_GEN_APP).getOutputDirectory())) {
			return false;
		}
		if (!cleanOutputDirectory(project, outputConfigurations.get(GeneratorConstants.SRC_GEN_ASYNC).getOutputDirectory())) {
			return false;
		}

		Log.info("Cleaning was successfull");
		return true;
	}

	/**
	 * 
	 * Cleaning target project's output folder
	 * @param: Project
	 * @param: Folder path
	 * 
	 */
	private boolean cleanOutputDirectory(IProject project, String path) {
		Log.info("Cleaning - " + path);

		IContainer container = getContainer(project, path);
		if (container.exists()) {
			try {
				for (IResource r : container.members()) {
					r.delete(IResource.KEEP_HISTORY, null);
				}

				return true;
			} catch (CoreException e) {
				Log.error("Error occured while cleaning output - " + path);
				return false;
			}
		}

		return true;
	}

	/**
	 * 
	 * Loading Componentmodel form XMI resource file
	 * @param: XMI Resource file
	 * 
	 */
	private boolean loadXMIResource(IFile file) {
		Log.header("STARTING TO LOAD COMPONENTMODEL FROM XMI FILE");

		Log.info("Loading resource - " + file.getFullPath().toString());
		Resource resource = rs.getResource(URI.createURI(file.getFullPath().toString()), true);

		if (resource == null) {
			Log.error("Error occured while loading resource - " + file.getFullPath().toString());
			return false;
		} else {
			Log.info("Resource loaded");
			return true;
		}
	}

}
