package hu.bme.mit.mobilegen.componentmodel.ui.builder;

import hu.bme.mit.mobilegen.componentmodel.generator.GeneratorData;
import hu.bme.mit.mobilegen.componentmodel.logger.Log;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceChangeEvent;
import org.eclipse.core.resources.IResourceChangeListener;
import org.eclipse.core.resources.IResourceDelta;
import org.eclipse.core.resources.IResourceDeltaVisitor;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jdt.core.IClasspathEntry;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.JavaModelException;

/**
 * 
 * Configuration file change listener
 * 
 */
public class ConfigurationChangeListener implements IResourceChangeListener {
	
	/**
	 * 
	 * Resource Changed Event Handler
	 * @param: Event
	 * 
	 */
	@Override
	public void resourceChanged(IResourceChangeEvent event) {
		
        switch (event.getType()) {
           case IResourceChangeEvent.POST_CHANGE:
             
              try {
            	  event.getDelta().accept(new IResourceDeltaVisitor() {

            		  @Override
            		  public boolean visit(IResourceDelta delta) throws CoreException {
            			  return handleChange(delta.getResource());
            		  }
					  
            	  });
				
              } catch (CoreException e) {
            	  e.printStackTrace();
              }
           
              break;
              
        }
        
	}
	
	/**
	 * 
	 * Handling change of resource
	 * @param: Resource
	 * 
	 */
	private boolean handleChange(IResource res){
		if(res instanceof IFile){
    		if(((IFile)res).getFileExtension().equals(FileExtensions.CONFIGURATION_FILE_EXTENSION)){
    			
    			Job job = new Job("Update ClassPath") {
    			
    				@Override
    				protected IStatus run(IProgressMonitor monitor) {
    					updateClassPath((IFile)res, res.getProject());
    					return Status.OK_STATUS;
    				}
    			};

    			job.schedule(); 
    			
    			return true;
    		}
    	}
		else if(res instanceof IFolder){
			if(((IFolder)res).getName().equals("bin")){
				return false;
			}
		}
		
		return true;
	}
	
	/**
	 * 
	 * Updating projects classpath
	 * @param: Configuration file
	 * @param: Project
	 * 
	 */
	private void updateClassPath(IFile file, IProject project){
		ConfigurationParser.parseConfiguration(file);
		
		IProject targetProject = ResourcesPlugin.getWorkspace().getRoot().getProject(GeneratorData.getProjectName());
    	
    	if(!targetProject.exists()){
    		Log.error(GeneratorData.getProjectName() + " project does not exists in this Workspace");
    		return;
    	}
    	
    	IJavaProject javaProject = JavaCore.create(project);
		
		try {
			IClasspathEntry[] oldEntries = javaProject.getRawClasspath();
			boolean found = false;
			for(IClasspathEntry ent : oldEntries){
				System.out.println(ent.getPath().toString() + " is still added to Java Build path");
			
				if(ent.getPath().toOSString().equals("\\" + GeneratorData.getProjectName())){
					found = true;
				}
			}
			if(!found){
				IClasspathEntry[] newEntries = new IClasspathEntry[oldEntries.length +1];
				System.arraycopy(oldEntries, 0, newEntries, 0, oldEntries.length);
				newEntries[oldEntries.length] = JavaCore.newProjectEntry(new Path("\\" + GeneratorData.getProjectName()), false);
				javaProject.setRawClasspath(newEntries, null);
				
				System.out.println("New ClassPath Entry Added - " + "\\" + GeneratorData.getProjectName());
				Log.info("New ClassPath Entry Added - " + "\\" + GeneratorData.getProjectName());
			}
			
		} catch (JavaModelException e) {
			System.out.println("Error occured in setting Java ClassPath");
		}
    	
	}

}
