package hu.bme.mit.mobilegen.componentmodel.generator.activity

import hu.bme.mit.mobilegen.componentmodel.componentmodel.Activity
import hu.bme.mit.mobilegen.componentmodel.componentmodel.ActivityType
import hu.bme.mit.mobilegen.componentmodel.componentmodel.Association
import hu.bme.mit.mobilegen.componentmodel.componentmodel.AsyncTask
import hu.bme.mit.mobilegen.componentmodel.componentmodel.ConnectionType
import hu.bme.mit.mobilegen.componentmodel.componentmodel.Fragment
import hu.bme.mit.mobilegen.componentmodel.componentmodel.Property
import hu.bme.mit.mobilegen.componentmodel.componentmodel.StartActivity
import hu.bme.mit.mobilegen.componentmodel.generator.GeneratorConstants
import hu.bme.mit.mobilegen.componentmodel.generator.GeneratorData
import hu.bme.mit.mobilegen.componentmodel.helper.FormatPropertyType
import hu.bme.mit.mobilegen.componentmodel.logger.Log
import org.eclipse.xtext.generator.IFileSystemAccess

class ActivityGenerator {
	
	def generateActivityUnit(Activity a, IFileSystemAccess fsa){
		Log.info("Generating " + a.name.toFirstUpper + ".java");
		fsa.generateFile(a.name.toFirstUpper + ".java", GeneratorConstants.SRC_GEN_ACT, generateActivity(a));
	}
	
	def private generateActivity(Activity a) '''
		
		«a.classHeader»
		
		«packageDeclaration»
		
		«a.classImports»
		
		«a.classDeclaration»
		
	'''
	
	def private classHeader(Activity a) '''
		//------------------------- Geneated Java Class File For Activity - «a.name» ---------------------------------
	'''
	
	def private packageDeclaration() '''
		package «GeneratorData.packageName».«GeneratorConstants.SRC_GEN_ACT»;
	'''
	
	def private classImports(Activity a) '''
		«IF a.type.equals(ActivityType.LIST_ACTIVITY)»
			import android.app.ListActivity;
		«ELSE»
			import android.app.Activity;
		«ENDIF»
		import android.os.Bundle;
		
		import android.content.BroadcastReceiver;
		import android.content.Context;
		import android.content.Intent;
		import android.support.v4.content.LocalBroadcastManager;
		import android.content.IntentFilter;
		import java.util.List;
		import android.support.v4.app.Fragment;
		import android.support.v4.app.FragmentActivity;
		import android.support.v4.app.FragmentManager;
		import «GeneratorData.packageName».«GeneratorConstants.SRC_GEN_APP».«GeneratorData.getProjectName + "Application"»;
		
		«IF a.fragments.length > 0»
			import «GeneratorData.packageName».«GeneratorConstants.SRC_GEN_FRAG».*;
		«ENDIF»
		import «GeneratorData.packageName».R;
		«FOR f:a.async_tasks»
			import «GeneratorData.packageName».«GeneratorConstants.SRC_GEN_ASYNC».«f.name.toFirstUpper»AsyncTask;
		«ENDFOR»
		
	'''
	
	def private classDeclaration(Activity a) '''
		public class «a.name.toFirstUpper» extends «IF a.fragments.length > 0»Fragment«ELSEIF a.type.equals(ActivityType.LIST_ACTIVITY)»List«ENDIF»Activity {
			
			«a.properties»
			
			«a.associations»
			
			«a.resultAsyncTasks»
			
			«a.overrideMethods»
			
			«a.startActivities»
			
			«a.startAsyncTasks»
			
			«a.finishActivity»
		}
	'''
	
	def private properties(Activity a) '''
		«FOR f:a.property»
			«f.propertyDeclaration»
		«ENDFOR»
	'''
	
	def private propertyDeclaration(Property p) '''
		«IF p.connectionType.equals(ConnectionType.ONE)»
			«p.simpleProperty»
		«ELSE»
			«p.listProperty»
		«ENDIF»
	'''
	
	def private simpleProperty(Property p) '''
		protected «FormatPropertyType.formatPrimitive(p.type)» «p.name.toFirstLower»;
	'''
	
	def private listProperty(Property p) '''
		protected List<«FormatPropertyType.formatClass(p.type)»> «p.name.toFirstLower»;
	'''
	
	def private associations(Activity a) '''
		«FOR f:a.association»
			«f.associationDeclaration»
		«ENDFOR»
	'''
	
	def private associationDeclaration(Association a) '''
		«IF a.connectionType.equals(ConnectionType.ONE)»
			«a.simpleAssociation»
		«ELSE»
			«a.listAssociation»
		«ENDIF»
	'''
	
	def private simpleAssociation(Association a) '''
		protected «a.connectTo.qualifiedName» «a.name.toFirstLower»;
	'''
	
	def private listAssociation(Association a) '''
		protected List<«a.connectTo.qualifiedName»> «a.name.toFirstLower»;
	'''
	
	def private overrideMethods(Activity a) '''
		protected void onCreate(Bundle savedInstanceState){
			super.onCreate(savedInstanceState);
			«IF a.fragments.length > 0»
				String fragment = getIntent().getExtras().getString(«GeneratorData.getProjectName + "Application"».INTENT_FRAGMENT);
				
				«FOR f:a.fragments»
					«fragments(f, a.fargment_frame)»
				«ENDFOR»
			«ENDIF»
		}

		protected void onStart(){
			super.onStart();
		}

		protected void onRestart(){
			super.onRestart();
		}

		protected void onResume(){
			«IF a.result_async_tasks.length  > 0»
				LocalBroadcastManager.getInstance(this).registerReceiver(messageReceiver, new IntentFilter(«GeneratorData.getProjectName + "Application"».INTENT_FILTER));
			«ENDIF»
			super.onResume();
		}

		protected void onPause(){
			super.onPause();
			«IF a.result_async_tasks.length  > 0»
				LocalBroadcastManager.getInstance(this).unregisterReceiver(messageReceiver);
			«ENDIF»
		}
		
		protected void onStop(){
			super.onStop();
		}
		
		protected void onDestroy(){
			super.onDestroy();
		}
	'''
	
	def private fragments(Fragment f, String fragmentFrame) '''
		if(«GeneratorData.getProjectName + "Application"».«f.name.toUpperCase».equals(fragment)){
			try {
				FragmentManager fragmentManager = getSupportFragmentManager();
				fragmentManager.beginTransaction().replace(R.id.«fragmentFrame», (Fragment) «GeneratorData.getProjectName + "Application"».get«f.name.toFirstUpper»Class().newInstance()).commit();
			} catch (InstantiationException e) {
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				e.printStackTrace();
			}
		}
	'''
	
	def private startActivities(Activity a) '''
		// Strating other atcivities
		
		«FOR f:a.activities»
			«startActivity(f, a)»
			
		«ENDFOR»
	'''
	
	def private startActivity(StartActivity sa, Activity a) '''
		protected void start«sa.activity.name.toFirstUpper»«IF sa.showFragment != null»«sa.showFragment.name.toFirstUpper»«ENDIF»«IF sa.noReturn»WithoutReturn«ENDIF»(Class<? extends «sa.activity.name.toFirstUpper»> clazz){
			Intent i = new Intent(getApplicationContext(), clazz);
			«FOR f:sa.sendAssoc»
				i.putExtra(«GeneratorData.getProjectName + "Application"».«a.name.toUpperCase + "_" + f.name.toFirstLower.toUpperCase», «f.name.toFirstLower»);
			«ENDFOR»
			«IF sa.showFragment != null»
				i.putExtra(«GeneratorData.getProjectName + "Application"».INTENT_FRAGMENT, «GeneratorData.getProjectName + "Application"».«sa.showFragment.name.toUpperCase»);
			«ENDIF»
			startActivity(i);
			«IF sa.noReturn»finish();«ENDIF»
		}
	'''
	
	def private startAsyncTasks(Activity a) '''
		// Strating other async tasks
		
		«FOR f:a.async_tasks»
			«startAsyncTask(f)»
			
		«ENDFOR»
	'''
	
	def private startAsyncTask(AsyncTask a) '''
		protected void start«a.name.toFirstUpper»AsyncTask(String param){
			new «a.name.toFirstUpper»AsyncTask(getApplicationContext()).execute(param);
		}
	'''
	
	def private resultAsyncTasks(Activity a) '''
		// Strating other result
		
		«IF a.result_async_tasks.length  > 0»
			protected BroadcastReceiver messageReceiver = new BroadcastReceiver() {
				
				@Override
				public void onReceive(Context context, Intent intent) {
					«FOR f:a.result_async_tasks»
						{
							«resultAsyncTask(f)»
						}
					«ENDFOR»
				}
			};
			
			«FOR f:a.result_async_tasks»
				«resultAsyncTaskMethod(f)»
			«ENDFOR»
			
		«ENDIF»
		
	'''
	
	def private resultAsyncTask(AsyncTask a) '''
		String result = intent.getStringExtra(«GeneratorData.getProjectName + "Application"».«a.name.toUpperCase»);
		if(result != null){
			result«a.name.toUpperCase»(result);
		}
	'''
	
	def private resultAsyncTaskMethod(AsyncTask a) '''
		protected void result«a.name.toUpperCase»(String result){
			
		}
	'''
	
	def private finishActivity(Activity a) '''
		protected void navigateBack(){
			finish();
		}
	'''
	
}