package hu.bme.mit.mobilegen.componentmodel.generator.application

import hu.bme.mit.mobilegen.componentmodel.componentmodel.Fragment
import hu.bme.mit.mobilegen.componentmodel.generator.GeneratorConstants
import hu.bme.mit.mobilegen.componentmodel.generator.GeneratorData
import hu.bme.mit.mobilegen.componentmodel.logger.Log
import java.util.List
import java.util.Set
import org.eclipse.xtext.generator.IFileSystemAccess

class ApplicationGenerator {
	
	Set<String> consts;
	List<Fragment> frags;
	
	def generateApplication(List<Fragment> fragments, Set<String> constants, IFileSystemAccess fsa){
		frags = fragments;
		consts = constants;
		
		Log.info("Generating " + GeneratorData.getProjectName + "Application" + ".java");
		fsa.generateFile(GeneratorData.getProjectName + "Application" + ".java", GeneratorConstants.SRC_GEN_APP, generateApplication);
	}
	
	def private generateApplication() '''
		«classHeader»
		
		«packageDeclaration»
		
		«classImports»
		
		«classDeclaration»
	'''
	
	def private classHeader() '''
		//------------------------- Geneated Java Class File For Application - «GeneratorData.getProjectName + "Application"» ---------------------------------
	'''
	
	def private packageDeclaration() '''
		package «GeneratorData.packageName».«GeneratorConstants.SRC_GEN_APP»;
	'''
	
	def private classImports() '''
		«IF frags.size > 0»
			import «GeneratorData.packageName».«GeneratorConstants.SRC_GEN_FRAG».*;
		«ENDIF»
		
		import android.app.Application;
	'''
	
	def private classDeclaration() '''
		public class «GeneratorData.getProjectName + "Application"» extends Application {
			
			«constants»
			
			«fragmentsClass»
			
			«callbacks»
			
			«fragmentsGetterSetters»
			
		}
	'''
	
	def private constants() '''
		«FOR f:consts»
			public static final String «f.toUpperCase» = "«f.toLowerCase»";
		«ENDFOR»
		
		public static final String INTENT_FILTER = "async_response_filter";
		
		public static final String INTENT_FRAGMENT = "fragment";
	'''
	
	def private fragmentsClass() '''
		«FOR f:frags»
			private static Class<? extends «f.name.toFirstUpper»> «f.name.toFirstLower»Class;
		«ENDFOR»
	'''
	
	def private callbacks() '''
		@Override
		public void onCreate() {
			super.onCreate();
		}

		@Override
		public void onTerminate() {
			super.onTerminate();
		}
	'''
	
	def private fragmentsGetterSetters() '''
		«FOR f:frags»
			«f.fragmentGetter»
			
			«f.fragmentSetter»
			
		«ENDFOR»
	'''
	
	def private fragmentGetter(Fragment f) '''
		public static Class<? extends «f.name.toFirstUpper»> get«f.name.toFirstUpper»Class() {
			return «f.name.toFirstLower»Class;
		}
	'''
	
	def private fragmentSetter(Fragment f) '''
		protected static void set«f.name.toFirstUpper»Class(Class<? extends «f.name.toFirstUpper»> «f.name.toFirstLower»Class) {
			«GeneratorData.getProjectName + "Application"».«f.name.toFirstLower»Class = «f.name.toFirstLower»Class;
		}
	'''
	
}