package hu.bme.mit.mobilegen.componentmodel.generator.asynctask

import hu.bme.mit.mobilegen.componentmodel.componentmodel.AsyncTask
import hu.bme.mit.mobilegen.componentmodel.generator.GeneratorConstants
import hu.bme.mit.mobilegen.componentmodel.generator.GeneratorData
import hu.bme.mit.mobilegen.componentmodel.logger.Log
import org.eclipse.xtext.generator.IFileSystemAccess

class AsyncTaskGenerator {
	
	def generateAsyncTaskUnit(AsyncTask a, IFileSystemAccess fsa){
		Log.info("Generating " + a.name.toFirstUpper + "AsyncTask.java");
		fsa.generateFile(a.name.toFirstUpper + "AsyncTask.java", GeneratorConstants.SRC_GEN_ASYNC, generateAsyncTask(a));
	}
	
	def private generateAsyncTask(AsyncTask a) '''
		«a.classHeader»
		
		«packageDeclaration»
		
		«classImports»
		
		«a.classDeclaration»
	'''
	
	def private classHeader(AsyncTask a) '''
		//------------------------- Geneated Java Class File For Async Task - «a.name» ---------------------------------
	'''
	
	def private packageDeclaration() '''
		package «GeneratorData.packageName».«GeneratorConstants.SRC_GEN_ASYNC»;
	'''
	
	def private classImports() '''
		import java.io.BufferedReader;
		import java.io.InputStream;
		import java.io.InputStreamReader;
		
		import org.apache.http.HttpEntity;
		import org.apache.http.HttpResponse;
		import org.apache.http.HttpStatus;
		import org.apache.http.client.HttpClient;
		import org.apache.http.client.methods.HttpGet;
		import org.apache.http.impl.client.DefaultHttpClient;

		import android.content.Context;
		import android.content.Intent;
		import android.os.AsyncTask;
		import android.support.v4.content.LocalBroadcastManager;
		
		import «GeneratorData.packageName».«GeneratorConstants.SRC_GEN_APP».«GeneratorData.getProjectName + "Application"»;
	'''
	
	def private classDeclaration(AsyncTask a) '''
		public class «a.name.toFirstUpper»AsyncTask extends AsyncTask<String, Void, String> {
			
			«a.asyncTaskMethods»
		}
	'''
	
	def private asyncTaskMethods(AsyncTask a) '''
		private Context context = null;

		public «a.name.toFirstUpper»AsyncTask(Context context) {
			this.context = context;
		}

		@Override
		protected void onPreExecute() {
			super.onPreExecute();
		}

		@Override
		protected String doInBackground(String... params) {
			if(params.length > 0){
				
				HttpClient httpclient = new DefaultHttpClient();
				HttpGet httpget = new HttpGet(params[0]);
				HttpResponse response = null;

				String result = "";
				InputStream is = null;
				
				try {
					response = httpclient.execute(httpget);

					System.out.println(response.getStatusLine().getStatusCode());
					if (response.getStatusLine().getStatusCode() == HttpStatus.SC_OK) {
						HttpEntity entity = response.getEntity();
						if (entity != null) {
							is = entity.getContent();

							BufferedReader reader = new BufferedReader(new InputStreamReader(is));
							StringBuilder sb = new StringBuilder();
							String line = null;

							while ((line = reader.readLine()) != null) {
								sb.append(line + "\n");
							}
							
							System.out.println(sb.toString());
							result = sb.toString();
						}
					}

				} catch (Exception e) {
					e.printStackTrace();
					result = e.getMessage();
				} finally {
					if (is != null) {
						try {
							is.close();
						} catch (Exception e) {
							e.printStackTrace();
						}
					}
				}
				return result;
			}
			
			return "";
		}

		@Override
		protected void onPostExecute(String result) {
			super.onPostExecute(result);
			System.out.println(result);
			Intent respIntent = new Intent(«GeneratorData.getProjectName + "Application"».INTENT_FILTER);
		
			respIntent.putExtra(«GeneratorData.getProjectName + "Application"».«a.name.toUpperCase», result);
			LocalBroadcastManager.getInstance(context).sendBroadcast(respIntent);
		}
	'''
	
}