package hu.bme.mit.mobilegen.componentmodel.serializer;

import com.google.inject.Inject;
import com.google.inject.Provider;
import hu.bme.mit.mobilegen.componentmodel.componentmodel.Activity;
import hu.bme.mit.mobilegen.componentmodel.componentmodel.Association;
import hu.bme.mit.mobilegen.componentmodel.componentmodel.AsyncTask;
import hu.bme.mit.mobilegen.componentmodel.componentmodel.Componentmodel;
import hu.bme.mit.mobilegen.componentmodel.componentmodel.ComponentmodelPackage;
import hu.bme.mit.mobilegen.componentmodel.componentmodel.Content;
import hu.bme.mit.mobilegen.componentmodel.componentmodel.Fragment;
import hu.bme.mit.mobilegen.componentmodel.componentmodel.Import;
import hu.bme.mit.mobilegen.componentmodel.componentmodel.Property;
import hu.bme.mit.mobilegen.componentmodel.componentmodel.StartActivity;
import hu.bme.mit.mobilegen.componentmodel.services.ComponentmodelGrammarAccess;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.xtext.serializer.acceptor.ISemanticSequenceAcceptor;
import org.eclipse.xtext.serializer.acceptor.SequenceFeeder;
import org.eclipse.xtext.serializer.diagnostic.ISemanticSequencerDiagnosticProvider;
import org.eclipse.xtext.serializer.diagnostic.ISerializationDiagnostic.Acceptor;
import org.eclipse.xtext.serializer.sequencer.AbstractDelegatingSemanticSequencer;
import org.eclipse.xtext.serializer.sequencer.GenericSequencer;
import org.eclipse.xtext.serializer.sequencer.ISemanticNodeProvider.INodesForEObjectProvider;
import org.eclipse.xtext.serializer.sequencer.ISemanticSequencer;
import org.eclipse.xtext.serializer.sequencer.ITransientValueService;
import org.eclipse.xtext.serializer.sequencer.ITransientValueService.ValueTransient;

@SuppressWarnings("all")
public class ComponentmodelSemanticSequencer extends AbstractDelegatingSemanticSequencer {

	@Inject
	private ComponentmodelGrammarAccess grammarAccess;
	
	public void createSequence(EObject context, EObject semanticObject) {
		if(semanticObject.eClass().getEPackage() == ComponentmodelPackage.eINSTANCE) switch(semanticObject.eClass().getClassifierID()) {
			case ComponentmodelPackage.ACTIVITY:
				if(context == grammarAccess.getActivityRule() ||
				   context == grammarAccess.getElementRule()) {
					sequence_Activity(context, (Activity) semanticObject); 
					return; 
				}
				else break;
			case ComponentmodelPackage.ASSOCIATION:
				if(context == grammarAccess.getAssociationRule()) {
					sequence_Association(context, (Association) semanticObject); 
					return; 
				}
				else break;
			case ComponentmodelPackage.ASYNC_TASK:
				if(context == grammarAccess.getAsyncTaskRule() ||
				   context == grammarAccess.getElementRule()) {
					sequence_AsyncTask(context, (AsyncTask) semanticObject); 
					return; 
				}
				else break;
			case ComponentmodelPackage.COMPONENTMODEL:
				if(context == grammarAccess.getComponentmodelRule()) {
					sequence_Componentmodel(context, (Componentmodel) semanticObject); 
					return; 
				}
				else break;
			case ComponentmodelPackage.CONTENT:
				if(context == grammarAccess.getContentRule()) {
					sequence_Content(context, (Content) semanticObject); 
					return; 
				}
				else break;
			case ComponentmodelPackage.FRAGMENT:
				if(context == grammarAccess.getElementRule() ||
				   context == grammarAccess.getFragmentRule()) {
					sequence_Fragment(context, (Fragment) semanticObject); 
					return; 
				}
				else break;
			case ComponentmodelPackage.IMPORT:
				if(context == grammarAccess.getImportRule()) {
					sequence_Import(context, (Import) semanticObject); 
					return; 
				}
				else break;
			case ComponentmodelPackage.PROPERTY:
				if(context == grammarAccess.getPropertyRule()) {
					sequence_Property(context, (Property) semanticObject); 
					return; 
				}
				else break;
			case ComponentmodelPackage.START_ACTIVITY:
				if(context == grammarAccess.getStartActivityRule()) {
					sequence_StartActivity(context, (StartActivity) semanticObject); 
					return; 
				}
				else break;
			}
		if (errorAcceptor != null) errorAcceptor.accept(diagnosticProvider.createInvalidContextOrTypeDiagnostic(semanticObject, context));
	}
	
	/**
	 * Constraint:
	 *     (
	 *         name=ID 
	 *         type=ActivityType 
	 *         (property+=Property property+=Property*)? 
	 *         (association+=Association association+=Association*)? 
	 *         (fargment_frame=STRING fragments+=[Fragment|QualifiedName] fragments+=[Fragment|QualifiedName]*)? 
	 *         (activities+=StartActivity activities+=StartActivity*)? 
	 *         (async_tasks+=[AsyncTask|QualifiedName] async_tasks+=[AsyncTask|QualifiedName]*)? 
	 *         (result_async_tasks+=[AsyncTask|QualifiedName] result_async_tasks+=[AsyncTask|QualifiedName]*)?
	 *     )
	 */
	protected void sequence_Activity(EObject context, Activity semanticObject) {
		genericSequencer.createSequence(context, semanticObject);
	}
	
	
	/**
	 * Constraint:
	 *     (connectionType=ConnectionType connectTo=[JvmType|QualifiedName] name=STRING)
	 */
	protected void sequence_Association(EObject context, Association semanticObject) {
		if(errorAcceptor != null) {
			if(transientValues.isValueTransient(semanticObject, ComponentmodelPackage.Literals.ASSOCIATION__CONNECTION_TYPE) == ValueTransient.YES)
				errorAcceptor.accept(diagnosticProvider.createFeatureValueMissing(semanticObject, ComponentmodelPackage.Literals.ASSOCIATION__CONNECTION_TYPE));
			if(transientValues.isValueTransient(semanticObject, ComponentmodelPackage.Literals.ASSOCIATION__CONNECT_TO) == ValueTransient.YES)
				errorAcceptor.accept(diagnosticProvider.createFeatureValueMissing(semanticObject, ComponentmodelPackage.Literals.ASSOCIATION__CONNECT_TO));
			if(transientValues.isValueTransient(semanticObject, ComponentmodelPackage.Literals.ASSOCIATION__NAME) == ValueTransient.YES)
				errorAcceptor.accept(diagnosticProvider.createFeatureValueMissing(semanticObject, ComponentmodelPackage.Literals.ASSOCIATION__NAME));
		}
		INodesForEObjectProvider nodes = createNodeProvider(semanticObject);
		SequenceFeeder feeder = createSequencerFeeder(semanticObject, nodes);
		feeder.accept(grammarAccess.getAssociationAccess().getConnectionTypeConnectionTypeEnumRuleCall_0_0(), semanticObject.getConnectionType());
		feeder.accept(grammarAccess.getAssociationAccess().getConnectToJvmTypeQualifiedNameParserRuleCall_1_0_1(), semanticObject.getConnectTo());
		feeder.accept(grammarAccess.getAssociationAccess().getNameSTRINGTerminalRuleCall_2_0(), semanticObject.getName());
		feeder.finish();
	}
	
	
	/**
	 * Constraint:
	 *     name=ID
	 */
	protected void sequence_AsyncTask(EObject context, AsyncTask semanticObject) {
		if(errorAcceptor != null) {
			if(transientValues.isValueTransient(semanticObject, ComponentmodelPackage.Literals.ELEMENT__NAME) == ValueTransient.YES)
				errorAcceptor.accept(diagnosticProvider.createFeatureValueMissing(semanticObject, ComponentmodelPackage.Literals.ELEMENT__NAME));
		}
		INodesForEObjectProvider nodes = createNodeProvider(semanticObject);
		SequenceFeeder feeder = createSequencerFeeder(semanticObject, nodes);
		feeder.accept(grammarAccess.getAsyncTaskAccess().getNameIDTerminalRuleCall_1_0(), semanticObject.getName());
		feeder.finish();
	}
	
	
	/**
	 * Constraint:
	 *     content=Content
	 */
	protected void sequence_Componentmodel(EObject context, Componentmodel semanticObject) {
		if(errorAcceptor != null) {
			if(transientValues.isValueTransient(semanticObject, ComponentmodelPackage.Literals.COMPONENTMODEL__CONTENT) == ValueTransient.YES)
				errorAcceptor.accept(diagnosticProvider.createFeatureValueMissing(semanticObject, ComponentmodelPackage.Literals.COMPONENTMODEL__CONTENT));
		}
		INodesForEObjectProvider nodes = createNodeProvider(semanticObject);
		SequenceFeeder feeder = createSequencerFeeder(semanticObject, nodes);
		feeder.accept(grammarAccess.getComponentmodelAccess().getContentContentParserRuleCall_0(), semanticObject.getContent());
		feeder.finish();
	}
	
	
	/**
	 * Constraint:
	 *     (name=QualifiedName imports+=Import* element+=Element*)
	 */
	protected void sequence_Content(EObject context, Content semanticObject) {
		genericSequencer.createSequence(context, semanticObject);
	}
	
	
	/**
	 * Constraint:
	 *     (name=ID type=FragmentType)
	 */
	protected void sequence_Fragment(EObject context, Fragment semanticObject) {
		if(errorAcceptor != null) {
			if(transientValues.isValueTransient(semanticObject, ComponentmodelPackage.Literals.ELEMENT__NAME) == ValueTransient.YES)
				errorAcceptor.accept(diagnosticProvider.createFeatureValueMissing(semanticObject, ComponentmodelPackage.Literals.ELEMENT__NAME));
			if(transientValues.isValueTransient(semanticObject, ComponentmodelPackage.Literals.FRAGMENT__TYPE) == ValueTransient.YES)
				errorAcceptor.accept(diagnosticProvider.createFeatureValueMissing(semanticObject, ComponentmodelPackage.Literals.FRAGMENT__TYPE));
		}
		INodesForEObjectProvider nodes = createNodeProvider(semanticObject);
		SequenceFeeder feeder = createSequencerFeeder(semanticObject, nodes);
		feeder.accept(grammarAccess.getFragmentAccess().getNameIDTerminalRuleCall_1_0(), semanticObject.getName());
		feeder.accept(grammarAccess.getFragmentAccess().getTypeFragmentTypeEnumRuleCall_4_0(), semanticObject.getType());
		feeder.finish();
	}
	
	
	/**
	 * Constraint:
	 *     importedNamespace=QualifiedNameWithWildcard
	 */
	protected void sequence_Import(EObject context, Import semanticObject) {
		if(errorAcceptor != null) {
			if(transientValues.isValueTransient(semanticObject, ComponentmodelPackage.Literals.IMPORT__IMPORTED_NAMESPACE) == ValueTransient.YES)
				errorAcceptor.accept(diagnosticProvider.createFeatureValueMissing(semanticObject, ComponentmodelPackage.Literals.IMPORT__IMPORTED_NAMESPACE));
		}
		INodesForEObjectProvider nodes = createNodeProvider(semanticObject);
		SequenceFeeder feeder = createSequencerFeeder(semanticObject, nodes);
		feeder.accept(grammarAccess.getImportAccess().getImportedNamespaceQualifiedNameWithWildcardParserRuleCall_1_0(), semanticObject.getImportedNamespace());
		feeder.finish();
	}
	
	
	/**
	 * Constraint:
	 *     (connectionType=ConnectionType type=PropertyType name=STRING)
	 */
	protected void sequence_Property(EObject context, Property semanticObject) {
		if(errorAcceptor != null) {
			if(transientValues.isValueTransient(semanticObject, ComponentmodelPackage.Literals.PROPERTY__CONNECTION_TYPE) == ValueTransient.YES)
				errorAcceptor.accept(diagnosticProvider.createFeatureValueMissing(semanticObject, ComponentmodelPackage.Literals.PROPERTY__CONNECTION_TYPE));
			if(transientValues.isValueTransient(semanticObject, ComponentmodelPackage.Literals.PROPERTY__TYPE) == ValueTransient.YES)
				errorAcceptor.accept(diagnosticProvider.createFeatureValueMissing(semanticObject, ComponentmodelPackage.Literals.PROPERTY__TYPE));
			if(transientValues.isValueTransient(semanticObject, ComponentmodelPackage.Literals.PROPERTY__NAME) == ValueTransient.YES)
				errorAcceptor.accept(diagnosticProvider.createFeatureValueMissing(semanticObject, ComponentmodelPackage.Literals.PROPERTY__NAME));
		}
		INodesForEObjectProvider nodes = createNodeProvider(semanticObject);
		SequenceFeeder feeder = createSequencerFeeder(semanticObject, nodes);
		feeder.accept(grammarAccess.getPropertyAccess().getConnectionTypeConnectionTypeEnumRuleCall_0_0(), semanticObject.getConnectionType());
		feeder.accept(grammarAccess.getPropertyAccess().getTypePropertyTypeEnumRuleCall_1_0(), semanticObject.getType());
		feeder.accept(grammarAccess.getPropertyAccess().getNameSTRINGTerminalRuleCall_2_0(), semanticObject.getName());
		feeder.finish();
	}
	
	
	/**
	 * Constraint:
	 *     (
	 *         noReturn?='@NoReturn'? 
	 *         (sendAssoc+=[Association|ID] sendAssoc+=[Association|ID]*)? 
	 *         (sendProp+=[Property|ID] sendProp+=[Property|ID]*)? 
	 *         showFragment=[Fragment|QualifiedName]? 
	 *         activity=[Activity|QualifiedName]
	 *     )
	 */
	protected void sequence_StartActivity(EObject context, StartActivity semanticObject) {
		genericSequencer.createSequence(context, semanticObject);
	}
}
