package hu.bme.mit.mobilegen.componentmodel.preprocessor;

import hu.bme.mit.mobilegen.componentmodel.componentmodel.Activity;
import hu.bme.mit.mobilegen.componentmodel.componentmodel.AsyncTask;
import hu.bme.mit.mobilegen.componentmodel.componentmodel.Fragment;
import hu.bme.mit.mobilegen.componentmodel.logger.Log;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.emf.common.util.TreeIterator;
import org.eclipse.emf.ecore.resource.ResourceSet;

import com.google.common.collect.Iterators;

/**
 * 
 * ModelWrapper
 * 
 */
public class ModelWrapper {
	
	/**
	 * 
	 * Processing model
	 * @param: List of model elements
	 * 
	 */
	public static void processModel(ResourceSet rs, List<Activity> activities, List<Fragment> fragments, List<AsyncTask> async_tasks){
		activities.addAll(processActivities(rs));
		fragments.addAll(processFragments(rs));
		async_tasks.addAll(processAsyncTasks(rs));
	}
	
	/**
	 * 
	 * Processing activities
	 * @param: ResourceSet with whole model
	 * 
	 */
	private static List<Activity> processActivities(ResourceSet rs){
		List<Activity> list = new ArrayList<Activity>();
		
		TreeIterator<Notifier> allCont = rs.getAllContents();
		Iterator<Activity> activities = Iterators.<Activity>filter(allCont, Activity.class);
		
		while(activities.hasNext()){
			Activity a = activities.next();
			list.add(a);
			
			Log.info("Activity: " + a.getName());
		}
		
		return list;
	}
	
	/**
	 * 
	 * Processing fragments
	 * @param: ResourceSet with whole model
	 * 
	 */
	private static List<Fragment> processFragments(ResourceSet rs){
		List<Fragment> list = new ArrayList<Fragment>();
		
		TreeIterator<Notifier> allCont = rs.getAllContents();
		Iterator<Fragment> fragments = Iterators.<Fragment>filter(allCont, Fragment.class);
		
		while(fragments.hasNext()){
			Fragment a = fragments.next();
			list.add(a);
			
			Log.info("Fragment: " + a.getName());
		}
		
		return list;
	}
	
	/**
	 * 
	 * Processing async tasks
	 * @param: ResourceSet with whole model
	 * 
	 */
	private static List<AsyncTask> processAsyncTasks(ResourceSet rs){
		List<AsyncTask> list = new ArrayList<AsyncTask>();
		
		TreeIterator<Notifier> allCont = rs.getAllContents();
		Iterator<AsyncTask> async_tasks = Iterators.<AsyncTask>filter(allCont, AsyncTask.class);
		
		while(async_tasks.hasNext()){
			AsyncTask a = async_tasks.next();
			list.add(a);
			
			Log.info("Async Task: " + a.getName());
		}
		
		return list;
	}
	
}
