package hu.bme.mit.mobilegen.componentmodel.validation;

import hu.bme.mit.mobilegen.componentmodel.componentmodel.Activity;
import hu.bme.mit.mobilegen.componentmodel.componentmodel.Association;
import hu.bme.mit.mobilegen.componentmodel.componentmodel.AsyncTask;
import hu.bme.mit.mobilegen.componentmodel.componentmodel.ComponentmodelPackage;
import hu.bme.mit.mobilegen.componentmodel.componentmodel.Content;
import hu.bme.mit.mobilegen.componentmodel.componentmodel.Element;
import hu.bme.mit.mobilegen.componentmodel.componentmodel.Fragment;
import hu.bme.mit.mobilegen.componentmodel.componentmodel.Property;
import hu.bme.mit.mobilegen.componentmodel.componentmodel.StartActivity;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.TreeIterator;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.xtext.resource.IResourceDescription;
import org.eclipse.xtext.resource.IResourceDescriptions;
import org.eclipse.xtext.resource.IResourceServiceProvider;
import org.eclipse.xtext.resource.impl.ResourceDescriptionsProvider;
import org.eclipse.xtext.validation.Check;
import org.eclipse.xtext.validation.CheckType;

import com.google.common.collect.Iterators;
import com.google.inject.Inject;

/**
 * 
 * Validator
 * 
 */
public class ComponentmodelValidatorJava extends ComponentmodelValidator {
	
	@Inject
	protected ResourceDescriptionsProvider resourceDescriptionsProvider;
	
	@Inject
	private IResourceServiceProvider resourceServiceProvider;
	
	/**
	 * 
	 * Validating Element name length
	 * @param: Element
	 * 
	 */
	@Check(CheckType.NORMAL)
	public void checkNameLengthElement(Element e){
		if(e.getName().length() < 2){
			error("Names must contain at least 2 characters!", ComponentmodelPackage.Literals.ELEMENT__NAME);
		}
	}
	
	/**
	 * 
	 * Validating URI
	 * @param: Content
	 * 
	 */
	@Check(CheckType.FAST)
	public void checkURI(Content c){
		
		for(String s : ReservedStrings.getReservedEndings()){
			if(c.getName().toLowerCase().endsWith(s.toLowerCase())){
				error("URI can not end with " + s + "! It's a reserved keyword for the generator!", ComponentmodelPackage.Literals.ELEMENT__NAME);
			}
		}
		
		for(String s : ReservedStrings.getReservedStrings()){
			if(c.getName().toLowerCase().equals(s.toLowerCase())){
				error("URI can not be " + s + "! It's a reserved keyword for the Java language!", ComponentmodelPackage.Literals.ELEMENT__NAME);
			}
		}
		
		for(String s : ReservedStrings.getForbiddenStrings()){
			if(c.getName().toLowerCase().contains(s.toLowerCase())){
				error("URI can not contain " + s + "! It's a forbidden String!", ComponentmodelPackage.Literals.ELEMENT__NAME);
			}
		}
		
	}
	
	/**
	 * 
	 * Validating Element name
	 * @param: Element
	 * 
	 */
	@Check(CheckType.FAST)
	public void checkElementName(Element e){
		
		for(String s : ReservedStrings.getReservedEndings()){
			if(e.getName().toLowerCase().endsWith(s.toLowerCase())){
				error("Element name can not end with " + s + "! It's a reserved keyword for the generator!", ComponentmodelPackage.Literals.ELEMENT__NAME);
			}
		}
		
		for(String s : ReservedStrings.getReservedStrings()){
			if(e.getName().toLowerCase().equals(s.toLowerCase())){
				error("Element name can not be " + s + "! It's a reserved keyword for the Java language!", ComponentmodelPackage.Literals.ELEMENT__NAME);
			}
		}
		
		for(String s : ReservedStrings.getForbiddenStrings()){
			if(e.getName().toLowerCase().contains(s.toLowerCase())){
				error("Element name can not contain " + s + "! It's a forbidden String!", ComponentmodelPackage.Literals.ELEMENT__NAME);
			}
		}
		
	}
	
	/**
	 * 
	 * Validating Element names
	 * @param: Element
	 * 
	 */
	@Check(CheckType.FAST)
	public void checkElementNames(Element e){
		ResourceSet rs = findOtherContent(e.eResource());
		TreeIterator<Notifier> content = rs.getAllContents();
		Iterator<Element> elements = Iterators.<Element>filter(content, Element.class);
		
		boolean error = false;
		while(elements.hasNext()){
			if(elements.next().getName().equals(e.getName())){
				error = true;
				break;
			}
		}
		
		if(error){
			error("An other Element has the same name, change one, because Entitiy name must be unique!", ComponentmodelPackage.Literals.ELEMENT__NAME);
		}
		
	}
	
	/**
	 * 
	 * Validating URIs
	 * @param: Content
	 * 
	 */
	@Check(CheckType.FAST)
	public void checkURIs(Content c){
		ResourceSet rs = findOtherContent(c.eResource());
		TreeIterator<Notifier> content = rs.getAllContents();
		Iterator<Content> contents = Iterators.<Content>filter(content, Content.class);
		
		boolean error = false;
		while(contents.hasNext()){
			if(contents.next().getName().equals(c.getName())){
				error = true;
				break;
			}
		}
		
		if(error){
			error("An other Entity has the same URI, change one, because File URIs must be unique!", ComponentmodelPackage.Literals.CONTENT__NAME);
		}
		
	}
	
	/**
	 * 
	 * Validating Fragment List
	 * @param: Activity
	 * 
	 */
	@Check(CheckType.FAST)
	public void checkFragmentList(Activity a){
		Set<Fragment> fragmentSet = new HashSet<Fragment>();
		List<Fragment> fragmentList = a.getFragments();
		for(Fragment f : fragmentList){
			if(!fragmentSet.add(f)){
				error("A Fragment must be present once!", ComponentmodelPackage.Literals.ACTIVITY__FRAGMENTS);
			}	
		}
	}
	
	/**
	 * 
	 * Validating Fragment frame name
	 * @param: Activity
	 * 
	 */
	@Check(CheckType.FAST)
	public void checkFragmentFrame(Activity a){
		String name = a.getFargment_frame();
		for(String s : ReservedStrings.getReservedStrings()){
			if(name.toLowerCase().equals(s.toLowerCase())){
				error("FF can not be " + s + "! It's a reserved keyword for the Java language!", ComponentmodelPackage.Literals.ACTIVITY__FARGMENT_FRAME);
			}
		}
	}
	
	/**
	 * 
	 * Validating Activity Field names unique
	 * @param: Activity
	 * 
	 */
	@Check(CheckType.FAST)
	public void checkActFieldNames(Activity act){
		EList<Property> props = act.getProperty();
		EList<Association> as = act.getAssociation();
		Map<String, EObject> eObjs = new HashMap<String, EObject>();
		Set<EObject> objErrors = new HashSet<EObject>();
		
		for(Property p : props){
			if(eObjs.containsKey(p.getName())){
				objErrors.add(p);
				objErrors.add(eObjs.get(p.getName()));
			}
			else{
				eObjs.put(p.getName(), p);
			}
		}
		
		for(Association a : as){
			if(eObjs.containsKey(a.getName())){
				objErrors.add(a);
				objErrors.add(eObjs.get(a.getName()));
			}
			else{
				eObjs.put(a.getName(), a);
			}
		}
		
		for(EObject eo : objErrors){
			if(eo instanceof Property){
				error("Duplicate local filed!", (Property)eo, ComponentmodelPackage.Literals.PROPERTY__NAME);
			}
			else{
				error("Duplicate local filed!", (Association)eo, ComponentmodelPackage.Literals.ASSOCIATION__NAME);
			}
		}
		
	}
	
	/**
	 * 
	 * Validating AsyncTask
	 * @param: Activity
	 * 
	 */
	@Check(CheckType.FAST)
	public void asyncTasks(Activity a){
		
		List<AsyncTask> list = a.getAsync_tasks();
		
		Set<AsyncTask> asyncSet = new HashSet<AsyncTask>();
		for(AsyncTask as : list){
			if(!asyncSet.add(as)){
				error("An AS must be present once!", ComponentmodelPackage.Literals.ACTIVITY__ASYNC_TASKS);
			}	
		}
	}
	
	/**
	 * 
	 * Validating ResultAsyncTask
	 * @param: Activity
	 * 
	 */
	@Check(CheckType.FAST)
	public void resultAsyncTasks(Activity a){
		
		List<AsyncTask> list = a.getResult_async_tasks();
		
		Set<AsyncTask> asyncSet = new HashSet<AsyncTask>();
		for(AsyncTask as : list){
			if(!asyncSet.add(as)){
				error("A Result AS must be present once!", ComponentmodelPackage.Literals.ACTIVITY__RESULT_ASYNC_TASKS);
			}	
		}
	}
	
	/**
	 * 
	 * Validating StartActivity with fragments
	 * @param: Activity
	 * 
	 */
	@Check(CheckType.FAST)
	public void startActivities(Activity a){
		
		List<StartActivity> list = a.getActivities();
		
		Set<String> sAct = new HashSet<String>();
		for(StartActivity sa : list){
			String s1 = sa.getActivity().getName();
			String s2 = null;
			if(sa.getShowFragment() != null){
				s2 = sa.getShowFragment().getName();
			}
			String s3 = s1 + s2;
			if(!sAct.add(s3)){
				error("You can only start an Activity once or with another fragment!", ComponentmodelPackage.Literals.ACTIVITY__ACTIVITIES);
			}
		}
	}
	
	/**
	 * 
	 * Validating Send entity
	 * @param: StartActivity
	 * 
	 */
	@Check(CheckType.FAST)
	public void sendEntity(StartActivity a){
		
		List<Association> list = a.getSendAssoc();
		
		Set<Association> assoc = new HashSet<Association>();
		for(Association as : list){
			if(!assoc.add(as)){
				error("An Entity must be present once!", ComponentmodelPackage.Literals.START_ACTIVITY__SEND_ASSOC);
			}	
		}
	}
	
	/**
	 * 
	 * Validating Send property
	 * @param: StartActivity
	 * 
	 */
	@Check(CheckType.FAST)
	public void sendProp(StartActivity a){
		
		List<Property> list = a.getSendProp();
		
		Set<Property> props = new HashSet<Property>();
		for(Property p : list){
			if(!props.add(p)){
				error("A Property must be present once!", ComponentmodelPackage.Literals.START_ACTIVITY__SEND_PROP);
			}	
		}
	}
	
	/**
	 * 
	 * Finding other contents in other resources
	 * @param: Current resource
	 * 
	 */
	private ResourceSet findOtherContent(Resource resource) {
		ResourceSet rs = new ResourceSetImpl();
		IResourceDescriptions index = resourceDescriptionsProvider.createResourceDescriptions();
		IResourceDescription resDesc = resourceServiceProvider.getResourceDescriptionManager().getResourceDescription(resource);
		List<org.eclipse.xtext.resource.IContainer> visibleContainers = resourceServiceProvider.getContainerManager().getVisibleContainers(resDesc, index);
		for (org.eclipse.xtext.resource.IContainer c : visibleContainers) {
			for (IResourceDescription rd : c.getResourceDescriptions()) {
				if(!rd.getURI().equals(resource.getURI())){
					rs.getResource(rd.getURI(), true);
				}
			}
		}
		return rs;
	}
	
}