package hu.bme.mit.mobilegen.datamodel.ui.activator;

import java.util.Collections;
import java.util.Map;

import org.apache.log4j.Logger;
import org.eclipse.xtext.ui.shared.SharedStateModule;
import org.eclipse.xtext.util.Modules2;
import org.osgi.framework.BundleContext;

import com.google.common.collect.Maps;
import com.google.inject.Guice;
import com.google.inject.Injector;
import com.google.inject.Module;

import hu.bme.mit.mobilegen.datamodel.DatamodelGeneratorModule;
import hu.bme.mit.mobilegen.datamodel.ui.internal.DatamodelActivator;

public class DatamodelUIActivator extends DatamodelActivator {

public static final String HU_BME_MIT_MOBILEGEN_DATAMODEL_DATAMODEL = "hu.bme.mit.mobilegen.datamodel.Datamodel";
	
	private static final Logger logger = Logger.getLogger(DatamodelActivator.class);
	
	private static DatamodelActivator INSTANCE;
	
	private Map<String, Injector> injectors = Collections.synchronizedMap(Maps.<String, Injector> newHashMapWithExpectedSize(1));
	
	@Override
	public void start(BundleContext context) throws Exception {
		super.start(context);
		INSTANCE = this;
	}
	
	@Override
	public void stop(BundleContext context) throws Exception {
		injectors.clear();
		INSTANCE = null;
		super.stop(context);
	}
	
	public static DatamodelActivator getInstance() {
		return INSTANCE;
	}
	
	public Injector getInjector(String language) {
		synchronized (injectors) {
			Injector injector = injectors.get(language);
			if (injector == null) {
				injectors.put(language, injector = createInjector(language));
			}
			return injector;
		}
	}
	
	protected Injector createInjector(String language) {
		try {
			DatamodelGeneratorModule genMod = new DatamodelGeneratorModule();
			Module runtimeModule = getRuntimeModule(language);
			Module sharedStateModule = getSharedStateModule();
			Module uiModule = getUiModule(language);
			Module mergedModule = Modules2.mixin(runtimeModule, sharedStateModule, uiModule, genMod);
			return Guice.createInjector(mergedModule);
		} catch (Exception e) {
			logger.error("Failed to create injector for " + language);
			logger.error(e.getMessage(), e);
			throw new RuntimeException("Failed to create injector for " + language, e);
		}
	}

	protected Module getRuntimeModule(String grammar) {
		if (HU_BME_MIT_MOBILEGEN_DATAMODEL_DATAMODEL.equals(grammar)) {
			return new hu.bme.mit.mobilegen.datamodel.DatamodelRuntimeModule();
		}
		
		throw new IllegalArgumentException(grammar);
	}
	
	protected Module getUiModule(String grammar) {
		if (HU_BME_MIT_MOBILEGEN_DATAMODEL_DATAMODEL.equals(grammar)) {
			return new hu.bme.mit.mobilegen.datamodel.ui.DatamodelUiModule(this);
		}
		
		throw new IllegalArgumentException(grammar);
	}
	
	protected Module getSharedStateModule() {
		return new SharedStateModule();
	}
	
}
