package hu.bme.mit.mobilegen.datamodel.ui.builder;

import hu.bme.mit.mobilegen.datamodel.generator.DeviceType;
import hu.bme.mit.mobilegen.datamodel.generator.GeneratorData;
import hu.bme.mit.mobilegen.datamodel.generator.database.ORMType;
import hu.bme.mit.mobilegen.datamodel.logger.Log;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;

public class ConfigurationParser {

	public static boolean getConfigurations(IProject project){
		
		Log.header("STARTING TO PARSE CONFIGURATION FILE");
		
		IFolder srcFolder = project.getFolder(BuilderConstants.SRC_FOLDER);
    	IResource[] res = null;
    	
		try {
			res = srcFolder.members();
		} catch (CoreException e) {
			Log.error("Datamodel files not found in project - " + project.getFullPath().toString());
			return false;
		}
		
		List<IFile> confFiles = new ArrayList<IFile>();
		
    	for(IResource r : res){
    		if(r instanceof IFile){
    			if(((IFile)r).getFileExtension().equals(FileExtensions.CONFIGURATION_FILE_EXTENSION)){
    				confFiles.add((IFile)r);
    			}
    		}
    	}
    	
    	if(confFiles.size() == 0){
    		Log.error("No configuration files found in" + BuilderConstants.SRC_FOLDER + " folder  with " + FileExtensions.CONFIGURATION_FILE_EXTENSION + "extension");
    		return false;
    	}
    	else if (confFiles.size() > 1){
    		Log.error("More than one configuration files found in" + BuilderConstants.SRC_FOLDER + " folder  with " + FileExtensions.CONFIGURATION_FILE_EXTENSION + "extension");
    		return false;
    	}
    	else{
    		return parseConfiguration(confFiles.get(0));
    	}
    }
	
	public static boolean parseConfiguration(IFile file){
		IFile conf = file;
		Log.info("Parsing configuration file - " + conf.getFullPath().toString());
		
		IPath location = conf.getLocation();
		File f = location.toFile();
		
		boolean projName = false;
		boolean packName = false;
		
		BufferedReader br = null;
		try {
			br = new BufferedReader(new FileReader(f));
			String line;
			while((line = br.readLine()) != null){
				if(line.startsWith(ConfigurationFileParameters.PROJECT_NAME)){
    				int i = line.indexOf(":");
    				String projectName = line.substring(i + 1).trim();
    				if(projectName.length() != 0){
    					GeneratorData.setProjectName(projectName);
    					projName = true;
    				}
    				else{
    					Log.error("Project name parameter is not set in configuration file - " + conf.getFullPath().toString());
    		    		return false;
    				}
    			}
    			else if(line.startsWith(ConfigurationFileParameters.PACKAGE_NAME)){
    				int i = line.indexOf(":");
    				String packageName = line.substring(i + 1).trim();
    				if(packageName.length() != 0){
    					GeneratorData.setPackageName(packageName);
    					packName = true;
    				}
    				else{
    					Log.error("Package name parameter is not set in configuration file - " + conf.getFullPath().toString());
    		    		return false;
    				}
    			}
    			else if(line.startsWith(ConfigurationFileParameters.DEVICE)){
    				int i = line.indexOf(":");
    				String device = line.substring(i + 1).trim();
    				if(device.length() != 0){
    					boolean found = false;
    					for(DeviceType dt : DeviceType.values()){
    						if(dt.toString().equals(device)){
    							GeneratorData.setDevice(dt);
    							found = true;
    						}
    					}
    					if(!found){
    						Log.error("Device parameter is not valid in configuration file - " + conf.getFullPath().toString());
        		    		return false;
    					}
    				}
    			}
    			else if(line.startsWith(ConfigurationFileParameters.ORM_TYPE)){
    				int i = line.indexOf(":");
    				String orm = line.substring(i + 1).trim();
    				if(orm.length() != 0){
    					boolean found = false;
    					for(ORMType ot : ORMType.values()){
    						if(ot.toString().equals(orm)){
    							GeneratorData.setOrm(ot);
    							found = true;
    						}
    					}
    					if(!found){
    						Log.error("ORM parameter is not valid in configuration file - " + conf.getFullPath().toString());
        		    		return false;
    					}
    				}
    			}
    			else if(line.startsWith(ConfigurationFileParameters.DB_NAME)){
    				int i = line.indexOf(":");
    				String db_name = line.substring(i + 1).trim();
    				if(db_name.length() != 0){
    					GeneratorData.setDBName(db_name);
    				}
    			}
    			else if(line.startsWith(ConfigurationFileParameters.DB_VERSION)){
    				int i = line.indexOf(":");
    				String db_version = line.substring(i + 1).trim();
    				if(db_version.length() != 0){
    					try{
    						int ver = Integer.parseInt(db_version);
        					GeneratorData.setDBVersion(ver);
    					} catch(NumberFormatException e){
    						Log.error("DB Version parameter must be an Integer in configuration file - " + conf.getFullPath().toString());
    						return false;
    					}
    				}
    			}
			}
			
			if(!projName){
				Log.error("Project name parameter is not set in configuration file - " + conf.getFullPath().toString());
	    		return false;
			}
			
			if(!packName){
				Log.error("Package name parameter is not set in configuration file - " + conf.getFullPath().toString());
	    		return false;
			}
			
			br.close();
			
			return true;
			
		} catch (FileNotFoundException e) {
			return false;
		} catch (IOException e) {
			return false;
		} finally{
			try {
				br.close();
			} catch (IOException e) {
				return false;
			}
		}
	}
	
}
