package hu.bme.mit.mobilegen.datamodel.ui.builder;

import hu.bme.mit.mobilegen.datamodel.generator.GeneratorConstants;
import hu.bme.mit.mobilegen.datamodel.generator.GeneratorData;
import hu.bme.mit.mobilegen.datamodel.logger.Log;

import java.util.HashSet;
import java.util.Set;

import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.Path;
import org.eclipse.jdt.core.IClasspathEntry;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.IPackageFragmentRoot;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.xtext.generator.IFileSystemAccess;
import org.eclipse.xtext.generator.OutputConfiguration;

public class DatamodelOutputConfigurations {

	public Set<OutputConfiguration> getOutputConfigurations(IProject project) {
		Set<OutputConfiguration> configs = new HashSet<OutputConfiguration>();
		
		IFolder src = project.getFolder(GeneratorConstants.SRC);
		IFolder srcGen = project.getFolder(GeneratorConstants.SRC_GEN);
		
		Log.info("Checking tartget project");
		
		if(!src.exists()){
			try {
				Log.warning(GeneratorConstants.SRC + " folder does not exists");
				Log.info("Creating folder: " + src.getFullPath().toString());
				
				src.create(false, false, null);
				
				Log.info("Succeeded to create folder: " + src.getFullPath().toString());
			} catch (CoreException e) {
				Log.error("Failed to create folder: " + src.getFullPath().toString());
				e.printStackTrace();
			}
		}
		
		if(!srcGen.exists()){
			try {
				Log.warning(GeneratorConstants.SRC_GEN + " folder does not exists");
				Log.info("Creating folder: " + srcGen.getFullPath().toString());
				
				srcGen.create(false, false, null);
				
				Log.info("Succeeded to create folder: " + srcGen.getFullPath().toString());
			} catch (CoreException e) {
				Log.error("Failed to create folder: " + srcGen.getFullPath().toString());
				e.printStackTrace();
			}
		}
		
		Log.info("Checking Java Build paths");
		
		IJavaProject javaProject = JavaCore.create(project);
		IPackageFragmentRoot root = javaProject.getPackageFragmentRoot(srcGen);
		
		try {
			IClasspathEntry[] oldEntries = javaProject.getRawClasspath();
			boolean found = false;
			for(IClasspathEntry ent : oldEntries){
				Log.info(ent.getPath().toString() + " is still added to Java Build path");
			
				if(ent.getPath().toString().equals(srcGen.getFullPath().toString())){
					found = true;
				}
			}
			if(!found){
				Log.info("Adding new ClassPath Entry - " + root.getPath().toString());
				
				IClasspathEntry[] newEntries = new IClasspathEntry[oldEntries.length +1];
				System.arraycopy(oldEntries, 0, newEntries, 0, oldEntries.length);
				newEntries[oldEntries.length] = JavaCore.newSourceEntry(root.getPath());
				javaProject.setRawClasspath(newEntries, null);
			}
			
		} catch (JavaModelException e) {
			Log.error("Error occured in setting Java ClassPath");
			e.printStackTrace();
		}
		
		IFolder src_package = src.getFolder(GeneratorData.getPackageName().replaceAll("[//.]", "/"));
		IFolder srcGen_package = srcGen.getFolder(GeneratorData.getPackageName().replaceAll("[//.]", "/"));
		
		configs.add(createConfiguration(GeneratorConstants.SRC, src.getProjectRelativePath().toString(), false, false, true, false, false));
		configs.add(createConfiguration(GeneratorConstants.SRC_ENTITIES, src_package.getFolder(new Path(GeneratorConstants.SRC_ENTITIES)).getProjectRelativePath().toString(), true, true, true, true, true));
		configs.add(createConfiguration(GeneratorConstants.SRC_GEN, srcGen.getProjectRelativePath().toString(), true, true, true, true, true));
		configs.add(createConfiguration(GeneratorConstants.SRC_GEN_ENTITIES, srcGen_package.getFolder(new Path(GeneratorConstants.SRC_GEN_ENTITIES)).getProjectRelativePath().toString(), true, true, true, true, true));
		configs.add(createConfiguration(GeneratorConstants.SRC_GEN_SER, srcGen_package.getFolder(new Path(GeneratorConstants.SRC_GEN_SER)).getProjectRelativePath().toString(), true, true, true, true, true));
		configs.add(createConfiguration(GeneratorConstants.SRC_GEN_PREFS, srcGen_package.getFolder(new Path(GeneratorConstants.SRC_GEN_PREFS)).getProjectRelativePath().toString(), true, true, true, true, true));
		configs.add(createConfiguration(GeneratorConstants.SRC_GEN_DB, srcGen_package.getFolder(new Path(GeneratorConstants.SRC_GEN_DB)).getProjectRelativePath().toString(), true, true, true, true, true));
		configs.add(createConfiguration(IFileSystemAccess.DEFAULT_OUTPUT, srcGen.getProjectRelativePath().toString(), true, true, true, true, true));
		
		return configs;
	}
	
	private OutputConfiguration createConfiguration(String name, String outputDirectory, boolean canClearOutput, boolean cleanDerived, boolean createOutputDirectory, boolean overrideExistingResources, boolean setDerivedProperty) {
		final OutputConfiguration config = new OutputConfiguration(name);
		config.setOutputDirectory(outputDirectory);
		config.setCanClearOutputDirectory(canClearOutput);
		config.setCleanUpDerivedResources(cleanDerived);
		config.setCreateOutputDirectory(createOutputDirectory);
		config.setOverrideExistingResources(overrideExistingResources);
		config.setSetDerivedProperty(setDerivedProperty);
		return config;
	}

}
