package hu.bme.mit.mobilegen.datamodel.ui.handler;

import hu.bme.mit.mobilegen.datamodel.logger.Log;
import hu.bme.mit.mobilegen.datamodel.ui.builder.DatamodelBuilder;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.IHandler;
import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jdt.internal.core.JavaProject;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.handlers.HandlerUtil;

import com.google.inject.Inject;

@SuppressWarnings("restriction")
public class DatamodelXMIGeneratorHandler extends AbstractHandler implements IHandler {

	@Inject
	private DatamodelBuilder builder;

	@Override
	public Object execute(ExecutionEvent event) throws ExecutionException {

		ISelection selection = HandlerUtil.getCurrentSelection(event);

		if (selection instanceof IStructuredSelection) {

			IStructuredSelection structuredSelection = (IStructuredSelection) selection;
			Object firstElement = structuredSelection.getFirstElement();

			boolean isProject = false;
			IProject developerProject = null;

			if (firstElement instanceof IProject) {
				developerProject = (IProject) firstElement;
				try {
					IMarker[] ms = developerProject.findMarkers(IMarker.PROBLEM, true, IResource.DEPTH_INFINITE);
					for (IMarker m : ms) {
						Integer sevT = (Integer) m.getAttribute(IMarker.SEVERITY);
						if (sevT.intValue() == IMarker.SEVERITY_ERROR) {
							Log.error("Project contains errors");
							Log.terminated();
							return null;
						}
					}
				} catch (CoreException e) {
					Log.error("Error occured during generation process");
					Log.terminated();
					return null;
				}

				isProject = true;
			} else if (firstElement instanceof JavaProject) {
				JavaProject javaProject = (JavaProject) firstElement;
				developerProject = javaProject.getProject();
				try {
					IMarker[] ms = developerProject.findMarkers(IMarker.PROBLEM, true, IResource.DEPTH_INFINITE);
					for (IMarker m : ms) {
						Integer sevT = (Integer) m.getAttribute(IMarker.SEVERITY);
						if (sevT.intValue() == IMarker.SEVERITY_ERROR) {
							Log.error("Project contains errors");
							Log.terminated();
							return null;
						}
					}
				} catch (CoreException e) {
					Log.error("Error occured during generation process");
					Log.terminated();
					return null;
				}

				isProject = true;
			}

			if (isProject) {
				builder.exportDataModelToXMI(developerProject);
			}
		} else {
			Log.error("Selected item is not appropriate for generation");
			Log.terminated();
		}

		return null;
	}

	@Override
	public boolean isEnabled() {
		return true;
	}

}
