package hu.bme.mit.mobilegen.datamodel.generator.database.ormlite

import hu.bme.mit.mobilegen.datamodel.generator.GeneratorConstants
import hu.bme.mit.mobilegen.datamodel.generator.GeneratorData
import hu.bme.mit.mobilegen.datamodel.helper.WrappedEntity
import hu.bme.mit.mobilegen.datamodel.logger.Log
import java.util.ArrayList
import java.util.List
import org.eclipse.xtext.generator.IFileSystemAccess

class ORMLiteDataCacheGenerator {
	
	List<WrappedEntity> wes;
	List<String> imports;
	
	def generateDataCache(List<WrappedEntity> entities, IFileSystemAccess fsa){
		wes = new ArrayList<WrappedEntity>();
		for(WrappedEntity we : entities){
			if(we.entity.cache){
				wes.add(we);
			}
		}
		collectImports();
		Log.info("Generating DataCache.java");
		fsa.generateFile("DataCache.java", GeneratorConstants.SRC_GEN_DB, generateDataCache());
	}
	
	def generateGenericDAOInterface(IFileSystemAccess fsa){
		Log.info("GenericDAOInterface.java");
		fsa.generateFile("GenericDAOInterface.java", GeneratorConstants.SRC_GEN_DB, generateGenericDAOInterface);
	}
	
	def generateGenericDAO(IFileSystemAccess fsa){
		Log.info("GenericDAO.java");
		fsa.generateFile("GenericDAO.java", GeneratorConstants.SRC_GEN_DB, generateGenericDAO);
	}
	
	def private collectImports(){
		imports = new ArrayList<String>();
		
		for(WrappedEntity we : wes){
			imports.add(GeneratorData.packageName + ".entities." + we.entity.name.toFirstUpper);
		}
		
		imports.add("java.sql.SQLException");
		imports.add("java.util.ArrayList");
		imports.add("java.util.List");
		
	}
	
	def private generateDataCache() '''
		«classHeader»
		
		«packageDeclaration»
		
		«classImports»
		
		//
		«classDeclaration»
	'''
	
	def private classHeader() '''
		//------------------------- Geneated Java Class File For DataCache Class ---------------------------------
	'''
	
	def private packageDeclaration() '''
		package «GeneratorData.packageName».dbutility;
	'''
	
	def private classImports() '''
		«FOR f:imports»
			import «f»;
		«ENDFOR»
	'''
		
	def private classJavaDoc() '''
		/**
		 * <!-- begin-user-doc -->
		 * This Class contains the representation of the model object '<em><b></b></em>'.
		 * For each Entity the generator creates an EntityGEN.java file,
		 * which contains EntityGEN class with every Property and Association for the Entity.
		 * This Java file will be override, with each generation.
		 * The generator creates an Entity.java file, with the Entity class,
		 * which extends form EntityGEN. This file normally (if the Entity is not deleted)
		 * won't be overrided, so this file can be modified between the generations.
		 * 
		 * <p>
		 * This is The EntityGEN.java file. It must contain only the generated code.
		 * Do not edit it, or it will be overrided with each generation!
		 * </p>
		 * <!-- end-user-doc -->
		 *
		 * <p>
		 * The following features are supported:
		 * <ul>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getCache <em>Cache</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getPersistentType <em>Persistent Type</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#isAbstract <em>Abstract</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getName <em>Name</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getSuperType <em>Super Type</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getRelationship <em>Relationship</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getProperty <em>Property</em>}</li>
		 * </ul>
		 * </p>
		 *
		 * @see hu.bme.hellga.onlab.OnlabPackage#getEntity()
		 * @model
		 * @generated
		 */
	'''
	
	def private classDeclaration() '''
		public class DataCache {
			
			«classBody»
		}
	'''
	
	def private classBody() '''
		«properties»
		
		«initCache»
		
		«loadCache»
		
		«entities»
		
		«loadEntities»
		
	'''
	
	def private properties() '''
		«FOR f:wes»
			//
			private static List<«f.entity.name.toFirstUpper»> «f.entity.name.toFirstLower»s = new ArrayList<«f.entity.name.toFirstUpper»>();
		«ENDFOR»
	'''
	
	def private initCache() '''
		//
		public static void initCache(){
			«FOR f:wes»
				loadCache(«f.entity.name.toFirstUpper».class, «f.entity.name.toFirstLower»s);
			«ENDFOR»
		}
	'''
	
	def private loadCache() '''
		@SuppressWarnings("unchecked")
		private static <T> void loadCache(Class<T> entity, List<T> cache){
			try {
				List<T> list = DAOFactory.getGenericDAO(entity).queryForAll();
				for(Object o : list){
					cache.add((T)o);
				}
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}
	'''
	
	def private getEntities() '''
		«FOR f:wes»
			//
			public static List<«f.entity.name.toFirstUpper»> get«f.entity.name.toFirstUpper»s(){
				return «f.entity.name.toFirstLower»s;
			}
			
		«ENDFOR»
	'''
	
	def private loadEntities() '''
		«FOR f:wes»
			//
			public static void load«f.entity.name.toFirstUpper»s(){
				«f.entity.name.toFirstLower»s.clear();
				loadCache(«f.entity.name.toFirstUpper».class, «f.entity.name.toFirstLower»s);
			}
			
		«ENDFOR»
	'''
	
	def private generateGenericDAOInterface() '''
		package «GeneratorData.packageName».dbutility;
		
		import com.j256.ormlite.dao.Dao;

		public interface GenericDAOInterface {
			public <T> Dao<T, Integer> getGenericDAO(Class<T> t);
		}
	'''
	
	def private generateGenericDAO() '''
		package «GeneratorData.packageName».dbutility;
		
		import java.sql.SQLException;

		import com.j256.ormlite.dao.Dao;

		public class GenericDAO implements GenericDAOInterface{

			@Override
			public <T> Dao<T, Integer> getGenericDAO(Class<T> t) {
				try {
					return DatabaseFactory.getGenericDAO(t);
				} catch (SQLException e) {
					e.printStackTrace();
					return null;
				}
			}
			
		}
	'''
	
}