package hu.bme.mit.mobilegen.datamodel.generator.database.ormlite

import hu.bme.mit.mobilegen.datamodel.generator.GeneratorConstants
import hu.bme.mit.mobilegen.datamodel.generator.GeneratorData
import hu.bme.mit.mobilegen.datamodel.helper.WrappedEntity
import hu.bme.mit.mobilegen.datamodel.logger.Log
import java.util.ArrayList
import java.util.List
import org.eclipse.xtext.generator.IFileSystemAccess

class ORMLiteDatabaseFactoryGenerator {
	
	List<WrappedEntity> wes;
	List<String> imports;
	
	def generateDatabaseFactory(List<WrappedEntity> entities, IFileSystemAccess fsa){
		wes = entities;
		collectImports();
		Log.info("Generating DatabaseFactory.java");
		fsa.generateFile("DatabaseFactory.java", GeneratorConstants.SRC_GEN_DB, generateDatabaseFactory())
	}
	
	def private collectImports(){
		imports = new ArrayList<String>();
		
		for(WrappedEntity we : wes){
			imports.add(GeneratorData.packageName + ".entities." + we.entity.name.toFirstUpper);
		}
		
		imports.add("java.sql.SQLException");
		imports.add("android.content.Context");
		imports.add("com.j256.ormlite.android.apptools.OpenHelperManager");
		imports.add("com.j256.ormlite.dao.Dao");
	}
	
	def private generateDatabaseFactory() '''
		«classHeader»
		
		«packageDeclaration»
		
		«classImports»
		
		//
		«classDeclaration»
	'''
	
	def private classHeader() '''
		//------------------------- Geneated Java Class File For DatabaseFactory Class ---------------------------------
	'''
	
	def private packageDeclaration() '''
		package «GeneratorData.packageName».«GeneratorConstants.SRC_GEN_DB»;
	'''
	
	def private classImports() '''
		«FOR f:imports»
			import «f»;
		«ENDFOR»
	'''
		
	def private classJavaDoc() '''
		/**
		 * <!-- begin-user-doc -->
		 * This Class contains the representation of the model object '<em><b></b></em>'.
		 * For each Entity the generator creates an EntityGEN.java file,
		 * which contains EntityGEN class with every Property and Association for the Entity.
		 * This Java file will be override, with each generation.
		 * The generator creates an Entity.java file, with the Entity class,
		 * which extends form EntityGEN. This file normally (if the Entity is not deleted)
		 * won't be overrided, so this file can be modified between the generations.
		 * 
		 * <p>
		 * This is The EntityGEN.java file. It must contain only the generated code.
		 * Do not edit it, or it will be overrided with each generation!
		 * </p>
		 * <!-- end-user-doc -->
		 *
		 * <p>
		 * The following features are supported:
		 * <ul>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getCache <em>Cache</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getPersistentType <em>Persistent Type</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#isAbstract <em>Abstract</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getName <em>Name</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getSuperType <em>Super Type</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getRelationship <em>Relationship</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getProperty <em>Property</em>}</li>
		 * </ul>
		 * </p>
		 *
		 * @see hu.bme.hellga.onlab.OnlabPackage#getEntity()
		 * @model
		 * @generated
		 */
	'''
	
	def private classDeclaration() '''
		class DatabaseFactory {
			
			«classBody»
		}
	'''
	
	def private classBody() '''
		«properties»
		
		«initDB»
		
		«closeDB»
		
		«dbName»
		
		«dbVersion»
		
		«daoGetters»
		
		«genericDAO»
	'''
	
	def private properties() '''
		//
		private static DatabaseHelper dbHelper = null;
		
		«FOR f:wes»
			«f.daos»
		«ENDFOR»
	'''
	
	def private daos(WrappedEntity we) '''
		//
		«we.daoDeclaration»
	'''
	
	def private daoJavaDoc(WrappedEntity we) '''
		/**
		 * <p>
		 * JavaDoc
		 * </p>
		 * <!-- end-user-doc -->
		 *
		 * <p>
		 * The following features are supported:
		 *
		 * @see hu.bme.hellga.onlab.OnlabPackage#getEntity()
		 * @model
		 * @generated
		 */
	'''
	
	def private daoDeclaration(WrappedEntity we) '''
		private static Dao<«we.entity.name.toFirstUpper», Integer> «we.entity.name.toFirstLower»DAO = null;
		
	'''
	
	def private initDB() '''
		//
		static void initDatabase(Context c){
			if(dbHelper == null){
				dbHelper = OpenHelperManager.getHelper(c, DatabaseHelper.class);
			}
		};
	'''
	
	def private closeDB() '''
		//
		static void colseDatabase(){
			if(dbHelper != null){
				OpenHelperManager.releaseHelper();
				dbHelper = null;
			}
		}
	'''
	
	def private dbName() '''
		//
		static String getDBName(){
			return DatabaseHelper.DATABASE_NAME;
		}
	'''
	
	def private dbVersion() '''
		//
		static int getDBVersion(){
			return DatabaseHelper.DATABASE_VERSION;
		}
	'''
	
	def private daoGetters() '''
		«FOR f:wes»
			//
			«f.daoGetter»
			
		«ENDFOR»
	'''
	
	def private daoGetterJavaDoc(WrappedEntity we) '''
		/**
		 * <p>
		 * JavaDoc
		 * </p>
		 * <!-- end-user-doc -->
		 *
		 * <p>
		 * The following features are supported:
		 *
		 * @see hu.bme.hellga.onlab.OnlabPackage#getEntity()
		 * @model
		 * @generated
		 */
	'''
	
	def private daoGetter(WrappedEntity we) '''
		static Dao<«we.entity.name.toFirstUpper», Integer> get«we.entity.name.toFirstUpper»DAO() throws SQLException{
			if(«we.entity.name.toFirstLower»DAO == null){
				«we.entity.name.toFirstLower»DAO = dbHelper.getDao(«we.entity.name.toFirstUpper».class);
			}
			return «we.entity.name.toFirstLower»DAO;
		}
	'''
	
	def private genericDAO() '''
		«IF GeneratorData.cache»
			//
			static <T> Dao<T, Integer> getGenericDAO(Class<T> t) throws SQLException{
				return dbHelper.getDao(t);
			}
		«ENDIF»
	'''
	
}