package hu.bme.mit.mobilegen.datamodel.generator.database.ormlite

import hu.bme.mit.mobilegen.datamodel.generator.GeneratorConstants
import hu.bme.mit.mobilegen.datamodel.generator.GeneratorData
import hu.bme.mit.mobilegen.datamodel.helper.WrappedEntity
import hu.bme.mit.mobilegen.datamodel.logger.Log
import java.util.ArrayList
import java.util.List
import org.eclipse.xtext.generator.IFileSystemAccess

class ORMLiteDatabaseHelperGenerator {
	
	List<WrappedEntity> wes;
	List<String> imports;
	
	def generateDatabaseHelper(List<WrappedEntity> entities, IFileSystemAccess fsa){
		wes = entities;
		collectImports();
		Log.info("Generating DatabaseHelper.java");
		fsa.generateFile("DatabaseHelper.java", GeneratorConstants.SRC_GEN_DB, generateDatabaseHelper())
	}
	
	def private collectImports(){
		imports = new ArrayList<String>();
		
		for(WrappedEntity we : wes){
			imports.add(GeneratorData.packageName + ".entities." + we.entity.name.toFirstUpper);
		}
		
		imports.add("java.sql.SQLException");
		imports.add("android.content.Context");
		imports.add("android.database.sqlite.SQLiteDatabase");
		imports.add("com.j256.ormlite.android.apptools.OrmLiteSqliteOpenHelper");
		imports.add("com.j256.ormlite.support.ConnectionSource");
		imports.add("com.j256.ormlite.table.TableUtils");
	}
	
	def private generateDatabaseHelper() '''
		«classHeader»
		
		«packageDeclaration»
		
		«classImports»
		
		//
		«classDeclaration»
	'''
	
	def private classHeader() '''
		//------------------------- Geneated Java Class File For DatabaseHelper Class ---------------------------------
	'''
	
	def private packageDeclaration() '''
		package «GeneratorData.packageName».«GeneratorConstants.SRC_GEN_DB»;
	'''
	
	def private classImports() '''
		«FOR f:imports»
			import «f»;
		«ENDFOR»
	'''
		
	def private classJavaDoc() '''
		/**
		 * <!-- begin-user-doc -->
		 * This Class contains the representation of the model object '<em><b></b></em>'.
		 * For each Entity the generator creates an EntityGEN.java file,
		 * which contains EntityGEN class with every Property and Association for the Entity.
		 * This Java file will be override, with each generation.
		 * The generator creates an Entity.java file, with the Entity class,
		 * which extends form EntityGEN. This file normally (if the Entity is not deleted)
		 * won't be overrided, so this file can be modified between the generations.
		 * 
		 * <p>
		 * This is The EntityGEN.java file. It must contain only the generated code.
		 * Do not edit it, or it will be overrided with each generation!
		 * </p>
		 * <!-- end-user-doc -->
		 *
		 * <p>
		 * The following features are supported:
		 * <ul>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getCache <em>Cache</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getPersistentType <em>Persistent Type</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#isAbstract <em>Abstract</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getName <em>Name</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getSuperType <em>Super Type</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getRelationship <em>Relationship</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getProperty <em>Property</em>}</li>
		 * </ul>
		 * </p>
		 *
		 * @see hu.bme.hellga.onlab.OnlabPackage#getEntity()
		 * @model
		 * @generated
		 */
	'''
	
	def private classDeclaration() '''
		class DatabaseHelper extends OrmLiteSqliteOpenHelper {
			
			«classBody»
		}
	'''
	
	def private classBody() '''
		// Property declarations
		
		«properties»
		
		// Constructor
		
		«constructor»
		
		«onCreate»
		
		«onUpgrade»
		
		«close»
		
	'''
	
	def private properties() '''
		«databaseName»
		«databaseVersion»
	'''
	
	def private databaseName() '''
		//
		public static final String DATABASE_NAME = "«GeneratorData.getDBName»";
	'''
	
	def private databaseVersion() '''
		//
		public static final int DATABASE_VERSION = «GeneratorData.DBVersion»;
	'''
	
	def private constructor() '''
		//
		«constructorDeclaration»
	'''
	
	def private constructorJavaDoc() '''
		/**
		 * <p>
		 * JavaDoc
		 * </p>
		 * <!-- end-user-doc -->
		 *
		 * <p>
		 * The following features are supported:
		 *
		 * @see hu.bme.hellga.onlab.OnlabPackage#getEntity()
		 * @model
		 * @generated
		 */
	'''
	
	def private constructorDeclaration() '''
		public DatabaseHelper(Context context) {
			super(context, DATABASE_NAME, null, DATABASE_VERSION);
		}
	'''
	
	def private onCreate() '''
		//
		@Override
		public void onCreate(SQLiteDatabase db, ConnectionSource connectionSource) {
			try {
				«FOR f:wes»
					«f.createTable»
				«ENDFOR»
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}
	'''
	
	def private createTable(WrappedEntity we) '''
		TableUtils.createTable(connectionSource, «we.entity.name.toFirstUpper».class);
	'''
	
	def private onUpgrade() '''
		//
		@Override
		public void onUpgrade(SQLiteDatabase db, ConnectionSource connectionSource, int oldVersion, int newVersion) {
			try {
				«FOR f:wes»
					«f.dropTable»
				«ENDFOR»
				onCreate(db, connectionSource);
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}
	'''
	
	def private dropTable(WrappedEntity we) '''
		TableUtils.dropTable(connectionSource, «we.entity.name.toFirstUpper».class, true);
	'''
	
	def private close() '''
		//
		@Override
		public void close() {
			super.close();
		}
	'''
	
}