package hu.bme.mit.mobilegen.datamodel.generator.entity

import hu.bme.mit.mobilegen.datamodel.datamodel.Property
import hu.bme.mit.mobilegen.datamodel.generator.GeneratorConstants
import hu.bme.mit.mobilegen.datamodel.generator.GeneratorData
import hu.bme.mit.mobilegen.datamodel.helper.FormatPropertyType
import hu.bme.mit.mobilegen.datamodel.helper.GeneratorHelper
import hu.bme.mit.mobilegen.datamodel.helper.WrappedEntity
import hu.bme.mit.mobilegen.datamodel.logger.Log
import java.util.ArrayList
import java.util.HashMap
import java.util.List
import java.util.Map
import org.eclipse.xtext.generator.IFileSystemAccess

class EntityFactoryGenerator {
	
	List<WrappedEntity> wes;
	List<Property> rp;
	
	def generateEntityFactory(List<WrappedEntity> entities, IFileSystemAccess fsa){
		wes = new ArrayList<WrappedEntity>();
		for(WrappedEntity we : entities){
			if(!(we.genHelp == GeneratorHelper.JoinClass)){
				wes.add(we);
			}
		}
		Log.info("Generating EntityFactory.java");
		fsa.generateFile("EntityFactory.java", GeneratorConstants.SRC_GEN_ENTITIES, generateEntityFactory())
	}
	
	def private void setReqProps(WrappedEntity we){
		var Map<String, Property> reqProps = new HashMap<String, Property>();
		var WrappedEntity w = we;
		while(w != null){
			for(Property p : w.requiredProperties){
				if(!reqProps.containsKey(p.name)){
					reqProps.put(p.name, p);
				}
			}
			w = w.superClass;
		}
		rp = new ArrayList<Property>(reqProps.values());
	}
	
	def private generateEntityFactory() '''
		«classHeader»
		
		«packageDeclaration»
		
		«classImports»
		
		//
		«classDeclaration»
	'''
	
	def private classHeader() '''
		//------------------------- Geneated Java Class File For EntityFactory ---------------------------------
	'''
	
	def private packageDeclaration() '''
		package «GeneratorData.packageName».«GeneratorConstants.SRC_GEN_ENTITIES»;
	'''
	
	def private classImports() '''
		//import
	'''
	
	def private classJavaDoc() '''
		/**
		 * <!-- begin-user-doc -->
		 * This Class contains the representation of the model object '<em><b></b></em>'.
		 * For each Entity the generator creates an EntityGEN.java file,
		 * which contains EntityGEN class with every Property and Association for the Entity.
		 * This Java file will be override, with each generation.
		 * The generator creates an Entity.java file, with the Entity class,
		 * which extends form EntityGEN. This file normally (if the Entity is not deleted)
		 * won't be overrided, so this file can be modified between the generations.
		 * 
		 * <p>
		 * This is The Entity.java file. It is created at the first generation, after that,
		 * it can be modified, it won't be overrided, until the Entity is renamed, or the
		 * Entity is deleted.
		 * </p>
		 * <!-- end-user-doc -->
		 *
		 * <p>
		 * The following features are supported:
		 * <ul>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getCache <em>Cache</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getPersistentType <em>Persistent Type</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#isAbstract <em>Abstract</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getName <em>Name</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getSuperType <em>Super Type</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getRelationship <em>Relationship</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getProperty <em>Property</em>}</li>
		 * </ul>
		 * </p>
		 *
		 * @see hu.bme.hellga.onlab.OnlabPackage#getEntity()
		 * @model
		 * @generated
		 */
	'''
	
	def private classDeclaration() '''
		public class EntityFactory {
			
			«factories»
		}
	'''
	
	def private factories() '''
		«FOR f:wes»
			«setReqProps(f)»
			«IF !f.entity.abstract»
				//
				«f.factory»
				
				//
				«f.factoryCopy»
			«ENDIF»
		«ENDFOR»
	'''
	
	def private factoryJavaDoc(WrappedEntity we) '''
		/**
		 * <p>
		 *  FactoryMethod
		 * </p>
		 * <!-- end-user-doc -->
		 *
		 * @see hu.bme.hellga.onlab.OnlabPackage#getEntity()
		 * @model
		 * @generated
		 */
	'''
	
	def private factory(WrappedEntity we) '''
		public static «we.entity.name.toFirstUpper» create«we.entity.name.toFirstUpper»(«FOR f:rp SEPARATOR ', '»«IF f.enumeration != null»«f.enumeration.name.toFirstUpper»«ELSE»«FormatPropertyType.formatPrimitive(f.type)»«ENDIF» «f.name.toFirstLower»«ENDFOR»){
			«we.entity.name.toFirstUpper» «we.entity.name.substring(0,1).toFirstLower» = new «we.entity.name.toFirstUpper»();
			«FOR f:rp»
					«we.entity.name.substring(0,1).toFirstLower».set«f.name.toFirstUpper»(«f.name.toFirstLower»);
			«ENDFOR»
			return «we.entity.name.substring(0,1).toFirstLower»;
		}
	'''
	
	def private factoryCopy(WrappedEntity we) '''
		public static «we.entity.name.toFirstUpper» create«we.entity.name.toFirstUpper»(«we.entity.name.toFirstUpper» «we.entity.name.toFirstLower»){
			«we.entity.name.toFirstUpper» «we.entity.name.substring(0,1).toFirstLower» = new «we.entity.name.toFirstUpper»(«we.entity.name.toFirstLower»);
			return «we.entity.name.substring(0,1).toFirstLower»;
		}
	'''
	
}