package hu.bme.mit.mobilegen.datamodel.generator.entity

import com.google.inject.Inject
import hu.bme.mit.mobilegen.datamodel.datamodel.Association
import hu.bme.mit.mobilegen.datamodel.datamodel.ConnectionType
import hu.bme.mit.mobilegen.datamodel.datamodel.Entity
import hu.bme.mit.mobilegen.datamodel.datamodel.PersistentType
import hu.bme.mit.mobilegen.datamodel.datamodel.Property
import hu.bme.mit.mobilegen.datamodel.generator.GeneratorConstants
import hu.bme.mit.mobilegen.datamodel.generator.GeneratorData
import hu.bme.mit.mobilegen.datamodel.generator.database.IDatabaseGeneratorUtility
import hu.bme.mit.mobilegen.datamodel.helper.FormatPropertyType
import hu.bme.mit.mobilegen.datamodel.helper.WrappedEntity
import hu.bme.mit.mobilegen.datamodel.logger.Log
import java.util.ArrayList
import java.util.List
import org.eclipse.xtext.generator.IFileSystemAccess
import hu.bme.mit.mobilegen.datamodel.helper.GeneratorHelper
import hu.bme.mit.mobilegen.datamodel.datamodel.PropertyType

class EntityGENGenerator {
	
	@Inject
	IDatabaseGeneratorUtility dbGen;
	
	WrappedEntity wrappedEntity;
	Entity entity;
	List<String> imports;
	boolean only = false;
	
	def generateEntityGEN(WrappedEntity we, IFileSystemAccess fsa) {
		wrappedEntity = we;
		entity = we.entity;
		only = false;
		collectImports();
		if(entity.persistentType == PersistentType.DB){
			dbGen.collectEntityGENImports(imports, wrappedEntity);
		}
		Log.info("Generating " + entity.name.toFirstUpper + "GEN.java");
		fsa.generateFile(entity.name.toFirstUpper + "GEN.java",  GeneratorConstants.SRC_GEN_ENTITIES, entity.generateEntityGEN);
	}
	
	def generateEntityGENOnly(WrappedEntity we, IFileSystemAccess fsa) {
		wrappedEntity = we;
		entity = we.entity;
		only = true;
		collectImports();
		if(entity.persistentType == PersistentType.DB){
			dbGen.collectEntityGENImports(imports, wrappedEntity);
			dbGen.collectEntityImports(imports, wrappedEntity);
		}
		Log.info("Generating " + entity.name.toFirstUpper + ".java");
		fsa.generateFile(entity.name.toFirstUpper + ".java", GeneratorConstants.SRC_GEN_ENTITIES, entity.generateEntityGENOnly);
	}
	
	def private collectImports(){
		imports = new ArrayList<String>();
		var list = false;
		
		for(Property p : entity.property){
			if(p.connectionType == ConnectionType.MANY){
				list = true;
			}
		}
		
		for(Association a : entity.association){
			if(a.connectionType == ConnectionType.MANY && (!entity.persistentType.equals(PersistentType.DB) || a.static || a.transient)){
				list = true;
			}
		}
		
		if(list){
			imports.add("java.util.List");
		}
		
		var date = false;
		
		for(Property p : entity.property){
			if(p.type == PropertyType.DATE){
				date = true;
			}
		}
		
		if(date){
			imports.add("java.util.Date");
		}
		
		if(entity.persistentType == PersistentType.SERIALIZE){
			imports.add("java.io.Serializable");
		}
		
		imports.add("android.os.Parcel");
		imports.add("android.os.Parcelable");
	}
	
	def private generateEntityGEN(Entity e) '''
		«e.classHeader»
		
		«packageDeclaration»
		
		«e.classImports»
		
		//
		«e.classDeclaration»
	'''
	
	def private generateEntityGENOnly(Entity e) '''
		«e.classHeader»
		
		«packageDeclaration»
		
		«e.classImports»
		
		//
		«e.classAnnotations»
		«e.classDeclarationOnlyGEN»
	'''
	
	def private classHeader(Entity e) '''
		//------------------------- Geneated Java Class File For Entity - «e.name» ---------------------------------
		//------------------------- Model Implementation In «wrappedEntity.resourceURI» Resource File --------------
	'''
	
	def private packageDeclaration() '''
		package «GeneratorData.packageName».«GeneratorConstants.SRC_GEN_ENTITIES»;
	'''
	
	def private classImports(Entity e) '''
		«FOR f:imports»
			import «f»;
		«ENDFOR»
	'''
		
	def private classJavaDoc(Entity e) '''
		/**
		 * <!-- begin-user-doc -->
		 * This Class contains the representation of the model object '<em><b«e.name»</b></em>'.
		 * For each Entity the generator creates an EntityGEN.java file,
		 * which contains EntityGEN class with every Property and Association for the Entity.
		 * This Java file will be override, with each generation.
		 * The generator creates an Entity.java file, with the Entity class,
		 * which extends form EntityGEN. This file normally (if the Entity is not deleted)
		 * won't be overrided, so this file can be modified between the generations.
		 * 
		 * <p>
		 * This is The EntityGEN.java file. It must contain only the generated code.
		 * Do not edit it, or it will be overrided with each generation!
		 * </p>
		 * <!-- end-user-doc -->
		 *
		 * <p>
		 * The following features are supported:
		 * <ul>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getCache <em>Cache</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getPersistentType <em>Persistent Type</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#isAbstract <em>Abstract</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getName <em>Name</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getSuperType <em>Super Type</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getRelationship <em>Relationship</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getProperty <em>Property</em>}</li>
		 * </ul>
		 * </p>
		 *
		 * @see hu.bme.hellga.onlab.OnlabPackage#getEntity()
		 * @model
		 * @generated
		 */
	'''
	
	def private classAnnotations(Entity e) '''
		«IF entity.persistentType.equals(PersistentType.DB)»
			«dbGen.classAnnotations(e)»
		«ENDIF»
	'''
	
	def private classDeclaration(Entity e) '''
		«IF e.abstract»abstract «ENDIF»class «e.name.toFirstUpper»GEN «IF e.superClass != null»extends «e.superClass.name.toFirstUpper»GEN «ENDIF»implements Parcelable«IF e.persistentType == PersistentType.SERIALIZE», Serializable«ENDIF» {
			
			«e.classBody»
		}
	'''
	
	def private classDeclarationOnlyGEN(Entity e) '''
		public «IF e.abstract»abstract «ENDIF»class «e.name.toFirstUpper» «IF e.superClass != null»extends «e.superClass.name.toFirstUpper»«IF !only»GEN «ENDIF»«ENDIF»implements Parcelable«IF e.persistentType == PersistentType.SERIALIZE», Serializable«ENDIF» {
			
			«e.classBodyOnlyGEN»
		}
	'''
	
	def private classBody(Entity e) '''
		// Property declarations
		
		«e.properties»
		
		// Association declarations
		
		«e.associations»
		
		// Parcelable creator
		
		«e.parcelableCreator»
		
		// Constructor
		
		«e.constructor»
		
		// Parcel methods
		
		«e.parcelMethods»
		
		// Getter-setters
		
		«e.getterSetters»
	'''
	
	def private classBodyOnlyGEN(Entity e) '''
		// Property declarations
		
		«e.properties»
		
		// Association declarations
		
		«e.associations»
		
		// Parcelable creator
		
		«e.parcelableCreator»
		
		// Constructor
		
		«e.constructorOnlyGEN»
		
		// Parcel methods
		
		«e.parcelMethods»
		
		// Getter-setters
		
		«e.getterSetters»
	'''
	
	def private properties(Entity e) '''
		«IF e.persistentType.equals(PersistentType.SERIALIZE)»
			«serializeProperty»
			
		«ENDIF»
		«IF e.persistentType.equals(PersistentType.DB)»
			«dbGen.fieldNames(e)»
			«dbGen.propertyID(e)»
			
		«ENDIF»
		«FOR f:e.property»
			«f.property»
		«ENDFOR»
	'''
	
	def private serializeProperty() '''
		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;
	'''
	
	def private property(Property p) '''
		//
		«p.propertyAnnotations»
		«p.propertyDeclaration»
		
	'''
	
	def private propertyJavaDoc(Property p) '''
		«IF p.connectionType.equals(ConnectionType.ONE)»
			«p.simplePropertyJavaDoc»
		«ELSE»
			«p.listPropertyJavaDoc»
		«ENDIF»
	'''
	
	def private simplePropertyJavaDoc(Property p) '''
		/**
		 * <p>
		 * «p.name» sproperty of «entity.name» entitiy.
		 * </p>
		 * <!-- end-user-doc -->
		 *
		 * <p>
		 * The following features are supported:
		 *
		 * @see hu.bme.hellga.onlab.OnlabPackage#getEntity()
		 * @model
		 * @generated
		 */
	'''
	
	def private listPropertyJavaDoc(Property p) '''
		/**
		 * <p>
		 * «p.name» lproperty of «entity.name» entitiy.
		 * </p>
		 * <!-- end-user-doc -->
		 *
		 * <p>
		 * The following features are supported:
		 *
		 * @see hu.bme.hellga.onlab.OnlabPackage#getEntity()
		 * @model
		 * @generated
		 */
	'''
	
	def private propertyAnnotations(Property p) '''
		«IF entity.persistentType.equals(PersistentType.DB)»
			«dbGen.propertyAnnotations(p)»
		«ENDIF»
	'''
	
	def private propertyDeclaration(Property p) '''
		«IF p.connectionType.equals(ConnectionType.ONE)»
			«p.simpleProperty»
		«ELSE»
			«p.listProperty»
		«ENDIF»
	'''
	
	def private simpleProperty(Property p) '''
		protected «IF p.transient»transient «ENDIF»«IF p.static»static «ENDIF»«IF p.enumeration != null»«p.enumeration.name.toFirstUpper»«ELSE»«FormatPropertyType.formatPrimitive(p.type)»«ENDIF» «p.name.toFirstLower»;
	'''
	
	def private listProperty(Property p) '''
		protected «IF p.transient»transient «ENDIF»«IF p.static»static «ENDIF»List<«IF p.enumeration != null»«p.enumeration.name.toFirstUpper»«ELSE»«FormatPropertyType.formatClass(p.type)»«ENDIF»> «p.name.toFirstLower»;
	'''
	
	def private associations(Entity e) '''
		«FOR f:e.association»
			«f.association»
		«ENDFOR»
	'''
	
	def private association(Association a) '''
		//
		«a.associationAnnotations»
		«a.associationDeclaration»
		
	'''
	
	def private associationJavaDoc(Association a) '''
		«IF a.connectionType.equals(ConnectionType.ONE)»
			«a.simpleAssociationJavaDoc»
		«ELSE»
			«a.listAssociationJavaDoc»
		«ENDIF»
	'''
	
	def private simpleAssociationJavaDoc(Association a) '''
		/**
		 * <p>
		 * «a.name» sassociation of «entity.name» entitiy.
		 * </p>
		 * <!-- end-user-doc -->
		 *
		 * <p>
		 * The following features are supported:
		 *
		 * @see hu.bme.hellga.onlab.OnlabPackage#getEntity()
		 * @model
		 * @generated
		 */
	'''
	
	def private listAssociationJavaDoc(Association a) '''
		/**
		 * <p>
		 * «a.name» lassociation of «entity.name» entitiy.
		 * </p>
		 * <!-- end-user-doc -->
		 *
		 * <p>
		 * The following features are supported:
		 *
		 * @see hu.bme.hellga.onlab.OnlabPackage#getEntity()
		 * @model
		 * @generated
		 */
	'''
	
	def private associationAnnotations(Association a) '''
		«IF entity.persistentType.equals(PersistentType.DB)»
			«dbGen.associationAnnotations(a)»
		«ENDIF»
	'''
	
	def private associationDeclaration(Association a) '''
		«IF a.connectionType.equals(ConnectionType.ONE)»
			«a.simpleAssociation»
		«ELSE»
			«IF entity.persistentType.equals(PersistentType.DB) && !a.transient && !a.static»
				«dbGen.associationListDeclaration(a)»
			«ELSE»
				«a.listAssociation»
			«ENDIF»
		«ENDIF»
	'''
	
	def private simpleAssociation(Association a) '''
		protected «IF a.transient»transient «ENDIF»«IF a.static»static «ENDIF»«a.connectTo.name» «a.name.toFirstLower»;
	'''
	
	def private listAssociation(Association a) '''
		protected «IF a.transient»transient «ENDIF»«IF a.static»static «ENDIF»List<«a.connectTo.name»> «a.name.toFirstLower»;
	'''
	
	def private parcelableCreator(Entity e) '''
		//
		public static final Parcelable.Creator<«e.name.toFirstUpper»> CREATOR = new Parcelable.Creator<«e.name.toFirstUpper»>() {
			public «e.name.toFirstUpper» createFromParcel(Parcel pc) {
				return new «e.name.toFirstUpper»(pc);
			}
			public «e.name.toFirstUpper»[] newArray(int size) {
				return new «e.name.toFirstUpper»[size];
			}
		};
	'''
	
	def private constructor(Entity e) '''
		//
		«e.emptyConstructor»
		
		«e.copyConstructor»
		
		«e.parcelConstructor»
		
	'''
	
	def private constructorOnlyGEN(Entity e) '''
		//
		«e.emptyConstructorOnlyGEN»
		
		«e.copyConstructorOnlyGEN»
		
		«e.parcelConstructorOnlyGEN»
		
	'''
	
	def private emptyConstructorJavaDoc(Entity e)'''
		/**
		 * <p>
		 * Empty constructor of «e.name» entity class.
		 * </p>
		 * <!-- end-user-doc -->
		 *
		 * @see hu.bme.hellga.onlab.OnlabPackage#getEntity()
		 * @model
		 * @generated
		 */
	'''
	
	def private emptyConstructor(Entity e) '''
		«IF wrappedEntity.genHelp.equals(GeneratorHelper.JoinClass)»public «ELSE»protected «ENDIF»«e.name.toFirstUpper»GEN(){
			super();
		}
	'''
	
	def private emptyConstructorOnlyGEN(Entity e) '''
		«IF wrappedEntity.genHelp.equals(GeneratorHelper.JoinClass)»public «ELSE»protected «ENDIF»«e.name.toFirstUpper»(){
			super();
		}
	'''
	
	def private copyConstructor(Entity e) '''
		«IF wrappedEntity.genHelp.equals(GeneratorHelper.JoinClass)»public «ELSE»protected «ENDIF» «e.name.toFirstUpper»GEN(«e.name.toFirstUpper» «e.name.toFirstLower»){
			super();
		}
	'''
	
	def private copyConstructorOnlyGEN(Entity e) '''
		«IF wrappedEntity.genHelp.equals(GeneratorHelper.JoinClass)»public «ELSE»protected «ENDIF»«e.name.toFirstUpper»(«e.name.toFirstUpper» «e.name.toFirstLower»){
			super();
		}
	'''
	
	def private parcelConstructor(Entity e) '''
		public «e.name.toFirstUpper»GEN(Parcel pc){
			super();
			
			«FOR f:e.property»
				«f.parcelProperty»
			«ENDFOR»
			«FOR f:e.association»
				«f.parcelAssociation»
			«ENDFOR»
		}
	'''
	
	def private parcelConstructorOnlyGEN(Entity e) '''
		public «e.name.toFirstUpper»(Parcel pc){
			super();
			
			«FOR f:e.property»
				«f.parcelPropertyBack»
			«ENDFOR»
			«FOR f:e.association»
				«f.parcelAssociationBack»
			«ENDFOR»
		}
	'''
	
	def private parcelPropertyBack(Property p) '''
		«IF p.enumeration != null»«p.name.toFirstLower» = («p.enumeration.name.toFirstUpper») pc.readValue(«p.enumeration.name.toFirstUpper».class.getClassLoader());«ELSEIF p.type.equals(PropertyType.DATE)»«p.name.toFirstLower» = («FormatPropertyType.formatClass(p.type)») pc.readValue(«FormatPropertyType.formatClass(p.type)».class.getClassLoader());«ELSEIF p.type.equals(PropertyType.STRING)»«p.name.toFirstLower» = pc.readString();«ELSEIF p.type.equals(PropertyType.INTEGER)»«p.name.toFirstLower» = pc.readInt();«ENDIF»
	'''
	
	def private parcelAssociationBack(Association a) '''
		«a.name.toFirstLower» = («a.name.toFirstUpper») pc.readValue(«a.name.toFirstUpper».class.getClassLoader());
	'''
	
	def private parcelMethods(Entity e) '''
		
		«parcelDesribeContents»
		
		«e.parcelWriteToLocalParcel»
		
	'''
	
	def private parcelDesribeContents() '''
		@Override
		public int describeContents() {
			return 0;
		}
	'''
	
	def private parcelWriteToLocalParcel(Entity e) '''
		@Override
		public void writeToParcel(Parcel dest, int flags) {
			«FOR f:e.property»
				«f.parcelProperty»
			«ENDFOR»
			«FOR f:e.association»
				«f.parcelAssociation»
			«ENDFOR»
		}
	'''
	
	def private parcelProperty(Property p) '''
		«IF p.enumeration != null || p.type.equals(PropertyType.DATE)»dest.writeValue(«p.name.toFirstLower»);«ELSEIF p.type.equals(PropertyType.STRING)»dest.writeString(«p.name.toFirstLower»);«ELSEIF p.type.equals(PropertyType.INTEGER)»dest.writeInt(«p.name.toFirstLower»);«ENDIF»
	'''
	
	def private parcelAssociation(Association a) '''
		dest.writeValue(«a.name.toFirstLower»);
	'''
	
	def private getterSetters(Entity e)'''
		«IF e.persistentType.equals(PersistentType.DB)»
			«dbGen.idGetter(e)»
			
		«ENDIF»
		«e.propertyGettersSetters»
		«e.associationGettersSetters»
	'''
	
	def private propertyGettersSetters(Entity e) '''
		«FOR f:e.property»
			//
			«f.propertyGetter»
			
			//
			«f.propertySetter»
			
		«ENDFOR»
	'''
	
	def private propertyGetterJavaDoc(Property p) '''
		«IF p.connectionType.equals(ConnectionType.ONE)»
			«p.simplePropertyGetterJavaDoc»
		«ELSE»
			«p.listPropertyGetterJavaDoc»
		«ENDIF»
	'''
	
	def private simplePropertyGetterJavaDoc(Property p) '''
		/**
		 * <p>
		 * «p.name» sproperty of «entity.name» entitiy.
		 * </p>
		 * <!-- end-user-doc -->
		 *
		 * <p>
		 * The following features are supported:
		 *
		 * @see hu.bme.hellga.onlab.OnlabPackage#getEntity()
		 * @model
		 * @generated
		 */
	'''
	
	def private listPropertyGetterJavaDoc(Property p) '''
		/**
		 * <p>
		 * «p.name» lproperty of «entity.name» entitiy.
		 * </p>
		 * <!-- end-user-doc -->
		 *
		 * <p>
		 * The following features are supported:
		 *
		 * @see hu.bme.hellga.onlab.OnlabPackage#getEntity()
		 * @model
		 * @generated
		 */
	'''
	
	def private propertyGetter(Property p)'''
		«IF p.connectionType.equals(ConnectionType.ONE)»
			«p.simplePropertyGetter»
		«ELSE»
			«p.listPropertyGetter»
		«ENDIF»
	'''
	
	def private simplePropertyGetter(Property p) '''
		public «IF p.static»static «ENDIF»«IF p.enumeration != null»«p.enumeration.name.toFirstUpper»«ELSE»«FormatPropertyType.formatPrimitive(p.type)»«ENDIF» get«p.name.toFirstUpper»(){
			return «IF !p.static»this.«ENDIF»«p.name.toFirstLower»;
		}
	'''
	
	def private listPropertyGetter(Property p) '''
		public «IF p.static»static «ENDIF»List<«IF p.enumeration != null»«p.enumeration.name.toFirstUpper»«ELSE»«FormatPropertyType.formatClass(p.type)»«ENDIF»> get«p.name.toFirstUpper»(){
			return «IF !p.static»this.«ENDIF»«p.name.toFirstLower»;
		}
	'''
	
	def private propertySetterJavaDoc(Property p) '''
		«IF p.connectionType.equals(ConnectionType.ONE)»
			«p.simplePropertySetterJavaDoc»
		«ELSE»
			«p.listPropertySetterJavaDoc»
		«ENDIF»
	'''
	
	def private simplePropertySetterJavaDoc(Property p) '''
		/**
		 * <p>
		 * «p.name» sproperty of «entity.name» entitiy.
		 * </p>
		 * <!-- end-user-doc -->
		 *
		 * <p>
		 * The following features are supported:
		 *
		 * @see hu.bme.hellga.onlab.OnlabPackage#getEntity()
		 * @model
		 * @generated
		 */
	'''
	
	def private listPropertySetterJavaDoc(Property p) '''
		/**
		 * <p>
		 * «p.name» lproperty of «entity.name» entitiy.
		 * </p>
		 * <!-- end-user-doc -->
		 *
		 * <p>
		 * The following features are supported:
		 *
		 * @see hu.bme.hellga.onlab.OnlabPackage#getEntity()
		 * @model
		 * @generated
		 */
	'''
	
	def private propertySetter(Property p)'''
		«IF p.connectionType.equals(ConnectionType.ONE)»
			«p.simplePropertySetter»
		«ELSE»
			«p.listPropertySetter»
		«ENDIF»
	'''
	
	def private simplePropertySetter(Property p) '''
		public «IF p.static»static «ENDIF»void set«p.name.toFirstUpper»(«IF p.enumeration != null»«p.enumeration.name.toFirstUpper»«ELSE»«FormatPropertyType.formatPrimitive(p.type)»«ENDIF» «p.name.toFirstLower»){
			«IF !p.static»this.«ELSE»«entity.name.toFirstUpper»«IF !only»GEN«ENDIF».«ENDIF»«p.name.toFirstLower» = «p.name.toFirstLower»;
		}
	'''
	
	def private listPropertySetter(Property p) '''
		public «IF p.static»static «ENDIF»void set«p.name.toFirstUpper»(List<«IF p.enumeration != null»«p.enumeration.name.toFirstUpper»«ELSE»«FormatPropertyType.formatClass(p.type)»«ENDIF»> «p.name.toFirstLower»){
			«IF !p.static»this.«ELSE»«entity.name.toFirstUpper»«IF !only»GEN«ENDIF».«ENDIF»«p.name.toFirstLower» = «p.name.toFirstLower»;
		}
	'''
	
	def private associationGettersSetters(Entity e) '''
		«FOR f:e.association»
			//
			«f.associationGetter»
			
			//
			«f.associationSetter»
			
		«ENDFOR»
	'''
	
	def private associationGetterJavaDoc(Association a) '''
		«IF a.connectionType.equals(ConnectionType.ONE)»
			«a.simpleAssociationGetterJavaDoc»
		«ELSE»
			«a.listAssociationGetterJavaDoc»
		«ENDIF»
	'''
	
	def private simpleAssociationGetterJavaDoc(Association a) '''
		/**
		 * <p>
		 * «a.name» sassociation of «entity.name» entitiy.
		 * </p>
		 * <!-- end-user-doc -->
		 *
		 * <p>
		 * The following features are supported:
		 *
		 * @see hu.bme.hellga.onlab.OnlabPackage#getEntity()
		 * @model
		 * @generated
		 */
	'''
	
	def private listAssociationGetterJavaDoc(Association a) '''
		/**
		 * <p>
		 * «a.name» lassociation of «entity.name» entitiy.
		 * </p>
		 * <!-- end-user-doc -->
		 *
		 * <p>
		 * The following features are supported:
		 *
		 * @see hu.bme.hellga.onlab.OnlabPackage#getEntity()
		 * @model
		 * @generated
		 */
	'''
	
	def private associationGetter(Association a)'''
		«IF a.connectionType.equals(ConnectionType.ONE)»
			«a.simpleAssociationGetter»
		«ELSE»
			«IF entity.persistentType.equals(PersistentType.DB) && !a.static && !a.transient»
				«dbGen.associationListGetter(a)»
			«ELSE»
				«a.listAssociationGetter»
			«ENDIF»
		«ENDIF»
	'''
	
	def private simpleAssociationGetter(Association a) '''
		public «IF a.static»static «ENDIF»«a.connectTo.name» get«a.name.toFirstUpper»(){
			return «IF !a.static»this.«ENDIF»«a.name.toFirstLower»;
		}
	'''
	
	def private listAssociationGetter(Association a) '''
		public «IF a.static»static «ENDIF»List<«a.connectTo.name»> get«a.name.toFirstUpper»(){
			return «IF !a.static»this.«ENDIF»«a.name.toFirstLower»;
		}
	'''
	
	def private associationSetterJavaDoc(Association a) '''
		«IF a.connectionType.equals(ConnectionType.ONE)»
			«a.simpleAssociationSetterJavaDoc»
		«ELSE»
			«a.listAssociationSetterJavaDoc»
		«ENDIF»
	'''
	
	def private simpleAssociationSetterJavaDoc(Association a) '''
		/**
		 * <p>
		 * «a.name» sassociation of «entity.name» entitiy.
		 * </p>
		 * <!-- end-user-doc -->
		 *
		 * <p>
		 * The following features are supported:
		 *
		 * @see hu.bme.hellga.onlab.OnlabPackage#getEntity()
		 * @model
		 * @generated
		 */
	'''
	
	def private listAssociationSetterJavaDoc(Association a) '''
		/**
		 * <p>
		 * «a.name» lassociation of «entity.name» entitiy.
		 * </p>
		 * <!-- end-user-doc -->
		 *
		 * <p>
		 * The following features are supported:
		 *
		 * @see hu.bme.hellga.onlab.OnlabPackage#getEntity()
		 * @model
		 * @generated
		 */
	'''
	
	def private associationSetter(Association a)'''
		«IF a.connectionType.equals(ConnectionType.ONE)»
			«a.simpleAssociationSetter»
		«ELSE»
			«IF entity.persistentType.equals(PersistentType.DB) && !a.static && !a.transient»
				«dbGen.associationListSetter(a)»
			«ELSE»
				«a.listAssociationSetter»
			«ENDIF»
		«ENDIF»
	'''
	
	def private simpleAssociationSetter(Association a) '''
		public «IF a.static»static «ENDIF»void set«a.name.toFirstUpper»(«a.connectTo.name» «a.name.toFirstLower»){
			«IF !a.static»this.«ELSE»«entity.name.toFirstUpper»«IF !only»GEN«ENDIF».«ENDIF»«a.name.toFirstLower» = «a.name.toFirstLower»;
		}
	'''
	
	def private listAssociationSetter(Association a) '''
		public «IF a.static»static «ENDIF»void set«a.name.toFirstUpper»(List<«a.connectTo.name»> «a.name.toFirstLower»){
			«IF !a.static»this.«ELSE»«entity.name.toFirstUpper»«IF !only»GEN«ENDIF».«ENDIF»«a.name.toFirstLower» = «a.name.toFirstLower»;
		}
	'''
	
}