package hu.bme.mit.mobilegen.datamodel.serializer;

import com.google.inject.Inject;
import com.google.inject.Provider;
import hu.bme.mit.mobilegen.datamodel.datamodel.Association;
import hu.bme.mit.mobilegen.datamodel.datamodel.Content;
import hu.bme.mit.mobilegen.datamodel.datamodel.Datamodel;
import hu.bme.mit.mobilegen.datamodel.datamodel.DatamodelPackage;
import hu.bme.mit.mobilegen.datamodel.datamodel.Entity;
import hu.bme.mit.mobilegen.datamodel.datamodel.Enumeration;
import hu.bme.mit.mobilegen.datamodel.datamodel.Import;
import hu.bme.mit.mobilegen.datamodel.datamodel.Property;
import hu.bme.mit.mobilegen.datamodel.services.DatamodelGrammarAccess;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.xtext.serializer.acceptor.ISemanticSequenceAcceptor;
import org.eclipse.xtext.serializer.acceptor.SequenceFeeder;
import org.eclipse.xtext.serializer.diagnostic.ISemanticSequencerDiagnosticProvider;
import org.eclipse.xtext.serializer.diagnostic.ISerializationDiagnostic.Acceptor;
import org.eclipse.xtext.serializer.sequencer.AbstractDelegatingSemanticSequencer;
import org.eclipse.xtext.serializer.sequencer.GenericSequencer;
import org.eclipse.xtext.serializer.sequencer.ISemanticNodeProvider.INodesForEObjectProvider;
import org.eclipse.xtext.serializer.sequencer.ISemanticSequencer;
import org.eclipse.xtext.serializer.sequencer.ITransientValueService;
import org.eclipse.xtext.serializer.sequencer.ITransientValueService.ValueTransient;

@SuppressWarnings("all")
public class DatamodelSemanticSequencer extends AbstractDelegatingSemanticSequencer {

	@Inject
	private DatamodelGrammarAccess grammarAccess;
	
	public void createSequence(EObject context, EObject semanticObject) {
		if(semanticObject.eClass().getEPackage() == DatamodelPackage.eINSTANCE) switch(semanticObject.eClass().getClassifierID()) {
			case DatamodelPackage.ASSOCIATION:
				if(context == grammarAccess.getAssociationRule()) {
					sequence_Association(context, (Association) semanticObject); 
					return; 
				}
				else break;
			case DatamodelPackage.CONTENT:
				if(context == grammarAccess.getContentRule()) {
					sequence_Content(context, (Content) semanticObject); 
					return; 
				}
				else break;
			case DatamodelPackage.DATAMODEL:
				if(context == grammarAccess.getDatamodelRule()) {
					sequence_Datamodel(context, (Datamodel) semanticObject); 
					return; 
				}
				else break;
			case DatamodelPackage.ENTITY:
				if(context == grammarAccess.getElementRule() ||
				   context == grammarAccess.getEntityRule()) {
					sequence_Entity(context, (Entity) semanticObject); 
					return; 
				}
				else break;
			case DatamodelPackage.ENUMERATION:
				if(context == grammarAccess.getElementRule() ||
				   context == grammarAccess.getEnumerationRule()) {
					sequence_Enumeration(context, (Enumeration) semanticObject); 
					return; 
				}
				else break;
			case DatamodelPackage.IMPORT:
				if(context == grammarAccess.getImportRule()) {
					sequence_Import(context, (Import) semanticObject); 
					return; 
				}
				else break;
			case DatamodelPackage.PROPERTY:
				if(context == grammarAccess.getPropertyRule()) {
					sequence_Property(context, (Property) semanticObject); 
					return; 
				}
				else break;
			}
		if (errorAcceptor != null) errorAcceptor.accept(diagnosticProvider.createInvalidContextOrTypeDiagnostic(semanticObject, context));
	}
	
	/**
	 * Constraint:
	 *     (
	 *         static?='@Static'? 
	 *         transient?='@Transient'? 
	 *         cantBeNull?='@CantBeNull'? 
	 *         (directionSet?='@Direction: ' connectionDir=ConnectionDirection)? 
	 *         (inverseNameSet?='@InverseReferenceName: ' inverseName=STRING)? 
	 *         containment?='@Containment'? 
	 *         connectionType=ConnectionType 
	 *         connectTo=[Entity|QualifiedName] 
	 *         name=STRING
	 *     )
	 */
	protected void sequence_Association(EObject context, Association semanticObject) {
		genericSequencer.createSequence(context, semanticObject);
	}
	
	
	/**
	 * Constraint:
	 *     (name=QualifiedName imports+=Import* element+=Element*)
	 */
	protected void sequence_Content(EObject context, Content semanticObject) {
		genericSequencer.createSequence(context, semanticObject);
	}
	
	
	/**
	 * Constraint:
	 *     content=Content
	 */
	protected void sequence_Datamodel(EObject context, Datamodel semanticObject) {
		if(errorAcceptor != null) {
			if(transientValues.isValueTransient(semanticObject, DatamodelPackage.Literals.DATAMODEL__CONTENT) == ValueTransient.YES)
				errorAcceptor.accept(diagnosticProvider.createFeatureValueMissing(semanticObject, DatamodelPackage.Literals.DATAMODEL__CONTENT));
		}
		INodesForEObjectProvider nodes = createNodeProvider(semanticObject);
		SequenceFeeder feeder = createSequencerFeeder(semanticObject, nodes);
		feeder.accept(grammarAccess.getDatamodelAccess().getContentContentParserRuleCall_0(), semanticObject.getContent());
		feeder.finish();
	}
	
	
	/**
	 * Constraint:
	 *     (
	 *         onlyInContainment?='@OnlyInContainment'? 
	 *         cache?='@Cache'? 
	 *         persistentType=PersistentType? 
	 *         abstract?='abstract'? 
	 *         name=ID 
	 *         (extends?='extends' superClass=[Entity|QualifiedName])? 
	 *         (property+=Property property+=Property*)? 
	 *         (association+=Association association+=Association*)?
	 *     )
	 */
	protected void sequence_Entity(EObject context, Entity semanticObject) {
		genericSequencer.createSequence(context, semanticObject);
	}
	
	
	/**
	 * Constraint:
	 *     (name=ID enumeration+=STRING enumeration+=STRING*)
	 */
	protected void sequence_Enumeration(EObject context, Enumeration semanticObject) {
		genericSequencer.createSequence(context, semanticObject);
	}
	
	
	/**
	 * Constraint:
	 *     importedNamespace=QualifiedNameWithWildcard
	 */
	protected void sequence_Import(EObject context, Import semanticObject) {
		if(errorAcceptor != null) {
			if(transientValues.isValueTransient(semanticObject, DatamodelPackage.Literals.IMPORT__IMPORTED_NAMESPACE) == ValueTransient.YES)
				errorAcceptor.accept(diagnosticProvider.createFeatureValueMissing(semanticObject, DatamodelPackage.Literals.IMPORT__IMPORTED_NAMESPACE));
		}
		INodesForEObjectProvider nodes = createNodeProvider(semanticObject);
		SequenceFeeder feeder = createSequencerFeeder(semanticObject, nodes);
		feeder.accept(grammarAccess.getImportAccess().getImportedNamespaceQualifiedNameWithWildcardParserRuleCall_1_0(), semanticObject.getImportedNamespace());
		feeder.finish();
	}
	
	
	/**
	 * Constraint:
	 *     (
	 *         static?='@Static'? 
	 *         transient?='@Transient'? 
	 *         cantBeNull?='@CantBeNull'? 
	 *         unique?='@Unique'? 
	 *         indexed?='@Index'? 
	 *         connectionType=ConnectionType 
	 *         (enumeration=[Enumeration|QualifiedName] | type=PropertyType) 
	 *         name=STRING
	 *     )
	 */
	protected void sequence_Property(EObject context, Property semanticObject) {
		genericSequencer.createSequence(context, semanticObject);
	}
}
