package hu.bme.mit.mobilegen.datamodel.generator.database.ormlite

import hu.bme.mit.mobilegen.datamodel.generator.GeneratorConstants
import hu.bme.mit.mobilegen.datamodel.generator.GeneratorData
import hu.bme.mit.mobilegen.datamodel.helper.GeneratorHelper
import hu.bme.mit.mobilegen.datamodel.helper.WrappedEntity
import hu.bme.mit.mobilegen.datamodel.logger.Log
import java.util.ArrayList
import java.util.List
import org.eclipse.xtext.generator.IFileSystemAccess

class ORMLiteDAOFactoryGenerator {
	
	List<WrappedEntity> wes;
	List<String> imports;
	
	def generateDAOFactory(List<WrappedEntity> entities, IFileSystemAccess fsa){
		wes = entities;
		collectImports();
		Log.info("Generating DAOFactory.java");
		fsa.generateFile("DAOFactory.java", GeneratorConstants.SRC_GEN_DB, generateDAOFactory());
	}
	
	def private collectImports(){
		imports = new ArrayList<String>();
		
		imports.add("android.content.Context");
		
		if(GeneratorData.cache){
			imports.add("com.j256.ormlite.dao.Dao");
		}
		
	}
	
	def private generateDAOFactory() '''
		«classHeader»
		
		«packageDeclaration»
		
		«classImports»
		
		//
		«classDeclaration»
	'''
	
	def private classHeader() '''
		//------------------------- Geneated Java Class File For DAOFactory Class ---------------------------------
	'''
	
	def private packageDeclaration() '''
		package «GeneratorData.packageName».«GeneratorConstants.SRC_GEN_DB»;
	'''
	
	def private classImports() '''
		«FOR f:imports»
			import «f»;
		«ENDFOR»
	'''
		
	def private classJavaDoc() '''
		/**
		 * <!-- begin-user-doc -->
		 * This Class contains the representation of the model object '<em><b></b></em>'.
		 * For each Entity the generator creates an EntityGEN.java file,
		 * which contains EntityGEN class with every Property and Association for the Entity.
		 * This Java file will be override, with each generation.
		 * The generator creates an Entity.java file, with the Entity class,
		 * which extends form EntityGEN. This file normally (if the Entity is not deleted)
		 * won't be overrided, so this file can be modified between the generations.
		 * 
		 * <p>
		 * This is The EntityGEN.java file. It must contain only the generated code.
		 * Do not edit it, or it will be overrided with each generation!
		 * </p>
		 * <!-- end-user-doc -->
		 *
		 * <p>
		 * The following features are supported:
		 * <ul>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getCache <em>Cache</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getPersistentType <em>Persistent Type</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#isAbstract <em>Abstract</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getName <em>Name</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getSuperType <em>Super Type</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getRelationship <em>Relationship</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getProperty <em>Property</em>}</li>
		 * </ul>
		 * </p>
		 *
		 * @see hu.bme.hellga.onlab.OnlabPackage#getEntity()
		 * @model
		 * @generated
		 */
	'''
	
	def private classDeclaration() '''
		public class DAOFactory {
			
			«classBody»
		}
	'''
	
	def private classBody() '''
		«properties»
		
		«initDB»
		
		«closeDB»
		
		«dbName»
		
		«dbVersion»
		
		«daoGetters»
		
		«genericDAO»
	'''
	
	def private properties() '''
		«FOR f:wes»
			//
			«f.propertyDeclaration»
			
		«ENDFOR»
		«IF GeneratorData.cache»
			//
			private static GenericDAOInterface genericDI;
		«ENDIF»
		
	'''
	
	def private propertyJavaDoc(WrappedEntity we) '''
		/**
		 * <p>
		 * JavaDoc
		 * </p>
		 * <!-- end-user-doc -->
		 *
		 * <p>
		 * The following features are supported:
		 *
		 * @see hu.bme.hellga.onlab.OnlabPackage#getEntity()
		 * @model
		 * @generated
		 */
	'''
	
	def private propertyDeclaration(WrappedEntity we) '''
		private static «we.entity.name.toFirstUpper»DAOInterface «we.entity.name.toFirstLower»DI;
		
	'''
	
	def private initDB() '''
		//
		public static void initDatabase(Context c){
			DatabaseFactory.initDatabase(c);
		}
	'''
	
	def private closeDB() '''
		//
		public static void closeDatabase(){
			DatabaseFactory.colseDatabase();
		}
	'''
	
	def private dbName() '''
		//
		public static void getDatabaseName(){
			DatabaseFactory.getDBName();
		}
	'''
	
	def private dbVersion() '''
		//
		public static void getDatabaseVersion(){
			DatabaseFactory.getDBVersion();
		}
	'''
	
	def private daoGetters() '''
		«FOR f:wes»
			//
			«f.daoGetter»
			
		«ENDFOR»
	'''
	
	def private daoGetterJavaDoc(WrappedEntity we) '''
		/**
		 * <p>
		 * JavaDoc
		 * </p>
		 * <!-- end-user-doc -->
		 *
		 * <p>
		 * The following features are supported:
		 *
		 * @see hu.bme.hellga.onlab.OnlabPackage#getEntity()
		 * @model
		 * @generated
		 */
	'''
	
	def private daoGetter(WrappedEntity we) '''
		«IF !(we.genHelp == GeneratorHelper.JoinClass)»public «ENDIF»static «we.entity.name.toFirstUpper»DAOInterface get«we.entity.name.toFirstUpper»DAO(){
			if(«we.entity.name.toFirstLower»DI == null){
				«we.entity.name.toFirstLower»DI = new «we.entity.name.toFirstUpper»DAO();
			}
			return «we.entity.name.toFirstLower»DI;
		}
	'''
	
	def private genericDAO() '''
		«IF GeneratorData.cache»
			static <T> Dao<T, Integer> getGenericDAO(Class<T> t){
				if(genericDI ==  null){
					genericDI = new GenericDAO();
				}
				return genericDI.getGenericDAO(t);
			}
		«ENDIF»
	'''
	
}