package hu.bme.mit.mobilegen.datamodel.generator.database.ormlite

import hu.bme.mit.mobilegen.datamodel.datamodel.Association
import hu.bme.mit.mobilegen.datamodel.datamodel.Entity
import hu.bme.mit.mobilegen.datamodel.generator.GeneratorConstants
import hu.bme.mit.mobilegen.datamodel.generator.GeneratorData
import hu.bme.mit.mobilegen.datamodel.helper.GeneratorHelper
import hu.bme.mit.mobilegen.datamodel.helper.WrappedAssociation
import hu.bme.mit.mobilegen.datamodel.helper.WrappedEntity
import hu.bme.mit.mobilegen.datamodel.logger.Log
import java.util.ArrayList
import java.util.List
import org.eclipse.xtext.generator.IFileSystemAccess

class ORMLiteDAOGenerator {
	
	WrappedEntity wrappedEntity;
	Entity entity;
	List<String> imports;
	
	def generateDAO(WrappedEntity we, IFileSystemAccess fsa) {
		wrappedEntity = we;
		entity = we.entity;
		collectImports();
		Log.info("Generating " + entity.name.toFirstUpper + "DAO.java");
		fsa.generateFile(entity.name.toFirstUpper + "DAO.java", GeneratorConstants.SRC_GEN_DB, entity.generateDAO);
	}
	
	def private collectImports(){
		imports = new ArrayList<String>();
		
		imports.add(GeneratorData.packageName + ".entities." + wrappedEntity.entity.name.toFirstUpper);
		
		for(Association a : wrappedEntity.dbManyToOneForeignKeysCont){
			imports.add(GeneratorData.packageName + ".entities." + a.connectTo.name.toFirstUpper);
		}
		for(Association a : wrappedEntity.dbOneToOneForeignKeysContInv){
			imports.add(GeneratorData.packageName + ".entities." + a.connectTo.name.toFirstUpper);
		}
		for(Association a : wrappedEntity.dbManyToOneForeignKeys){
			imports.add(GeneratorData.packageName + ".entities." + a.connectTo.name.toFirstUpper);
		}
		for(Association a : wrappedEntity.dbOneToOneForeignKeysInv){
			imports.add(GeneratorData.packageName + ".entities." + a.connectTo.name.toFirstUpper);
		}
		for(Entity e : wrappedEntity.dbManyToOneForeignKeysContRev){
			imports.add(GeneratorData.packageName + ".entities." + e.name.toFirstUpper);
		}
		for(Entity e : wrappedEntity.dbOneToOneForeignKeysContRevInv){
			imports.add(GeneratorData.packageName + ".entities." + e.name.toFirstUpper);
		}
		for(WrappedAssociation wa : wrappedEntity.dbManyToOneForeignKeysRev){
			imports.add(GeneratorData.packageName + ".entities." + wa.container.name.toFirstUpper);
		}
		for(WrappedAssociation wa : wrappedEntity.dbOneToOneForeignKeysRevInv){
			imports.add(GeneratorData.packageName + ".entities." + wa.container.name.toFirstUpper);
		}
		
		for(Entity e : wrappedEntity.dbManyToManyJoinTable){
			for(Association a : e.association){
				if(!a.connectTo.name.equals(entity.name)){
					imports.add(GeneratorData.packageName + ".entities." + a.connectTo.name.toFirstUpper);
				}
			}
		}
		
		imports.add("java.util.List");
		imports.add("com.j256.ormlite.dao.Dao");
		imports.add("java.sql.SQLException");
		
		if(wrappedEntity.hasJoinTable){
			imports.add("java.util.ArrayList");
		}
	}
	
	def private generateDAO(Entity e) '''
		«e.classHeader»
		
		«packageDeclaration»
		
		«e.classImports»
		
		//
		«e.classDeclaration»
	'''
	
	def private classHeader(Entity e) '''
		//------------------------- Geneated Java Class File For «e.name»DAO Class ---------------------------------
	'''
	
	def private packageDeclaration() '''
		package «GeneratorData.packageName».«GeneratorConstants.SRC_GEN_DB»;
	'''
	
	def private classImports(Entity e) '''
		«FOR f:imports»
			import «f»;
		«ENDFOR»
	'''
		
	def private classJavaDoc(Entity e) '''
		/**
		 * <!-- begin-user-doc -->
		 * This Class contains the representation of the model object '<em><b></b></em>'.
		 * For each Entity the generator creates an EntityGEN.java file,
		 * which contains EntityGEN class with every Property and Association for the Entity.
		 * This Java file will be override, with each generation.
		 * The generator creates an Entity.java file, with the Entity class,
		 * which extends form EntityGEN. This file normally (if the Entity is not deleted)
		 * won't be overrided, so this file can be modified between the generations.
		 * 
		 * <p>
		 * This is The EntityGEN.java file. It must contain only the generated code.
		 * Do not edit it, or it will be overrided with each generation!
		 * </p>
		 * <!-- end-user-doc -->
		 *
		 * <p>
		 * The following features are supported:
		 * <ul>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getCache <em>Cache</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getPersistentType <em>Persistent Type</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#isAbstract <em>Abstract</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getName <em>Name</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getSuperType <em>Super Type</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getRelationship <em>Relationship</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getProperty <em>Property</em>}</li>
		 * </ul>
		 * </p>
		 *
		 * @see hu.bme.hellga.onlab.OnlabPackage#getEntity()
		 * @model
		 * @generated
		 */
	'''
	
	def private classDeclaration(Entity e) '''
		public class «e.name.toFirstUpper»DAO implements «e.name.toFirstUpper»DAOInterface {
			
			«e.classBody»
		}
	'''
	
	def private classBody(Entity e) '''
		// Property declarations
		
		«e.properties»
		
		// Constructor
		
		«e.constructor»
		
		«e.body»
	'''
	
	def private properties(Entity e) '''
		//
		private static Dao<«e.name.toFirstUpper», Integer> «e.name.toFirstLower»DAO = null;
	'''
	
	def private constructor(Entity e) '''
		public «e.name.toFirstUpper»DAO(){
			try {
				«e.name.toFirstLower»DAO = DatabaseFactory.get«e.name.toFirstUpper»DAO();
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}
	'''
	
	def private body(Entity e) '''
		«e.create»
		
		«e.update»
		
		«e.deleteOne»
		
		«e.deleteMore»
		
		«e.refresh»
		
		«e.all»
		
		«e.byContainment»
		
		«e.byReference»
		
		«e.byID»
		
		«e.dao»
		
		«wrappedEntity.others»
		
	'''
	def private create(Entity e) '''
		«FOR f:wrappedEntity.dbManyToOneForeignKeysCont»
			@Override
			public boolean create«e.name.toFirstUpper»(«f.connectTo.name.toFirstUpper» «f.connectTo.name.toFirstLower», «e.name.toFirstUpper» «e.name.toFirstLower») {
				try {
					«e.name.toFirstLower».set«f.name.toFirstUpper»(«f.connectTo.name.toFirstLower»);
					«e.name.toFirstLower»DAO.create(«e.name.toFirstLower»);
				} catch (SQLException e) {
					e.printStackTrace();
					return false;
				}
				return true;
			}
		«ENDFOR»
		«FOR f:wrappedEntity.dbOneToOneForeignKeysContInv»
			@Override
			public boolean create«e.name.toFirstUpper»(«f.connectTo.name.toFirstUpper» «f.connectTo.name.toFirstLower», «e.name.toFirstUpper» «e.name.toFirstLower») {
				try {
					«e.name.toFirstLower».set«f.name.toFirstUpper»(«f.connectTo.name.toFirstLower»);
					«e.name.toFirstLower»DAO.create(«e.name.toFirstLower»);
				} catch (SQLException e) {
					e.printStackTrace();
					return false;
				}
				return true;
			}
		«ENDFOR»
		«IF !e.onlyInContainment»
			@Override
				public boolean create«e.name.toFirstUpper»(«e.name.toFirstUpper» «e.name.toFirstLower»){
					try {
						«e.name.toFirstLower»DAO.create(«e.name.toFirstLower»);
					} catch (SQLException e) {
						e.printStackTrace();
						return false;
					}
					return true;
				}
		«ENDIF»
	'''
	
	def private update(Entity e) '''
		@Override
		public boolean update«e.name.toFirstUpper»(«e.name.toFirstUpper» «e.name.toFirstLower») {
			try {
				«e.name.toFirstLower»DAO.update(«e.name.toFirstLower»);
			} catch (SQLException e) {
				e.printStackTrace();
				return false;
			}
			return true;
		}
	'''
	
	def private deleteOne(Entity e) '''
		public boolean delete«e.name.toFirstUpper»(«e.name.toFirstUpper» «e.name.toFirstLower») {
			try {
				«FOR f:wrappedEntity.dbManyToOneForeignKeysContRev»
					List<«f.name.toFirstUpper»> «f.name.toFirstLower»list = DAOFactory.get«f.name.toFirstUpper»DAO().get«f.name.toFirstUpper»sBy«e.name.toFirstUpper»Containment(«e.name.toFirstLower»);
					DAOFactory.get«f.name.toFirstUpper»DAO().delete«f.name.toFirstUpper»s(«f.name.toFirstLower»list);
					
				«ENDFOR»
				«FOR f:wrappedEntity.dbOneToOneForeignKeysContRevInv»
					«f.name.toFirstUpper» «f.name.toFirstLower» = DAOFactory.get«f.name.toFirstUpper»DAO().get«f.name.toFirstUpper»By«e.name.toFirstUpper»Containment(«e.name.toFirstLower»);
					DAOFactory.get«f.name.toFirstUpper»DAO().delete«f.name.toFirstUpper»(«f.name.toFirstLower»);
					
				«ENDFOR»
				«FOR f:wrappedEntity.dbManyToOneForeignKeysRev»
					List<«f.container.name.toFirstUpper»> «f.container.name.toFirstLower»list = DAOFactory.get«f.container.name.toFirstUpper»DAO().get«f.container.name.toFirstUpper»sBy«e.name.toFirstUpper»Reference(«e.name.toFirstLower»);
					
					for(«f.container.name.toFirstUpper» «f.container.name.substring(0,1).toFirstLower» : «f.container.name.toFirstLower»list){
						«f.container.name.substring(0,1).toFirstLower».set«f.association.name.toFirstUpper»(null);
					}
					
				«ENDFOR»
				«FOR f:wrappedEntity.dbOneToOneForeignKeysRevInv»
					«f.container.name.toFirstUpper» «f.container.name.toFirstLower» = DAOFactory.get«f.container.name.toFirstUpper»DAO().get«f.container.name.toFirstUpper»By«e.name.toFirstUpper»Reference(«e.name.toFirstLower»);
					«f.container.name.toFirstLower».set«f.association.name.toFirstUpper»(null);
					
				«ENDFOR»
				«e.name.toFirstLower»DAO.delete(«e.name.toFirstLower»);
			} catch (SQLException e) {
				e.printStackTrace();
				return false;
			}
			return true;
		}
	'''
	
	def private deleteMore(Entity e) '''
		«IF wrappedEntity.hasContainment»
			public boolean delete«e.name.toFirstUpper»s(List<«e.name.toFirstUpper»> «e.name.toFirstLower»s) {
				try {
					for(«e.name.toFirstUpper» «e.name.toFirstLower» : «e.name.toFirstLower»s){
						«FOR f:wrappedEntity.dbManyToOneForeignKeysContRev»
							List<«f.name.toFirstUpper»> «f.name.toFirstLower»list = DAOFactory.get«f.name.toFirstUpper»DAO().get«f.name.toFirstUpper»sBy«e.name.toFirstUpper»Containment(«e.name.toFirstLower»);
							DAOFactory.get«f.name.toFirstUpper»DAO().delete«f.name.toFirstUpper»s(«f.name.toFirstLower»list);
							
						«ENDFOR»
						«FOR f:wrappedEntity.dbOneToOneForeignKeysContRevInv»
							«f.name.toFirstUpper» «f.name.toFirstLower» = DAOFactory.get«f.name.toFirstUpper»DAO().get«f.name.toFirstUpper»By«e.name.toFirstUpper»Containment(«e.name.toFirstLower»);
							DAOFactory.get«f.name.toFirstUpper»DAO().delete«f.name.toFirstUpper»(«f.name.toFirstLower»);
							
						«ENDFOR»
						«FOR f:wrappedEntity.dbManyToOneForeignKeysRev»
							List<«f.container.name.toFirstUpper»> «f.container.name.toFirstLower»list = DAOFactory.get«f.container.name.toFirstUpper»DAO().get«f.container.name.toFirstUpper»sBy«e.name.toFirstUpper»Reference(«e.name.toFirstLower»);
							
							for(«f.container.name.toFirstUpper» «f.container.name.substring(0,1).toFirstLower» : «f.container.name.toFirstLower»list){
								«f.container.name.substring(0,1).toFirstLower».set«f.association.name.toFirstUpper»(null);
							}
							
						«ENDFOR»
						«FOR f:wrappedEntity.dbOneToOneForeignKeysRevInv»
							«f.container.name.toFirstUpper» «f.container.name.toFirstLower» = DAOFactory.get«f.container.name.toFirstUpper»DAO().get«f.container.name.toFirstUpper»By«e.name.toFirstUpper»Reference(«e.name.toFirstLower»);
							«f.container.name.toFirstLower».set«f.association.name.toFirstUpper»(null);
							
						«ENDFOR»
						«e.name.toFirstLower»DAO.delete(«e.name.toFirstLower»);
					}
				} catch (SQLException e) {
					e.printStackTrace();
					return false;
				}
				return true;
			}
		«ELSE»
			@Override
			public boolean delete«e.name.toFirstUpper»s(List<«e.name.toFirstUpper»> «e.name.toFirstLower»s) {
				try {
					«e.name.toFirstLower»DAO.delete(«e.name.toFirstLower»s);
				} catch (SQLException e) {
					e.printStackTrace();
					return false;
				}
				return true;
			}
		«ENDIF»
	'''
	
	def private refresh(Entity e) '''
		@Override
		public boolean refresh«e.name.toFirstUpper»(«e.name.toFirstUpper» «e.name.toFirstLower») {
			try {
				«e.name.toFirstLower»DAO.refresh(«e.name.toFirstLower»);
			} catch (SQLException e) {
				e.printStackTrace();
				return false;
			}
			return true;
		}
	'''
	
	def private getAll(Entity e) '''
		@Override
		public List<«e.name.toFirstUpper»> get«e.name.toFirstUpper»s() {
			try {
				return «e.name.toFirstLower»DAO.queryForAll();
			} catch (SQLException e) {
				e.printStackTrace();
				return null;
			}
		}
	'''
	
	def private getByContainment(Entity e) '''
		«FOR f:wrappedEntity.dbManyToOneForeignKeysCont»
			@Override
			public List<«e.name.toFirstUpper»> get«e.name.toFirstUpper»sBy«f.connectTo.name.toFirstUpper»Containment(«f.connectTo.name.toFirstUpper» «f.connectTo.name.toFirstLower») {
				try {
					List<«e.name.toFirstUpper»> «e.name.toFirstLower»List = «e.name.toFirstLower»DAO.query(«e.name.toFirstLower»DAO.queryBuilder().where().eq(«e.name.toFirstUpper».«f.name.toUpperCase»_ID_FIELD_NAME, «f.connectTo.name.toFirstLower».getId()).prepare());
					return «e.name.toFirstLower»List;
				} catch (SQLException e) {
					e.printStackTrace();
					return null;
				}
			}
		«ENDFOR»
		«FOR f:wrappedEntity.dbOneToOneForeignKeysContInv»
			@Override
			public «e.name.toFirstUpper» get«e.name.toFirstUpper»By«f.connectTo.name.toFirstUpper»Containment(«f.connectTo.name.toFirstUpper» «f.connectTo.name.toFirstLower») {
				try {
					List<«e.name.toFirstUpper»> «e.name.toFirstLower»List = «e.name.toFirstLower»DAO.query(«e.name.toFirstLower»DAO.queryBuilder().where().eq(«e.name.toFirstUpper».«f.name.toUpperCase»_ID_FIELD_NAME, «f.connectTo.name.toFirstLower».getId()).prepare());
					if(«e.name.toFirstLower»List.size() > 0){
						return «e.name.toFirstLower»List.get(0);
					}
					else{
						return null;
					}
				} catch (SQLException e) {
					e.printStackTrace();
					return null;
				}
			}
		«ENDFOR»
	'''
	
	def private getByReference(Entity e) '''
		«FOR f:wrappedEntity.dbManyToOneForeignKeys»
			@Override
			public List<«e.name.toFirstUpper»> get«e.name.toFirstUpper»sBy«f.connectTo.name.toFirstUpper»Reference(«f.connectTo.name.toFirstUpper» «f.connectTo.name.toFirstLower») {
				try {
					List<«e.name.toFirstUpper»> «e.name.toFirstLower»List = «e.name.toFirstLower»DAO.query(«e.name.toFirstLower»DAO.queryBuilder().where().eq(«e.name.toFirstUpper».«f.name.toUpperCase»_ID_FIELD_NAME, «f.connectTo.name.toFirstLower».getId()).prepare());
					return «e.name.toFirstLower»List;
				} catch (SQLException e) {
					e.printStackTrace();
					return null;
				}
			}
		«ENDFOR»
		«FOR f:wrappedEntity.dbOneToOneForeignKeysInv»
			@Override
			public «e.name.toFirstUpper» get«e.name.toFirstUpper»By«f.connectTo.name.toFirstUpper»Reference(«f.connectTo.name.toFirstUpper» «f.connectTo.name.toFirstLower») {
				try {
					List<«e.name.toFirstUpper»> «e.name.toFirstLower»List = «e.name.toFirstLower»DAO.query(«e.name.toFirstLower»DAO.queryBuilder().where().eq(«e.name.toFirstUpper».«f.name.toUpperCase»_ID_FIELD_NAME, «f.connectTo.name.toFirstLower».getId()).prepare());
					if(«e.name.toFirstLower»List.size() > 0){
						return «e.name.toFirstLower»List.get(0);
					}
					else{
						return null;
					}
				} catch (SQLException e) {
					e.printStackTrace();
					return null;
				}
			}
		«ENDFOR»
	'''
	
	def private getByID(Entity e) '''
		@Override
		public «e.name.toFirstUpper» get«e.name.toFirstUpper»ByID(int id) {
			try {
				return «e.name.toFirstLower»DAO.queryForId(id);
			} catch (SQLException e) {
				e.printStackTrace();
				return null;
			}
		}
	'''
	
	def private dao(Entity e) '''
		@Override
		public Dao<«e.name.toFirstUpper», Integer> getDAO() {
			return «e.name.toFirstLower»DAO;
		}
	'''
	
	def private others(WrappedEntity we) '''
		«IF wrappedEntity.genHelp == GeneratorHelper.JoinClass»
			«wrappedEntity.manyToManyJoinTable»
		«ENDIF»
		
		«IF wrappedEntity.hasJoinTable»
			«wrappedEntity.manyToManyConnection»
		«ENDIF»
	'''
	
	def private manyToManyConnection(WrappedEntity we) '''
		«IF wrappedEntity.hasJoinTable»
			«FOR f:wrappedEntity.dbManyToManyJoinTable»
				«we.manyToMany(f)»
			«ENDFOR»
		«ENDIF»
	'''
	
	def private manyToMany(WrappedEntity we, Entity join) '''
		«FOR f:join.association»
			«IF !f.connectTo.name.equals(we.entity.name)»
				«we.setManyToMany(join, f)»
				«we.getManyToMany(join, f)»
				«we.removeManyToMany(join, f)»
			«ENDIF»
		«ENDFOR»
		
	'''
	
	def private setManyToMany(WrappedEntity we, Entity join, Association f) '''
		@Override
		public void set«f.connectTo.name.toFirstUpper»To«we.entity.name.toFirstUpper»(«f.connectTo.name.toFirstUpper» «f.connectTo.name.toFirstLower», «we.entity.name.toFirstUpper» «we.entity.name.toFirstLower»){
			DAOFactory.get«join.name.toFirstUpper»DAO().create«join.name.toFirstUpper»(«f.connectTo.name.toFirstLower», «we.entity.name.toFirstLower»);
		}
		
	'''
	
	def private getManyToMany(WrappedEntity we, Entity join, Association f) '''
		@Override
		public List<«f.connectTo.name.toFirstUpper»> get«f.connectTo.name.toFirstUpper»sBy«we.entity.name.toFirstUpper»(«we.entity.name.toFirstUpper» «we.entity.name.toFirstLower»){
			List<«f.connectTo.name.toFirstUpper»> «f.connectTo.name.toFirstLower»List = new ArrayList<«f.connectTo.name.toFirstUpper»>();
			List<«join.name.toFirstUpper»> «join.name.toFirstLower»List = DAOFactory.get«join.name.toFirstUpper»DAO().get«join.name.toFirstUpper»sBy«we.entity.name.toFirstUpper»Containment(«we.entity.name.toFirstLower»);
			for(«join.name.toFirstUpper» «join.name.toFirstLower» : «join.name.toFirstLower»List){
				«f.connectTo.name.toFirstLower»List.add(DAOFactory.get«f.connectTo.name.toFirstUpper»DAO().get«f.connectTo.name.toFirstUpper»ByID(«join.name.toFirstLower».get«FOR a:join.association»«IF !a.connectTo.name.equals(we.entity.name)»«a.name.toFirstUpper»«ENDIF»«ENDFOR»().getId()));
			}
			return «f.connectTo.name.toFirstLower»List;
		}
		
	'''
	
	def private removeManyToMany(WrappedEntity we, Entity join, Association f) '''
		@Override
		public boolean remove«f.connectTo.name.toFirstUpper»By«we.entity.name.toFirstUpper»(«f.connectTo.name.toFirstUpper» «f.connectTo.name.toFirstLower», «we.entity.name.toFirstUpper» «we.entity.name.toFirstLower»){
			List<«join.name.toFirstUpper»> «join.name.toFirstLower»DelList = new ArrayList<«join.name.toFirstUpper»>();
			List<«join.name.toFirstUpper»> «join.name.toFirstUpper»List = DAOFactory.get«join.name.toFirstUpper»DAO().get«join.name.toFirstUpper»sBy«we.entity.name.toFirstUpper»Containment(«we.entity.name.toFirstLower»);
			for(«join.name.toFirstUpper» «join.name.toFirstLower» : «join.name.toFirstUpper»List){
				if(«join.name.toFirstLower».get«FOR a:join.association»«IF !a.connectTo.name.equals(we.entity.name)»«a.name.toFirstUpper»«ENDIF»«ENDFOR»().getId() == «f.connectTo.name.toFirstLower».getId()){
					«join.name.toFirstLower»DelList.add(«join.name.toFirstLower»);
				}
			}
			DAOFactory.get«join.name.toFirstUpper»DAO().delete«join.name.toFirstUpper»s(«join.name.toFirstLower»DelList);
			return true;
		}
		
	'''
	
	def private manyToManyJoinTable(WrappedEntity we) '''
		@Override
		public boolean create«we.entity.name.toFirstUpper»(«we.dbManyToOneForeignKeysCont.get(0).connectTo.name.toFirstUpper» «we.dbManyToOneForeignKeysCont.get(0).connectTo.name.toFirstLower», «we.dbManyToOneForeignKeysCont.get(1).connectTo.name.toFirstUpper» «we.dbManyToOneForeignKeysCont.get(1).connectTo.name.toFirstLower»){
			try {
				«we.entity.name.toFirstUpper» «we.entity.name.toFirstLower» = new «we.entity.name.toFirstUpper»();
				«FOR f:we.getDbManyToOneForeignKeysCont()»
					«we.entity.name.toFirstLower».set«f.name.toFirstUpper»(«f.connectTo.name.toFirstLower»);
				«ENDFOR»
				«we.entity.name.toFirstLower»DAO.create(«we.entity.name.toFirstLower»);
			} catch (SQLException e) {
				e.printStackTrace();
				return false;
			}
			return true;
		}
		
		@Override
		public boolean create«we.entity.name.toFirstUpper»(«we.dbManyToOneForeignKeysCont.get(1).connectTo.name.toFirstUpper» «we.dbManyToOneForeignKeysCont.get(1).connectTo.name.toFirstLower», «we.dbManyToOneForeignKeysCont.get(0).connectTo.name.toFirstUpper» «we.dbManyToOneForeignKeysCont.get(0).connectTo.name.toFirstLower»){
			try {
				«we.entity.name.toFirstUpper» «we.entity.name.toFirstLower» = new «we.entity.name.toFirstUpper»();
				«FOR f:we.getDbManyToOneForeignKeysCont()»
					«we.entity.name.toFirstLower».set«f.name.toFirstUpper»(«f.connectTo.name.toFirstLower»);
				«ENDFOR»
				«we.entity.name.toFirstLower»DAO.create(«we.entity.name.toFirstLower»);
			} catch (SQLException e) {
				e.printStackTrace();
				return false;
			}
			return true;
		}
		
	'''
	
}