package hu.bme.mit.mobilegen.datamodel.generator.database.ormlite

import hu.bme.mit.mobilegen.datamodel.datamodel.Association
import hu.bme.mit.mobilegen.datamodel.datamodel.Entity
import hu.bme.mit.mobilegen.datamodel.generator.GeneratorConstants
import hu.bme.mit.mobilegen.datamodel.generator.GeneratorData
import hu.bme.mit.mobilegen.datamodel.helper.GeneratorHelper
import hu.bme.mit.mobilegen.datamodel.helper.WrappedEntity
import hu.bme.mit.mobilegen.datamodel.logger.Log
import java.util.ArrayList
import java.util.List
import org.eclipse.xtext.generator.IFileSystemAccess

class ORMLiteDAOInterfaceGenerator {
	
	WrappedEntity wrappedEntity;
	Entity entity;
	List<String> imports;
	
	def generateDAOInterface(WrappedEntity we, IFileSystemAccess fsa) {
		wrappedEntity = we;
		entity = we.entity;
		collectImports();
		Log.info("Generating " + entity.name.toFirstUpper + "DAOInterface.java");
		fsa.generateFile(entity.name.toFirstUpper + "DAOInterface.java", GeneratorConstants.SRC_GEN_DB, entity.generateDAOInterface);
	}
	
	def private collectImports(){
		imports = new ArrayList<String>();
		
		imports.add(GeneratorData.packageName + ".entities." + wrappedEntity.entity.name.toFirstUpper);
		
		for(Association a : wrappedEntity.dbManyToOneForeignKeysCont){
			imports.add(GeneratorData.packageName + ".entities." + a.connectTo.name.toFirstUpper);
		}
		for(Association a : wrappedEntity.dbOneToOneForeignKeysContInv){
			imports.add(GeneratorData.packageName + ".entities." + a.connectTo.name.toFirstUpper);
		}
		for(Association a : wrappedEntity.dbManyToOneForeignKeys){
			imports.add(GeneratorData.packageName + ".entities." + a.connectTo.name.toFirstUpper);
		}
		for(Association a : wrappedEntity.dbOneToOneForeignKeysInv){
			imports.add(GeneratorData.packageName + ".entities." + a.connectTo.name.toFirstUpper);
		}
		for(Entity e : wrappedEntity.dbManyToManyJoinTable){
			for(Association a : e.association){
				if(!a.connectTo.name.equals(entity.name)){
					imports.add(GeneratorData.packageName + ".entities." + a.connectTo.name.toFirstUpper);
				}
			}
		}
		
		
		imports.add("java.util.List");
		imports.add("com.j256.ormlite.dao.Dao");
	}
	
	def private generateDAOInterface(Entity e) '''
		«e.classHeader»
		
		«packageDeclaration»
		
		«e.classImports»
		
		//
		«e.classDeclaration»
	'''
	
	def private classHeader(Entity e) '''
		//------------------------- Geneated Java Class File For «e.name»DAOInterface Class ---------------------------------
	'''
	
	def private packageDeclaration() '''
		package «GeneratorData.packageName».«GeneratorConstants.SRC_GEN_DB»;
	'''
	
	def private classImports(Entity e) '''
		«FOR f:imports»
			import «f»;
		«ENDFOR»
	'''
		
	def private classJavaDoc(Entity e) '''
		/**
		 * <!-- begin-user-doc -->
		 * This Class contains the representation of the model object '<em><b></b></em>'.
		 * For each Entity the generator creates an EntityGEN.java file,
		 * which contains EntityGEN class with every Property and Association for the Entity.
		 * This Java file will be override, with each generation.
		 * The generator creates an Entity.java file, with the Entity class,
		 * which extends form EntityGEN. This file normally (if the Entity is not deleted)
		 * won't be overrided, so this file can be modified between the generations.
		 * 
		 * <p>
		 * This is The EntityGEN.java file. It must contain only the generated code.
		 * Do not edit it, or it will be overrided with each generation!
		 * </p>
		 * <!-- end-user-doc -->
		 *
		 * <p>
		 * The following features are supported:
		 * <ul>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getCache <em>Cache</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getPersistentType <em>Persistent Type</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#isAbstract <em>Abstract</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getName <em>Name</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getSuperType <em>Super Type</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getRelationship <em>Relationship</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getProperty <em>Property</em>}</li>
		 * </ul>
		 * </p>
		 *
		 * @see hu.bme.hellga.onlab.OnlabPackage#getEntity()
		 * @model
		 * @generated
		 */
	'''
	
	def private classDeclaration(Entity e) '''
		public interface «e.name.toFirstUpper»DAOInterface {
			«e.classBody»
		}
	'''
	
	def private classBody(Entity e) '''
		«e.create»
		public boolean update«e.name.toFirstUpper»(«e.name.toFirstUpper» «e.name.toFirstLower»);
		public boolean delete«e.name.toFirstUpper»(«e.name.toFirstUpper» «e.name.toFirstLower»);
		public boolean delete«e.name.toFirstUpper»s(List<«e.name.toFirstUpper»> «e.name.toFirstLower»);
		public boolean refresh«e.name.toFirstUpper»(«e.name.toFirstUpper» «e.name.toFirstLower»);
		public List<«e.name.toFirstUpper»> get«e.name.toFirstUpper»s();
		«e.byContainment»
		«e.byReference»
		public «e.name.toFirstUpper» get«e.name.toFirstUpper»ByID(int id);
		public Dao<«e.name.toFirstUpper», Integer> getDAO();
		«wrappedEntity.others»
	'''
	
	def private create(Entity e) '''
		«FOR f:wrappedEntity.dbManyToOneForeignKeysCont»
			public boolean create«e.name.toFirstUpper»(«f.connectTo.name.toFirstUpper» «f.connectTo.name.toFirstLower», «e.name.toFirstUpper» «e.name.toFirstLower»);
		«ENDFOR»
		«FOR f:wrappedEntity.dbOneToOneForeignKeysContInv»
			public boolean create«e.name.toFirstUpper»(«f.connectTo.name.toFirstUpper» «f.connectTo.name.toFirstLower», «e.name.toFirstUpper» «e.name.toFirstLower»);
		«ENDFOR»
		«IF !e.onlyInContainment»
			public boolean create«e.name.toFirstUpper»(«e.name.toFirstUpper» «e.name.toFirstLower»);
		«ENDIF»
	'''
	
	def private getByContainment(Entity e) '''
		«FOR f:wrappedEntity.dbManyToOneForeignKeysCont»
			public List<«e.name.toFirstUpper»> get«e.name.toFirstUpper»sBy«f.connectTo.name.toFirstUpper»Containment(«f.connectTo.name.toFirstUpper» «f.connectTo.name.toFirstLower»);
		«ENDFOR»
		«FOR f:wrappedEntity.dbOneToOneForeignKeysContInv»
			public «e.name.toFirstUpper» get«e.name.toFirstUpper»By«f.connectTo.name.toFirstUpper»Containment(«f.connectTo.name.toFirstUpper» «f.connectTo.name.toFirstLower»);
		«ENDFOR»
	'''
	
	def private getByReference(Entity e) '''
		«FOR f:wrappedEntity.dbManyToOneForeignKeys»
			public List<«e.name.toFirstUpper»> get«e.name.toFirstUpper»sBy«f.connectTo.name.toFirstUpper»Reference(«f.connectTo.name.toFirstUpper» «f.connectTo.name.toFirstLower»);
		«ENDFOR»
		«FOR f:wrappedEntity.dbOneToOneForeignKeysInv»
			public «e.name.toFirstUpper» get«e.name.toFirstUpper»By«f.connectTo.name.toFirstUpper»Reference(«f.connectTo.name.toFirstUpper» «f.connectTo.name.toFirstLower»);
		«ENDFOR»
	'''
	
	def private others(WrappedEntity we) '''
		«IF wrappedEntity.genHelp == GeneratorHelper.JoinClass»
			«wrappedEntity.manyToManyJoinTable»
		«ENDIF»
		«IF wrappedEntity.hasJoinTable»
			«wrappedEntity.manyToManyConnection»
		«ENDIF»
	'''
	
	def private manyToManyConnection(WrappedEntity we) '''
		«IF wrappedEntity.hasJoinTable»
			«FOR f:wrappedEntity.dbManyToManyJoinTable»
				«we.manyToMany(f)»
			«ENDFOR»
		«ENDIF»
	'''
	
	def private manyToMany(WrappedEntity we, Entity join) '''
		«FOR f:join.association»
			«IF !f.connectTo.name.equals(we.entity.name)»
				public void set«f.connectTo.name.toFirstUpper»To«we.entity.name.toFirstUpper»(«f.connectTo.name.toFirstUpper» «f.connectTo.name.toFirstLower», «we.entity.name.toFirstUpper» «we.entity.name.toFirstLower»);
				public List<«f.connectTo.name.toFirstUpper»> get«f.connectTo.name.toFirstUpper»sBy«we.entity.name.toFirstUpper»(«we.entity.name.toFirstUpper» «we.entity.name.toFirstUpper»);
				public boolean remove«f.connectTo.name.toFirstUpper»By«we.entity.name.toFirstUpper»(«f.connectTo.name.toFirstUpper» «f.connectTo.name.toFirstLower», «we.entity.name.toFirstUpper» «we.entity.name.toFirstLower»);
			«ENDIF»
		«ENDFOR»
		
	'''
	
	def private manyToManyJoinTable(WrappedEntity we) '''
		public boolean create«we.entity.name.toFirstUpper»(«we.dbManyToOneForeignKeysCont.get(0).connectTo.name.toFirstUpper» «we.dbManyToOneForeignKeysCont.get(0).connectTo.name.toFirstLower», «we.dbManyToOneForeignKeysCont.get(1).connectTo.name.toFirstUpper» «we.dbManyToOneForeignKeysCont.get(1).connectTo.name.toFirstLower»);
		public boolean create«we.entity.name.toFirstUpper»(«we.dbManyToOneForeignKeysCont.get(1).connectTo.name.toFirstUpper» «we.dbManyToOneForeignKeysCont.get(1).connectTo.name.toFirstLower», «we.dbManyToOneForeignKeysCont.get(0).connectTo.name.toFirstUpper» «we.dbManyToOneForeignKeysCont.get(0).connectTo.name.toFirstLower»);
	'''
}