package hu.bme.mit.mobilegen.datamodel.generator.database.ormlite

import hu.bme.mit.mobilegen.datamodel.datamodel.Association
import hu.bme.mit.mobilegen.datamodel.datamodel.ConnectionDirection
import hu.bme.mit.mobilegen.datamodel.datamodel.ConnectionType
import hu.bme.mit.mobilegen.datamodel.datamodel.Entity
import hu.bme.mit.mobilegen.datamodel.datamodel.PersistentType
import hu.bme.mit.mobilegen.datamodel.datamodel.Property
import hu.bme.mit.mobilegen.datamodel.generator.database.IDatabaseGeneratorUtility
import hu.bme.mit.mobilegen.datamodel.helper.WrappedEntity
import java.util.List

class ORMLiteGeneratorUtility implements IDatabaseGeneratorUtility{
	
	override classAnnotations(Entity e) '''
		«IF !e.abstract»
			@DatabaseTable(tableName = "«e.name.toLowerCase»")
		«ENDIF»
	'''
	
	override propertyAnnotations(Property p) '''
		«IF !p.transient && !p.static»
			@DatabaseField(columnName = «p.name.toUpperCase»_FIELD_NAME«IF p.unique», unique = true«ENDIF»«IF p.cantBeNull», canBeNull = false«ENDIF»«IF p.indexed», index = true«ENDIF»)
		«ENDIF»
	'''
	
	override associationAnnotations(Association a) '''
		«IF !a.transient && !a.static»
			«IF a.connectionType == ConnectionType.MANY»
				@ForeignCollectionField(foreignFieldName = "«a.inverseName»")
			«ELSE»
				@DatabaseField(foreign = true, columnName = «a.name.toUpperCase»_ID_FIELD_NAME«IF a.connectionDir == ConnectionDirection.ONE_TO_ONE_UNI_DIRECTION || a.connectionDir == ConnectionDirection.ONE_TO_ONE_BI_DIRECTION || a.connectionDir == ConnectionDirection.ONE_TO_ONE_INVERSE_DIRECTION», unique = true«ENDIF»)
			«ENDIF»
		«ENDIF»
	'''
	
	override propertyID(Entity e) '''
		«IF !e.extends»
			@DatabaseField(columnName = "id", generatedId = true)
			private int id;
		«ENDIF»
	'''
	
	override associationListDeclaration(Association a) '''
		protected ForeignCollection<«a.connectTo.name.toFirstUpper»> «a.name.toFirstLower»;
	'''
	
	override associationListGetter(Association a) '''
		public ForeignCollection<«a.connectTo.name»> get«a.name.toFirstUpper»(){
			return this.«a.name.toFirstLower»;
		}
	'''
	
	override associationListSetter(Association a) '''
		public void set«a.name.toFirstUpper»(ForeignCollection<«a.connectTo.name»> «a.name.toFirstLower»){
			this.«a.name.toFirstLower» = «a.name.toFirstLower»;
		}
	'''
	
	override idGetter(Entity e) '''
		«IF !e.extends»
			//
			public int getId() {
				return id;
			}
		«ENDIF»
	'''
	
	override fieldNames(Entity e) '''
		«FOR f:e.property»
			«IF !f.transient»
				public static final String «f.name.toUpperCase»_FIELD_NAME = "«f.name.toLowerCase»";
			«ENDIF»
		«ENDFOR»
		«FOR f:e.association»
			«IF !f.transient»
				public static final String «f.name.toUpperCase»_ID_FIELD_NAME = "«f.name.toLowerCase»_id";
			«ENDIF»
		«ENDFOR»
	'''
	
	override collectEntityGENImports(List<String> imports, WrappedEntity we) {
		var field = false;
		var collection = false;
		for(Property p : we.entity.property){
			if(!p.transient && ! p.static){
				field = true;
			}
		}
		for(Association a : we.entity.association){
			if(!a.transient && ! a.static){
				if(a.connectionType == ConnectionType.MANY){
					collection = true;
				}
				else{
					field = true;
				}
			}
		}
		
		if(field){
			imports.add("com.j256.ormlite.field.DatabaseField");
		}
		if(collection){
			imports.add("com.j256.ormlite.dao.ForeignCollection");
			imports.add("com.j256.ormlite.field.ForeignCollectionField");
		}
	}
	
	override collectEntityImports(List<String> imports, WrappedEntity we) {
		if(we.entity.persistentType == PersistentType.DB && !we.entity.abstract){
			imports.add("com.j256.ormlite.table.DatabaseTable");
		}
	}
	
}