package hu.bme.mit.mobilegen.datamodel.generator.entity

import com.google.inject.Inject
import hu.bme.mit.mobilegen.datamodel.datamodel.Entity
import hu.bme.mit.mobilegen.datamodel.datamodel.PersistentType
import hu.bme.mit.mobilegen.datamodel.generator.GeneratorConstants
import hu.bme.mit.mobilegen.datamodel.generator.GeneratorData
import hu.bme.mit.mobilegen.datamodel.generator.database.IDatabaseGeneratorUtility
import hu.bme.mit.mobilegen.datamodel.helper.WrappedEntity
import hu.bme.mit.mobilegen.datamodel.logger.Log
import java.util.ArrayList
import java.util.List
import org.eclipse.xtext.generator.IFileSystemAccess

class EntityGenerator {
	
	@Inject
	IDatabaseGeneratorUtility dbGen;
	
	WrappedEntity wrappedEntity;
	Entity entity;
	List<String> imports;
	
	def generateEntity(WrappedEntity we, IFileSystemAccess fsa) {
		wrappedEntity = we;
		entity = we.entity;
		collectImports();
		if(entity.persistentType == PersistentType.DB){
			dbGen.collectEntityImports(imports, wrappedEntity);
		}
		Log.info("Generating " + entity.name.toFirstUpper + ".java");
		fsa.generateFile(entity.name.toFirstUpper + ".java",  GeneratorConstants.SRC_ENTITIES, entity.generateEntity);
	}
	
	def private collectImports(){
		imports = new ArrayList<String>();
	}
	
	def private generateEntity(Entity e) '''
		«e.classHeader»
		
		«packageDeclaration»
		
		«e.classImports»
		
		//
		«e.classAnnotations»
		«e.classDeclaration»
	'''
	
	def private classHeader(Entity e) '''
		//------------------------- Geneated Java Class File For Entity - «e.name» ---------------------------------
		//------------------------- Model Implementation In «wrappedEntity.resourceURI» Resource File ----------------------------
	'''
	
	def private packageDeclaration() '''
		package «GeneratorData.packageName».«GeneratorConstants.SRC_GEN_ENTITIES»;
	'''
	
	def private classImports(Entity e) '''
		«FOR f:imports»
			import «f»;
		«ENDFOR»
	'''
	
	def private classJavaDoc(Entity e) '''
		/**
		 * <!-- begin-user-doc -->
		 * This Class contains the representation of the model object '<em><b«e.name»</b></em>'.
		 * For each Entity the generator creates an EntityGEN.java file,
		 * which contains EntityGEN class with every Property and Association for the Entity.
		 * This Java file will be override, with each generation.
		 * The generator creates an Entity.java file, with the Entity class,
		 * which extends form EntityGEN. This file normally (if the Entity is not deleted)
		 * won't be overrided, so this file can be modified between the generations.
		 * 
		 * <p>
		 * This is The Entity.java file. It is created at the first generation, after that,
		 * it can be modified, it won't be overrided, until the Entity is renamed, or the
		 * Entity is deleted.
		 * </p>
		 * <!-- end-user-doc -->
		 *
		 * <p>
		 * The following features are supported:
		 * <ul>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getCache <em>Cache</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getPersistentType <em>Persistent Type</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#isAbstract <em>Abstract</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getName <em>Name</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getSuperType <em>Super Type</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getRelationship <em>Relationship</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getProperty <em>Property</em>}</li>
		 * </ul>
		 * </p>
		 *
		 * @see hu.bme.hellga.onlab.OnlabPackage#getEntity()
		 * @model
		 * @generated
		 */
	'''
	
	def private classAnnotations(Entity e) '''
		«IF entity.persistentType.equals(PersistentType.DB)»
			«dbGen.classAnnotations(e)»
		«ENDIF»
	'''
	
	def private classDeclaration(Entity e) '''
		«IF e.abstract»abstract «ENDIF»public class «e.name.toFirstUpper» extends «e.name.toFirstUpper»GEN{
			
			«e.classBody»
		}
	'''
	
	def private classBody(Entity e) '''
		// Class body, that can be edited
		
		// Constructor
		
		«e.constructor»
		
		«e.constructorCopy»
		
		//
		
		
	'''
	
	def private constructor(Entity e) '''
		protected «e.name.toFirstUpper»(){
			super();
		}
	'''
	
	def private constructorCopy(Entity e) '''
		protected «e.name.toFirstUpper»(«e.name.toFirstUpper» «e.name.toFirstLower»){
			super();
		}
	'''
	
}