package hu.bme.mit.mobilegen.datamodel.generator.serialize

import hu.bme.mit.mobilegen.datamodel.datamodel.Entity
import hu.bme.mit.mobilegen.datamodel.generator.GeneratorConstants
import hu.bme.mit.mobilegen.datamodel.generator.GeneratorData
import hu.bme.mit.mobilegen.datamodel.helper.WrappedEntity
import hu.bme.mit.mobilegen.datamodel.helper.WrappedModel
import hu.bme.mit.mobilegen.datamodel.logger.Log
import java.util.ArrayList
import java.util.List
import org.eclipse.xtext.generator.IFileSystemAccess
import hu.bme.mit.mobilegen.datamodel.generator.DeviceType

class SerializeGenerator {
	
	List<WrappedEntity> wes;
	List<String> imports;
	
	def generateSerializationUnit(WrappedModel wm, IFileSystemAccess fsa){
		wes = wm.serialized;
		collectImports();
		if(wes.size > 0){
			if(GeneratorData.device == DeviceType.Android){
				Log.info("Generating SerializationUnit.java");
				fsa.generateFile("SerializationUnit.java", GeneratorConstants.SRC_GEN_SER, generateSerializationUnit());
			}
		}
	}
	
	def private collectImports(){
		imports = new ArrayList<String>();
		for(WrappedEntity we : wes){
			imports.add(GeneratorData.packageName + ".entities." + we.entity.name);
		}
		
		imports.add("java.io.FileInputStream");
		imports.add("java.io.FileOutputStream");
		imports.add("java.io.IOException");
		imports.add("java.io.ObjectInputStream");
		imports.add("java.io.ObjectOutputStream");
		imports.add("android.content.Context");
	}
	
	def private generateSerializationUnit() '''
		«classHeader»
		
		«packageDeclaration»
		
		«classImports»
		
		//
		«classDeclaration»
	'''
	
	def private classHeader() '''
		//------------------------- Geneated Java Class File For SerializationUnit ---------------------------------
	'''
	
	def private packageDeclaration() '''
		package «GeneratorData.packageName».«GeneratorConstants.SRC_GEN_SER»;
	'''
	
	def private classImports() '''
		«FOR f:imports»
			import «f»;
		«ENDFOR»
	'''
	
	def private classJavaDoc() '''
		/**
		 * <!-- begin-user-doc -->
		 * This Class contains the representation of the model object '<em><b></b></em>'.
		 * For each Entity the generator creates an EntityGEN.java file,
		 * which contains EntityGEN class with every Property and Association for the Entity.
		 * This Java file will be override, with each generation.
		 * The generator creates an Entity.java file, with the Entity class,
		 * which extends form EntityGEN. This file normally (if the Entity is not deleted)
		 * won't be overrided, so this file can be modified between the generations.
		 * 
		 * <p>
		 * This is The Entity.java file. It is created at the first generation, after that,
		 * it can be modified, it won't be overrided, until the Entity is renamed, or the
		 * Entity is deleted.
		 * </p>
		 * <!-- end-user-doc -->
		 *
		 * <p>
		 * The following features are supported:
		 * <ul>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getCache <em>Cache</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getPersistentType <em>Persistent Type</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#isAbstract <em>Abstract</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getName <em>Name</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getSuperType <em>Super Type</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getRelationship <em>Relationship</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getProperty <em>Property</em>}</li>
		 * </ul>
		 * </p>
		 *
		 * @see hu.bme.hellga.onlab.OnlabPackage#getEntity()
		 * @model
		 * @generated
		 */
	'''
	
	def private classDeclaration() '''
		public class SerializationUnit {
			
			«serialization»
		}
	'''
	
	def private serialization() '''
		«FOR f:wes»
			«f.entity.serialize»
			«f.entity.deserialize»
		«ENDFOR»
	'''
	
	def private serialize(Entity e) '''
		//
		public static boolean serialize«e.name.toFirstUpper»(String filename, «e.name.toFirstUpper» «e.name.toFirstLower», Context ctx){
			try {
				FileOutputStream f = ctx.openFileOutput(filename, Context.MODE_PRIVATE);
				ObjectOutputStream out = new ObjectOutputStream(f);
				out.writeObject(«e.name.toFirstLower»);
				out.close();
				f.close();
			}catch(IOException ex) {
				return false;
			}
			return true;
		}
		
	'''
	
	def private deserialize(Entity e) '''
		//
		public static «e.name.toFirstUpper» deserialize«e.name.toFirstUpper»(String filename, Context ctx){
			«e.name.toFirstUpper» «e.name.toFirstLower»;
			try {
				FileInputStream f = ctx.openFileInput(filename);
				ObjectInputStream in = new ObjectInputStream(f);
				«e.name.toFirstLower» = («e.name.toFirstUpper»)in.readObject();
				in.close();
				f.close();
			}catch(IOException ex) {
				return null;
			}
			catch(ClassNotFoundException ex) {
				return null;
			}
			return «e.name.toFirstLower»;
		}
		
	'''
}