package hu.bme.mit.mobilegen.datamodel.generator.sharedpreferences

import hu.bme.mit.mobilegen.datamodel.generator.DeviceType
import hu.bme.mit.mobilegen.datamodel.generator.GeneratorConstants
import hu.bme.mit.mobilegen.datamodel.generator.GeneratorData
import hu.bme.mit.mobilegen.datamodel.helper.WrappedEntity
import hu.bme.mit.mobilegen.datamodel.helper.WrappedModel
import hu.bme.mit.mobilegen.datamodel.logger.Log
import java.util.List
import org.eclipse.xtext.generator.IFileSystemAccess

class SharedPreferencesGenerator {
	
	List<WrappedEntity> wes;
	
	def generateSharedPreferencesUnit(WrappedModel wm, IFileSystemAccess fsa){
		wes = wm.sharedPreferences;
		if(wes.size > 0){
			if(GeneratorData.device == DeviceType.Android){
				Log.info("Generating SharedPreferencesUnit.java");
				fsa.generateFile("SharedPreferencesUnit.java", GeneratorConstants.SRC_GEN_PREFS, generateSharedPreferencesUnit());
			}
		}
	}
	
	def private generateSharedPreferencesUnit() '''
		«classHeader»
		
		«packageDeclaration»
		
		«classImports»
		
		//
		«classDeclaration»
	'''
	
	def private classHeader() '''
		//------------------------- Geneated Java Class File For SerializationUnit ---------------------------------
	'''
	
	def private packageDeclaration() '''
		package «GeneratorData.packageName».«GeneratorConstants.SRC_GEN_PREFS»;
	'''
	
	def private classImports() '''
		import android.content.Context;
		import android.content.SharedPreferences;
		import android.content.SharedPreferences.Editor;
		import java.util.Set;
		import java.util.HashSet;
		import java.util.Map;
		
	'''
	
	def private classJavaDoc() '''
		/**
		 * <!-- begin-user-doc -->
		 * This Class contains the representation of the model object '<em><b></b></em>'.
		 * For each Entity the generator creates an EntityGEN.java file,
		 * which contains EntityGEN class with every Property and Association for the Entity.
		 * This Java file will be override, with each generation.
		 * The generator creates an Entity.java file, with the Entity class,
		 * which extends form EntityGEN. This file normally (if the Entity is not deleted)
		 * won't be overrided, so this file can be modified between the generations.
		 * 
		 * <p>
		 * This is The Entity.java file. It is created at the first generation, after that,
		 * it can be modified, it won't be overrided, until the Entity is renamed, or the
		 * Entity is deleted.
		 * </p>
		 * <!-- end-user-doc -->
		 *
		 * <p>
		 * The following features are supported:
		 * <ul>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getCache <em>Cache</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getPersistentType <em>Persistent Type</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#isAbstract <em>Abstract</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getName <em>Name</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getSuperType <em>Super Type</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getRelationship <em>Relationship</em>}</li>
		 *   <li>{@link hu.bme.hellga.onlab.Entity#getProperty <em>Property</em>}</li>
		 * </ul>
		 * </p>
		 *
		 * @see hu.bme.hellga.onlab.OnlabPackage#getEntity()
		 * @model
		 * @generated
		 */
	'''
	
	def private classDeclaration() '''
		public class SharedPreferencesUnit {
			
			«sharedpreferences»
		}
	'''
	
	def private sharedpreferences() '''
		«putBoolean»
		
		«putFloat»
		
		«putInt»
		
		«putLong»
		
		«putString»
		
		«putStrings»
		
		«getBoolean»
		
		«getFloat»
		
		«getInt»
		
		«getLong»
		
		«getString»
		
		«getStrings»
		
		«getAll»
	'''
	
	def private putBoolean() '''
		//
		public static void putBoolean(String preferenceName, String key, boolean value, Context ctx){
			SharedPreferences sp = ctx.getSharedPreferences(preferenceName, Context.MODE_PRIVATE);
			Editor editor = sp.edit();
			editor.putBoolean(key, value);
			editor.commit();
		}
		
	'''
	
	def private putFloat() '''
		//
		public static void putFloat(String preferenceName, String key, float value, Context ctx){
			SharedPreferences sp = ctx.getSharedPreferences(preferenceName, Context.MODE_PRIVATE);
			Editor editor = sp.edit();
			editor.putFloat(key, value);
			editor.commit();
		}
		
	'''
	
	def private putInt() '''
		//
		public static void putInt(String preferenceName, String key, int value, Context ctx){
			SharedPreferences sp = ctx.getSharedPreferences(preferenceName, Context.MODE_PRIVATE);
			Editor editor = sp.edit();
			editor.putInt(key, value);
			editor.commit();
		}
		
	'''
	
	def private putLong() '''
		//
		public static void putLong(String preferenceName, String key, long value, Context ctx){
			SharedPreferences sp = ctx.getSharedPreferences(preferenceName, Context.MODE_PRIVATE);
			Editor editor = sp.edit();
			editor.putLong(key, value);
			editor.commit();
		}
		
	'''
	
	def private putString() '''
		//
		public static void putString(String preferenceName, String key, String value, Context ctx){
			SharedPreferences sp = ctx.getSharedPreferences(preferenceName, Context.MODE_PRIVATE);
			Editor editor = sp.edit();
			editor.putString(key, value);
			editor.commit();
		}
		
	'''
	
	def private putStrings() '''
		//
		public static void putStringSet(String preferenceName, String key, Set<String> values, Context ctx){
			SharedPreferences sp = ctx.getSharedPreferences(preferenceName, Context.MODE_PRIVATE);
			Editor editor = sp.edit();
			editor.putStringSet(key, values);
			editor.commit();
		}
		
	'''
	
	def private getBoolean() '''
		//
		public static boolean getBoolean(String preferenceName, String key, Context ctx){
			SharedPreferences sp = ctx.getSharedPreferences(preferenceName, Context.MODE_PRIVATE);
			return sp.getBoolean(key, false);
		}
		
	'''
	
	def private getFloat() '''
		//
		public static float getFloat(String preferenceName, String key, Context ctx){
			SharedPreferences sp = ctx.getSharedPreferences(preferenceName, Context.MODE_PRIVATE);
			return sp.getFloat(key, 0);
		}
		
	'''
	
	def private getInt() '''
		//
		public static int getInt(String preferenceName, String key, Context ctx){
			SharedPreferences sp = ctx.getSharedPreferences(preferenceName, Context.MODE_PRIVATE);
			return sp.getInt(key, 0);
		}
		
	'''
	
	def private getLong() '''
		//
		public static long getLong(String preferenceName, String key, Context ctx){
			SharedPreferences sp = ctx.getSharedPreferences(preferenceName, Context.MODE_PRIVATE);
			return sp.getLong(key, 0);
		}
		
	'''
	
	def private getString() '''
		//
		public static String getString(String preferenceName, String key, Context ctx){
			SharedPreferences sp = ctx.getSharedPreferences(preferenceName, Context.MODE_PRIVATE);
			return sp.getString(key, "");
		}
		
	'''
	
	def private getStrings() '''
		//
		public static Set<String> getStringSet(String preferenceName, String key, Context ctx){
			SharedPreferences sp = ctx.getSharedPreferences(preferenceName, Context.MODE_PRIVATE);
			return sp.getStringSet(key, new HashSet<String>());
		}
		
	'''
	
	def private getAll() '''
		//
		public static Map<String, ?> getAll(String preferenceName, Context ctx){
			SharedPreferences sp = ctx.getSharedPreferences(preferenceName, Context.MODE_PRIVATE);
			return sp.getAll();
		}
		
	'''
	
}