package hu.bme.mit.mobilegen.datamodel.preprocessor;

import hu.bme.mit.mobilegen.datamodel.datamodel.Association;
import hu.bme.mit.mobilegen.datamodel.datamodel.ConnectionDirection;
import hu.bme.mit.mobilegen.datamodel.datamodel.ConnectionType;
import hu.bme.mit.mobilegen.datamodel.datamodel.Entity;
import hu.bme.mit.mobilegen.datamodel.generator.GeneratorData;
import hu.bme.mit.mobilegen.datamodel.helper.EntityCloner;
import hu.bme.mit.mobilegen.datamodel.helper.EntityFactory;
import hu.bme.mit.mobilegen.datamodel.helper.WrappedAssociation;
import hu.bme.mit.mobilegen.datamodel.helper.WrappedEntity;
import hu.bme.mit.mobilegen.datamodel.logger.Log;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.util.EList;

public class AnnotationProcessor {
	
	private static List<WrappedAssociation> newAssociations;
	
	public static void processAnnotations(List<WrappedEntity> entities){
		processConnectionDirs(entities);
		processCache(entities);
	}
	
	private static void processConnectionDirs(List<WrappedEntity> entities){
		newAssociations = new ArrayList<WrappedAssociation>();
		
		for(WrappedEntity we : entities){
			Entity entity = we.getEntity();
			EList<Association> associations = entity.getAssociation();
			Iterator<Association> iter = associations.iterator();
			while(iter.hasNext()){
				Association a = iter.next();
				if(a.isDirectionSet()){
					if(processAssociationConnectionDir(a, entity)){
						iter.remove();
					}
				}
			}
		}
		
		for(WrappedAssociation wa : newAssociations){
			wa.getContainer().getAssociation().add(wa.getAssociation());
		}
		
	}
	
	private static boolean processAssociationConnectionDir(Association a, Entity e){
		ConnectionDirection conDir = a.getConnectionDir();
		
		if(conDir == ConnectionDirection.MANY_TO_MANY){
			
			Association newAssoc = EntityFactory.getFactory().createAssociation();
			Entity refEntity = a.getConnectTo();
			
			EntityCloner.cloneAssociationWithoutReferences(a, newAssoc);
			
			newAssoc.setConnectionType(ConnectionType.MANY);
			newAssoc.setName(a.getInverseName());
			newAssoc.setInverseName(a.getName());
			newAssoc.setConnectTo(e);
			
			WrappedAssociation wa = new WrappedAssociation(refEntity, newAssoc);
			newAssociations.add(wa);
			
			Log.info("Processing Many To Many Connection between " + e.getName() + " and " + refEntity.getName());
			
			return false;
			
		}
		else if(conDir == ConnectionDirection.ONE_TO_MANY_BI_DIRECTION){
			
			Association newAssoc = EntityFactory.getFactory().createAssociation();
			Entity refEntity = a.getConnectTo();
			
			EntityCloner.cloneAssociationWithoutReferences(a, newAssoc);
			
			newAssoc.setConnectTo(refEntity);
			
			WrappedAssociation waclone = new WrappedAssociation(e, newAssoc);
			newAssociations.add(waclone);
			
			Entity e2 = a.getConnectTo();
			
			a.setConnectTo(e);
			a.setConnectionType(ConnectionType.ONE);
			a.setName(a.getInverseName());
			a.setInverseName(a.getName());
			
			WrappedAssociation wa = new WrappedAssociation(e2, a);
			newAssociations.add(wa);
			
			Log.info("Processing One To Many Connection between " + e.getName() + " and " + refEntity.getName() + " - New inverse association added");
			
			return true;
			
		}
		else if(conDir == ConnectionDirection.ONE_TO_MANY_INVERSE_DIRECTION){
			
			Entity e2 = a.getConnectTo();
			
			a.setConnectTo(e);
			a.setConnectionType(ConnectionType.ONE);
			a.setName(a.getInverseName());
			a.setInverseName(a.getName());
			
			WrappedAssociation wa = new WrappedAssociation(e2, a);
			newAssociations.add(wa);
			
			Log.info("Processing One To Many Connection between " + e.getName() + " and " + e2.getName());
			
			return true;
			
		}
		else if(conDir == ConnectionDirection.ONE_TO_ONE_BI_DIRECTION){
			
			Association newAssoc = EntityFactory.getFactory().createAssociation();
			Entity refEntity = a.getConnectTo();
			
			EntityCloner.cloneAssociationWithoutReferences(a, newAssoc);
			
			newAssoc.setConnectTo(refEntity);
			
			WrappedAssociation waclone = new WrappedAssociation(e, newAssoc);
			newAssociations.add(waclone);
			
			Entity e2 = a.getConnectTo();
			
			a.setConnectTo(e);
			a.setName(a.getInverseName());
			a.setInverseName(a.getName());
			
			WrappedAssociation wa = new WrappedAssociation(e2, a);
			newAssociations.add(wa);
			
			Log.info("Processing One To One Connection between " + e.getName() + " and " + refEntity.getName() + " - New inverse association added");
			
			return true;
			
		}
		else if(conDir == ConnectionDirection.ONE_TO_ONE_INVERSE_DIRECTION){
			
			Entity e2 = a.getConnectTo();
			
			a.setConnectTo(e);
			a.setName(a.getInverseName());
			a.setInverseName(a.getName());
			
			WrappedAssociation wa = new WrappedAssociation(e2, a);
			newAssociations.add(wa);
			
			Log.info("Processing One To One Connection between " + e.getName() + " and " + e2.getName());
			
			return true;
			
		}
		
		return false;
		
	}
	
	private static void processCache(List<WrappedEntity> entities){
		boolean cache = false;
		for(WrappedEntity we : entities){
			if(we.getEntity().isCache()){
				cache = true;
				
				Log.info("Found DB Cache annotation - DataCache will be created");
			}
		}
		GeneratorData.setCache(cache);
	}
	
}