package hu.bme.mit.mobilegen.datamodel.preprocessor;

import hu.bme.mit.mobilegen.datamodel.datamodel.Association;
import hu.bme.mit.mobilegen.datamodel.datamodel.ConnectionDirection;
import hu.bme.mit.mobilegen.datamodel.datamodel.ConnectionType;
import hu.bme.mit.mobilegen.datamodel.datamodel.Entity;
import hu.bme.mit.mobilegen.datamodel.datamodel.Enumeration;
import hu.bme.mit.mobilegen.datamodel.datamodel.PersistentType;
import hu.bme.mit.mobilegen.datamodel.datamodel.Property;
import hu.bme.mit.mobilegen.datamodel.helper.EntityCloner;
import hu.bme.mit.mobilegen.datamodel.helper.EntityFactory;
import hu.bme.mit.mobilegen.datamodel.helper.WrappedAssociation;
import hu.bme.mit.mobilegen.datamodel.helper.WrappedEntity;
import hu.bme.mit.mobilegen.datamodel.logger.Log;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.TreeIterator;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.ResourceSet;

import com.google.common.collect.Iterators;

public class ModelWrapper {

	private static Map<String, WrappedEntity> entityMap = null;
	private static Map<String, Enumeration> enumMap = null;
	
	/**
	 * 
	 * Processing model
	 * @param: List of model elements
	 * 
	 */
	public static void processModel(ResourceSet rs, List<WrappedEntity> entities, List<Enumeration> enumerations){
		enumerations.addAll(processEnumerations(rs));
		entities.addAll(processEntities(rs));
	}
	
	/**
	 * 
	 * Processing entities
	 * @param: ResourceSet with whole model
	 * 
	 */
	private static List<WrappedEntity> processEntities(ResourceSet rs){
		entityMap = new HashMap<String, WrappedEntity>();
		
		List<Entity> entityList = new ArrayList<Entity>();
		List<Association> associationList = new ArrayList<Association>();
		
		TreeIterator<Notifier> allContents = rs.getAllContents();
		TreeIterator<Notifier> allContents2 = rs.getAllContents();
		TreeIterator<Notifier> allContents3 = rs.getAllContents();
		Iterator<Entity> entities = Iterators.<Entity>filter(allContents, Entity.class);
		Iterator<Entity> entities2 = Iterators.<Entity>filter(allContents2, Entity.class);
		Iterator<Association> associations = Iterators.<Association>filter(allContents3, Association.class);
		
		while(entities.hasNext()){
			entityList.add(entities.next());
		}
		createEntities(entityList);
		
		while(entities2.hasNext()){
			createEntitiySuperClass(entities2.next());
		}
		
		while(associations.hasNext()){
			associationList.add(associations.next());
		}
		createAssociations(associationList);
		
		return new ArrayList<WrappedEntity>(entityMap.values());
	}	
	
	
	/**
	 * 
	 * Create entities
	 * @param: List of entities
	 * 
	 */
	private static void createEntities(List<Entity> eList){
		for(Entity e : eList){
			createEntity(e);
			Log.info("Wrapped Entity: " + e.getName());
		}
	}
	
	/**
	 * 
	 * Create entity
	 * @param: Entity
	 * 
	 */
	private static void createEntity(Entity e){
		Entity entity = EntityFactory.getFactory().createEntity();
		
		EntityCloner.cloneEntityWithoutReferences(e, entity, enumMap);
		
		WrappedEntity we = new WrappedEntity(entity);
		
		for(Property p : entity.getProperty()){
			if(p.isCantBeNull() && p.getConnectionType() == ConnectionType.ONE){
				we.getRequiredProperties().add(p);
			}
		}
		
		we.setResourceURI(e.eResource().getURI().toString());
		
		we.setPersistType(entity.getPersistentType());
		
		entityMap.put(entity.getName(), we);
	}
	
	/**
	 * 
	 * Create entity superclass
	 * @param: Entitiy
	 * 
	 */
	private static void createEntitiySuperClass(Entity e){
		if(e.isExtends()){
			
			Entity e2 = e.getSuperClass();
			
			WrappedEntity we1 = entityMap.get(e.getName());
			WrappedEntity we2 = entityMap.get(e2.getName());
			Entity e3 = we1.getEntity();
			Entity e4 = we2.getEntity();
			
			e3.setSuperClass(e4);
			e3.setExtends(true);
			we1.setSuperClass(we2);
			we1.setHasSuperClass(true);
			we2.getSubClasses().add(we1);
			we2.setHasSubClass(true);
			
			Log.info("Wrapped inheritence: " + e.getName() + " extends from " + e2.getName());
		}
	}
	
	/**
	 * 
	 * Create associations
	 * @param: List of associations
	 * 
	 */
	private static void createAssociations(List<Association> aList){
		for(Association a : aList){
			createAssociation(a);
		}
	}
	
	/**
	 * 
	 * Create association
	 * @param: Association
	 * 
	 */
	private static void createAssociation(Association a){
		EObject eo = a.eContainer();
		
		if(eo instanceof Entity){
			Entity e1 = (Entity)eo;
			Entity e2 = a.getConnectTo();
			Association assoc = EntityFactory.getFactory().createAssociation();
			
			EntityCloner.cloneAssociationWithoutReferences(a, assoc);
			
			if(!assoc.isDirectionSet()){
				if(assoc.getConnectionType() == ConnectionType.ONE){
					assoc.setDirectionSet(true);
					assoc.setConnectionDir(ConnectionDirection.ONE_TO_ONE_UNI_DIRECTION);
					Log.info("Association between: " + e1.getName() + " and " + e2.getName() + " set to default " + assoc.getConnectionDir());
				}
				else{
					assoc.setDirectionSet(true);
					assoc.setConnectionDir(ConnectionDirection.ONE_TO_MANY_UNI_DIRECTION);
					Log.info("Association between: " + e1.getName() + " and " + e2.getName() + " set to default " + assoc.getConnectionDir());
				}
			}
			
			WrappedEntity we1 = entityMap.get(e1.getName());
			WrappedEntity we2 = entityMap.get(e2.getName());
			Entity e3 = we1.getEntity();
			Entity e4 = we2.getEntity();
			
			if(assoc.isContainment()){
				we1.setHasContainment(true);
			}
			
			if(e4.getPersistentType() == PersistentType.DB && !assoc.isTransient() && !assoc.isStatic()){
				if(assoc.isDirectionSet()){
					if(assoc.getConnectionType() == ConnectionType.MANY){
						if(assoc.getConnectionDir() == ConnectionDirection.MANY_TO_MANY){
							
						}
						else{
							if(assoc.isContainment()){
								we2.getDbManyToOneForeignKeysCont().add(assoc);
								we1.getDbManyToOneForeignKeysContRev().add(e4);
							}
							else{
								we2.getDbManyToOneForeignKeys().add(assoc);
								we1.getDbManyToOneForeignKeysRev().add(new WrappedAssociation(e4, assoc));
							}
							
						}
					}
					else{
						if(assoc.getConnectionDir() == ConnectionDirection.ONE_TO_ONE_UNI_DIRECTION || assoc.getConnectionDir() == ConnectionDirection.ONE_TO_ONE_BI_DIRECTION){
							if(assoc.isContainment()){
								we1.getDbOneToOneForeignKeysCont().add(e4);
								we2.getDbOneToOneForeignKeysContRev().add(e3);
							}
							else{
								we1.getDbOneToOneForeignKeys().add(e4);
								we2.getDbOneToOneForeignKeysRev().add(e3);
							}
						}
						else{
							if(assoc.isContainment()){
								we2.getDbOneToOneForeignKeysContInv().add(assoc);
								we1.getDbOneToOneForeignKeysContRevInv().add(e4);
							}
							else{
								we2.getDbOneToOneForeignKeysInv().add(assoc);
								we1.getDbOneToOneForeignKeysRevInv().add(new WrappedAssociation(e4, assoc));
							}
						}
					}
				}
			}
			
			assoc.setConnectTo(e4);
			
			EList<Association> rs = e3.getAssociation();
			rs.add(assoc);
			
			Log.info("Wrapped association between: " + e3.getName() + " and " + e4.getName() + " - " + assoc.getConnectionDir());
		}
		
		
	}
	
	/**
	 * 
	 * Processing enumerations
	 * @param: ResourceSet with whole model
	 * 
	 */
	private static List<Enumeration> processEnumerations(ResourceSet rs){
		enumMap = new HashMap<String, Enumeration>();
		
		TreeIterator<Notifier> allCont = rs.getAllContents();
		Iterator<Enumeration> entities = Iterators.<Enumeration>filter(allCont, Enumeration.class);
		
		while(entities.hasNext()){
			Enumeration from = entities.next();
			Enumeration to = EntityFactory.getFactory().createEnumeration();
			EntityCloner.cloneEnumeration(from, to);
			enumMap.put(to.getName(), to);
			
			Log.info("Wrapped enumeration: " + to.getName());
		}
		
		return new ArrayList<Enumeration>(enumMap.values());
	}
	
}
