package hu.bme.mit.mobilegen.datamodel.preprocessor.database.ormlite;

import hu.bme.mit.mobilegen.datamodel.datamodel.Association;
import hu.bme.mit.mobilegen.datamodel.datamodel.ConnectionDirection;
import hu.bme.mit.mobilegen.datamodel.datamodel.ConnectionType;
import hu.bme.mit.mobilegen.datamodel.datamodel.Entity;
import hu.bme.mit.mobilegen.datamodel.datamodel.PersistentType;
import hu.bme.mit.mobilegen.datamodel.helper.EntityCloner;
import hu.bme.mit.mobilegen.datamodel.helper.EntityFactory;
import hu.bme.mit.mobilegen.datamodel.helper.GeneratorHelper;
import hu.bme.mit.mobilegen.datamodel.helper.WrappedAssociation;
import hu.bme.mit.mobilegen.datamodel.helper.WrappedEntity;
import hu.bme.mit.mobilegen.datamodel.logger.Log;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.util.EList;

public class ORMLiteProcessor {
	
	private static List<WrappedAssociation> newAssociations;

	public static void processDBEntities(List<WrappedEntity> all, List<WrappedEntity> db){
		processForiegnCollections(db);
		processManyToManyConnections(all, db);
	}
	
	private static void processForiegnCollections(List<WrappedEntity> entities){
		newAssociations = new ArrayList<WrappedAssociation>();
		
		for(WrappedEntity we : entities){
			Entity entity = we.getEntity();
			EList<Association> associations = entity.getAssociation();
			Iterator<Association> iter = associations.iterator();
			while(iter.hasNext()){
				Association a = iter.next();
				if(a.isDirectionSet()){
					if(processForiegnCollection(a, entity)){
						iter.remove();
					}
				}
			}
		}
		
		for(WrappedAssociation wa : newAssociations){
			wa.getContainer().getAssociation().add(wa.getAssociation());
		}
		
	}
	
	private static boolean processForiegnCollection(Association a, Entity e){
		
		if(a.getConnectionDir() == ConnectionDirection.ONE_TO_MANY_UNI_DIRECTION){
					
			a.setConnectionDir(ConnectionDirection.ONE_TO_MANY_BI_DIRECTION);
			a.setInverseNameSet(true);
			a.setInverseName(a.getName() + "Inverse");
			Association newAssoc = EntityFactory.getFactory().createAssociation();
			Entity refEntity = a.getConnectTo();
					
			EntityCloner.cloneAssociationWithoutReferences(a, newAssoc);
					
			newAssoc.setConnectTo(refEntity);
					
			WrappedAssociation waclone = new WrappedAssociation(e, newAssoc);
			newAssociations.add(waclone);
					
			Entity e2 = a.getConnectTo();
					
			a.setConnectTo(e);
			a.setConnectionType(ConnectionType.ONE);
			a.setName(a.getInverseName());
			a.setInverseName(a.getName());
					
			WrappedAssociation wa = new WrappedAssociation(e2, a);
			newAssociations.add(wa);
			
			Log.warning("ORMLite does not supprt One To Many Uni Direction Connections, so it was changed to a Bi Direction Connection between " + e.getName() + " and " + refEntity.getName());
					
			return true;
					
		}
		return false;
	}
	
	private static void processManyToManyConnections(List<WrappedEntity> all, List<WrappedEntity> db){
		List<WrappedEntity> newEntities = new ArrayList<WrappedEntity>();
		
		for(WrappedEntity we : db){
			for(Association a : we.getEntity().getAssociation()){
				if(a.getConnectionDir() == ConnectionDirection.MANY_TO_MANY){
					
					Entity e1 = we.getEntity();
					Entity e2 = a.getConnectTo();
					
					String name = e1.getName() + a.getName() + "Join" + e2.getName() + a.getInverseName();
					a.setTransient(true);
					
					boolean exists = false;
					for(WrappedEntity w : newEntities){
						String[] s1 = w.getEntity().getName().split("Join");
						String[] s2 = name.split("Join");
						if(s1[0].equals(s2[1])){
							exists = true;
							we.getDbManyToManyJoinTable().add(w.getEntity());
							we.setHasJoinTable(true);
							we.getDbManyToOneForeignKeysContRev().add(w.getEntity());
						}
						else if(w.getEntity().equals(name)){
							name = name + "2";
						}
					}
					if(!exists){
						
						Entity e = EntityFactory.getFactory().createEntity();
						e.setName(name);
						e.setPersistentType(PersistentType.DB);
						
						Association a1 = EntityFactory.getFactory().createAssociation();
						
						Association a2 = EntityFactory.getFactory().createAssociation();
						
						a1.setConnectionType(ConnectionType.ONE);
						a1.setConnectionDir(ConnectionDirection.ONE_TO_MANY_INVERSE_DIRECTION);
						a1.setConnectTo(e1);
						a1.setName(a.getInverseName());
						
						a2.setConnectionType(ConnectionType.ONE);
						a2.setConnectionDir(ConnectionDirection.ONE_TO_MANY_INVERSE_DIRECTION);
						a2.setConnectTo(e2);
						a2.setName(a.getName());
						
						e.getAssociation().add(a1);
						e.getAssociation().add(a2);
						
						we.getDbManyToOneForeignKeysContRev().add(e);
						
						WrappedEntity newwe = new WrappedEntity(e);
						newwe.getDbManyToOneForeignKeysCont().add(a1);
						newwe.getDbManyToOneForeignKeysCont().add(a2);
						newwe.setGenHelp(GeneratorHelper.JoinClass);
						newEntities.add(newwe);
						
						we.getDbManyToManyJoinTable().add(e);
						we.setHasJoinTable(true);
						
						Log.warning("ORMLite does not supprt Many To Many Uni Direction Connections, so a new Join Entity (" + e.getName() + ") has been created between " + e1.getName() + " and " + e2.getName());
						
					}
				}
			}
		}
		
		for(WrappedEntity we : newEntities){
			all.add(we);
			db.add(we);
		}
		
	}
	
}
