package hu.bme.mit.mobilegen.datamodel.validation;

import hu.bme.mit.mobilegen.datamodel.datamodel.Association;
import hu.bme.mit.mobilegen.datamodel.datamodel.ConnectionDirection;
import hu.bme.mit.mobilegen.datamodel.datamodel.ConnectionType;
import hu.bme.mit.mobilegen.datamodel.datamodel.Content;
import hu.bme.mit.mobilegen.datamodel.datamodel.DatamodelPackage;
import hu.bme.mit.mobilegen.datamodel.datamodel.Element;
import hu.bme.mit.mobilegen.datamodel.datamodel.Entity;
import hu.bme.mit.mobilegen.datamodel.datamodel.Enumeration;
import hu.bme.mit.mobilegen.datamodel.datamodel.PersistentType;
import hu.bme.mit.mobilegen.datamodel.datamodel.Property;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.TreeIterator;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.xtext.resource.IResourceDescription;
import org.eclipse.xtext.resource.IResourceDescriptions;
import org.eclipse.xtext.resource.IResourceServiceProvider;
import org.eclipse.xtext.resource.impl.ResourceDescriptionsProvider;
import org.eclipse.xtext.validation.Check;
import org.eclipse.xtext.validation.CheckType;

import com.google.common.collect.Iterators;
import com.google.inject.Inject;

public class DatamodelValidatorJava extends DatamodelValidator{
	
	@Inject
	protected ResourceDescriptionsProvider resourceDescriptionsProvider;
	
	@Inject
	private IResourceServiceProvider resourceServiceProvider;
	
	@Check(CheckType.NORMAL)
	public void checkNameLengthElement(Element e){
		if(e.getName().length() < 2){
			error("Names must contain at least 2 characters!", DatamodelPackage.Literals.ELEMENT__NAME);
		}
	}
	
	@Check(CheckType.NORMAL)
	public void checkNameLengthProperty(Property p){
		if(p.getName().length() < 2){
			error("Names must contain at least 2 characters!", DatamodelPackage.Literals.PROPERTY__NAME);
		}
	}
	
	@Check(CheckType.NORMAL)
	public void checkNameLengthAssociation(Association a){
		if(a.getName().length() < 2){
			error("Names must contain at least 2 characters!", DatamodelPackage.Literals.ASSOCIATION__NAME);
		}
	}

	@Check(CheckType.FAST)
	public void checkURI(Content c){
		
		for(String s : ReservedStrings.getReservedEndings()){
			if(c.getName().toLowerCase().endsWith(s.toLowerCase())){
				error("URI can not end with " + s + "! It's a reserved keyword for the generator!", DatamodelPackage.Literals.ELEMENT__NAME);
			}
		}
		
		for(String s : ReservedStrings.getReservedStrings()){
			if(c.getName().toLowerCase().equals(s.toLowerCase())){
				error("URI can not be " + s + "! It's a reserved keyword for the Java language!", DatamodelPackage.Literals.ELEMENT__NAME);
			}
		}
		
		for(String s : ReservedStrings.getForbiddenStrings()){
			if(c.getName().toLowerCase().contains(s.toLowerCase())){
				error("URI can not contain " + s + "! It's a forbidden String!", DatamodelPackage.Literals.ELEMENT__NAME);
			}
		}
		
	}
	
	@Check(CheckType.FAST)
	public void checkEntityName(Element e){
		
		for(String s : ReservedStrings.getReservedEndings()){
			if(e.getName().toLowerCase().endsWith(s.toLowerCase())){
				error("Entity name can not end with " + s + "! It's a reserved keyword for the generator!", DatamodelPackage.Literals.ELEMENT__NAME);
			}
		}
		
		for(String s : ReservedStrings.getReservedStrings()){
			if(e.getName().toLowerCase().equals(s.toLowerCase())){
				error("Entity name can not be " + s + "! It's a reserved keyword for the Java language!", DatamodelPackage.Literals.ELEMENT__NAME);
			}
		}
		
		for(String s : ReservedStrings.getForbiddenStrings()){
			if(e.getName().toLowerCase().contains(s.toLowerCase())){
				error("Entity name can not contain " + s + "! It's a forbidden String!", DatamodelPackage.Literals.ELEMENT__NAME);
			}
		}
		
	}
	
	@Check(CheckType.FAST)
	public void checkEnumerationName(Enumeration e){
		
		for(String s : ReservedStrings.getReservedEndings()){
			if(e.getName().toLowerCase().endsWith(s.toLowerCase())){
				error("Enumeration name can not end with " + s + "! It's a reserved keyword for the generator!", DatamodelPackage.Literals.ELEMENT__NAME);
			}
		}
		
		for(String s : ReservedStrings.getReservedStrings()){
			if(e.getName().toLowerCase().equals(s.toLowerCase())){
				error("Enumeration name can not be " + s + "! It's a reserved keyword for the Java language!", DatamodelPackage.Literals.ELEMENT__NAME);
			}
		}
		
		for(String s : ReservedStrings.getForbiddenStrings()){
			if(e.getName().toLowerCase().contains(s.toLowerCase())){
				error("Enumeration name can not contain " + s + "! It's a forbidden String!", DatamodelPackage.Literals.ELEMENT__NAME);
			}
		}
		
	}
	
	@Check(CheckType.FAST)
	public void checkEnumerationNames(Enumeration e){
		
		int count = 0;
		for(String en : e.getEnumeration()){
			
			if (en.equals("")) {
				error("Enumeration content name must contain at least one character!", DatamodelPackage.Literals.ENUMERATION__ENUMERATION, count++);
			}
			else if (en.contains(" ")) {
				error("Enumeration content name mustn't contain whitespace!", DatamodelPackage.Literals.ENUMERATION__ENUMERATION, count++);
			}
			
			for(String s : ReservedStrings.getReservedEndings()){
				if(en.toLowerCase().endsWith(s.toLowerCase())){
					error("Enumeration content can not end with " + s + "! It's a reserved keyword for the generator!", DatamodelPackage.Literals.ENUMERATION__ENUMERATION, count++);
				}
			}
			
			for(String s : ReservedStrings.getReservedStrings()){
				if(en.toLowerCase().equals(s.toLowerCase())){
					error("Enumeration content can not be " + s + "! It's a reserved keyword for the Java language!", DatamodelPackage.Literals.ENUMERATION__ENUMERATION, count++);
				}
			}
			
			for(String s : ReservedStrings.getForbiddenStrings()){
				if(e.getName().toLowerCase().contains(s.toLowerCase())){
					error("Enumeration content can not contain " + s + "! It's a forbidden String!", DatamodelPackage.Literals.ENUMERATION__ENUMERATION, count++);
				}
			}
			
			count++;
		}
		
	}
	
	@Check(CheckType.FAST)
	public void checkPropertyName(Property p) {
		
		if (p.getName().equals("")) {
			error("Property name must contain at least one character!", DatamodelPackage.Literals.PROPERTY__NAME);
		}
		else if (p.getName().contains(" ")) {
			error("Property name mustn't contain whitespace!", DatamodelPackage.Literals.PROPERTY__NAME);
		}
		
		for(String s : ReservedStrings.getReservedEndings()){
			if(p.getName().toLowerCase().endsWith(s.toLowerCase())){
				error("Property name can not end with " + s + "! It's a reserved keyword for the generator!", DatamodelPackage.Literals.PROPERTY__NAME);
			}
		}
		
		for(String s : ReservedStrings.getReservedStrings()){
			if(p.getName().toLowerCase().equals(s.toLowerCase())){
				error("Property name can not be " + s + "! It's a reserved keyword for the Java language!", DatamodelPackage.Literals.PROPERTY__NAME);
			}
		}
		
		for(String s : ReservedStrings.getForbiddenStrings()){
			if(p.getName().toLowerCase().contains(s.toLowerCase())){
				error("Property name can not contain " + s + "! It's a forbidden String!", DatamodelPackage.Literals.PROPERTY__NAME);
			}
		}
		
	}
	
	@Check(CheckType.FAST)
	public void checkAssociationName(Association a) {
		
		if (a.getName().equals("")) {
			error("Association name must contain at least one character!", DatamodelPackage.Literals.ASSOCIATION__NAME);
		}
		else if (a.getName().contains(" ")) {
			error("Association name mustn't contain whitespace!", DatamodelPackage.Literals.ASSOCIATION__NAME);
		}
		
		for(String s : ReservedStrings.getReservedEndings()){
			if(a.getName().toLowerCase().endsWith(s.toLowerCase())){
				error("Association name can not end with " + s + "! It's a reserved keyword for the generator!", DatamodelPackage.Literals.ASSOCIATION__NAME);
			}
		}
		
		for(String s : ReservedStrings.getReservedStrings()){
			if(a.getName().toLowerCase().equals(s.toLowerCase())){
				error("Association name can not be " + s + "! It's a reserved keyword for the Java language!", DatamodelPackage.Literals.ASSOCIATION__NAME);
			}
		}
		
		for(String s : ReservedStrings.getForbiddenStrings()){
			if(a.getName().toLowerCase().contains(s.toLowerCase())){
				error("Association name can not contain " + s + "! It's a forbidden String!", DatamodelPackage.Literals.ASSOCIATION__NAME);
			}
		}
		
	}
	
	@Check(CheckType.FAST)
	public void checkInverseReferenceName(Association a) {
		
		if(a.isInverseNameSet()){
			
			if (a.getInverseName().equals("")) {
				error("Inverse reference name must contain at least one character!", DatamodelPackage.Literals.ASSOCIATION__INVERSE_NAME);
			}
			else if (a.getInverseName().contains(" ")) {
				error("Inverse reference name mustn't contain whitespace!", DatamodelPackage.Literals.ASSOCIATION__INVERSE_NAME);
			}
			
			for(String s : ReservedStrings.getReservedEndings()){
				if(a.getInverseName().toLowerCase().endsWith(s.toLowerCase())){
					error("Inverse reference name can not end with " + s + "! It's a reserved keyword for the generator!", DatamodelPackage.Literals.ASSOCIATION__INVERSE_NAME);
				}
			}
			
			for(String s : ReservedStrings.getReservedStrings()){
				if(a.getInverseName().toLowerCase().equals(s.toLowerCase())){
					error("Inverse reference name can not be " + s + "! It's a reserved keyword for the Java language!", DatamodelPackage.Literals.ASSOCIATION__INVERSE_NAME);
				}
			}
			
			for(String s : ReservedStrings.getForbiddenStrings()){
				if(a.getName().toLowerCase().contains(s.toLowerCase())){
					error("Inverse reference name can not contain " + s + "! It's a forbidden String!", DatamodelPackage.Literals.ASSOCIATION__INVERSE_NAME);
				}
			}
			
		}
		
	}
	
	@Check(CheckType.FAST)
	public void checkSuperTypeFast(Entity e){
		if(e.isExtends()){
			if(e.getSuperClass().equals(e)){
				error("You can't extend from own class!", DatamodelPackage.Literals.ENTITY__SUPER_CLASS);
			}
		}
	}
	
	@Check(CheckType.NORMAL)
	public void checkSuperTypeNormal(Entity e){
		if(e.isExtends()){
			if(!e.getSuperClass().equals(e)){
				Entity superT = e.getSuperClass();
				while(superT != null){
					if(superT.equals(e)){
						error("You can't extend from a class that has this class as a superclass!", DatamodelPackage.Literals.ENTITY__SUPER_CLASS);
						break;
					}
					superT = superT.getSuperClass();
				}
			}
		}
	}
	
	@Check(CheckType.FAST)
	public void checkFieldNames(Entity e){
		EList<Property> props = e.getProperty();
		EList<Association> as = e.getAssociation();
		Map<String, EObject> eObjs = new HashMap<String, EObject>();
		Set<EObject> objErrors = new HashSet<EObject>();
		
		for(Property p : props){
			if(eObjs.containsKey(p.getName())){
				objErrors.add(p);
				objErrors.add(eObjs.get(p.getName()));
			}
			else{
				eObjs.put(p.getName(), p);
			}
		}
		
		for(Association a : as){
			if(eObjs.containsKey(a.getName())){
				objErrors.add(a);
				objErrors.add(eObjs.get(a.getName()));
			}
			else{
				eObjs.put(a.getName(), a);
			}
		}
		
		for(EObject eo : objErrors){
			if(eo instanceof Property){
				error("Duplicate local filed!", (Property)eo, DatamodelPackage.Literals.PROPERTY__NAME);
			}
			else{
				error("Duplicate local filed!", (Association)eo, DatamodelPackage.Literals.ASSOCIATION__NAME);
			}
		}
		
	}
	
	@Check(CheckType.FAST)
	public void checkPropertyAnnotations(Entity e){
		EList<Property> props = e.getProperty();
		for(Property p : props){
			
			if(p.isTransient()){
				if(e.getPersistentType() == PersistentType.NONE){
					warning("This annotation has no effect with a non-persistent Entity!", p, DatamodelPackage.Literals.PROPERTY__TRANSIENT);
				}
			}
			
			if(p.isUnique()){
				if(e.getPersistentType() != PersistentType.DB){
					warning("This annotation has no effect if the Entity is not persisted into DB!", p, DatamodelPackage.Literals.PROPERTY__UNIQUE);
				}
			}
			
			if(p.isIndexed()){
				if(e.getPersistentType() != PersistentType.DB){
					warning("This annotation has no effect if the Entity is not persisted into DB!", p, DatamodelPackage.Literals.PROPERTY__INDEXED);
				}
			}
			
			if(p.isCantBeNull() && p.isStatic()){
				if(e.getPersistentType() != PersistentType.DB){
					error("This annotation is inconsistent with the @Static annotation!", p, DatamodelPackage.Literals.PROPERTY__CANT_BE_NULL);
					error("This annotation is inconsistent with the @CantBeNull annotation!", p, DatamodelPackage.Literals.PROPERTY__STATIC);
				}
			}
			
			if(p.isTransient() && p.isStatic()){
				if(e.getPersistentType() != PersistentType.DB){
					warning("This annotation is inconsistent with the @Static annotation!", p, DatamodelPackage.Literals.PROPERTY__TRANSIENT);
					warning("This annotation is inconsistent with the @Transient annotation!", p, DatamodelPackage.Literals.PROPERTY__STATIC);
				}
			}
			
			if(p.isCantBeNull() && p.getConnectionType() == ConnectionType.MANY){
				warning("This annotation has no effect with many connections!", p, DatamodelPackage.Literals.PROPERTY__CANT_BE_NULL);
			}
			
		}
	}
	
	@Check(CheckType.FAST)
	public void checkEntityAnnotationsFast(Entity e){
		if(e.isCache()){
			if(e.getPersistentType() != PersistentType.DB){
				warning("This annotation has no effect if the Entity is not persisted into DB!", e, DatamodelPackage.Literals.ENTITY__CACHE);
			}
		}
	}
	
	@Check(CheckType.NORMAL)
	public void checkEntityAnnotationsNormal(Entity e){
		if(e.isOnlyInContainment()){
			if(e.getPersistentType() != PersistentType.DB){
				error("This annotation has no effect if the Entity is not persisted into DB!", e, DatamodelPackage.Literals.ENTITY__ONLY_IN_CONTAINMENT);
			}
			else{
				ResourceSet rs = findOtherContent(e.eResource());
				TreeIterator<Notifier> content = rs.getAllContents();
				Iterator<Entity> entities = Iterators.<Entity>filter(content, Entity.class);
				
				boolean error = true;
				while(entities.hasNext()){
					Entity ent = entities.next();
					for(Association a : ent.getAssociation()){
						if(a.getConnectTo().getName().equals(e.getName()) && a.isContainment()){
							error = false;
							break;
						}
					}
					if(! error){
						break;
					}
				}
				
				if(error){
					error("This annotation can only be placed to an Entity that is contained by another entity!", DatamodelPackage.Literals.ENTITY__ONLY_IN_CONTAINMENT);
				}
			}
		}
	}
	
	@Check(CheckType.FAST)
	public void checkPropertyConnectionType(Entity e){
		EList<Property> props = e.getProperty();
		for(Property p : props){
			
			if(p.getConnectionType() == ConnectionType.MANY){
				if(e.getPersistentType() == PersistentType.DB){
					error("If an Entity is persisted into DB, it can't have Many Connection Properties, it has to be placed in an other Entity, in order to make Association with it!", p, DatamodelPackage.Literals.PROPERTY__CONNECTION_TYPE);
				}
			}
			
		}
	}
	
	@Check(CheckType.FAST)
	public void checkAssociationAnnotations(Entity e){
		EList<Association> as = e.getAssociation();
		for(Association a : as){
			
			if(a.isTransient()){
				if(e.getPersistentType() == PersistentType.NONE){
					warning("This annotation has no effect with a non-persistent Entity!", a, DatamodelPackage.Literals.ASSOCIATION__TRANSIENT);
				}
			}
			
			if(a.isContainment()){
				if(a.isDirectionSet() && a.getConnectionDir() == ConnectionDirection.MANY_TO_MANY){
					error("In a Many To Many connection can't be containment hierarchy!", a, DatamodelPackage.Literals.ASSOCIATION__CONTAINMENT);
				}
			}
			
			if(a.isContainment()){
				if(e.getPersistentType() != PersistentType.DB){
					warning("This annotation has no effect if the Entity is not persisted into DB!", a, DatamodelPackage.Literals.ASSOCIATION__CONTAINMENT);
				}
			}
			
			if(a.isDirectionSet()){
				ConnectionDirection dir = a.getConnectionDir();
				if(dir == ConnectionDirection.MANY_TO_MANY || dir == ConnectionDirection.ONE_TO_MANY_BI_DIRECTION || dir == ConnectionDirection.ONE_TO_MANY_INVERSE_DIRECTION | dir == ConnectionDirection.ONE_TO_MANY_UNI_DIRECTION){
					if(a.getConnectionType() == ConnectionType.ONE){
						error("Annotation is inconsistent with Connection Type - One, this annotation can only be used with Many connections!", a, DatamodelPackage.Literals.ASSOCIATION__CONNECTION_DIR);
					}
				}
				else if(dir == ConnectionDirection.ONE_TO_ONE_BI_DIRECTION || dir == ConnectionDirection.ONE_TO_ONE_UNI_DIRECTION || dir == ConnectionDirection.ONE_TO_ONE_INVERSE_DIRECTION){
					if(a.getConnectionType() == ConnectionType.MANY){
						error("Annotation is inconsistent with Connection Type - Many, this annotation can only be used with One connections!", a, DatamodelPackage.Literals.ASSOCIATION__CONNECTION_DIR);
					}
				}
				
				if(dir == ConnectionDirection.MANY_TO_MANY && e.getName().equals(a.getConnectTo().getName())){
					error("There cant be Many To many connection between the same entities!", a, DatamodelPackage.Literals.ASSOCIATION__CONNECTION_DIR);
				}
			}
			
			if(a.isCantBeNull() && a.isStatic()){
				if(e.getPersistentType() != PersistentType.DB){
					error("This annotation is inconsistent with the @Static annotation!", a, DatamodelPackage.Literals.ASSOCIATION__CANT_BE_NULL);
					error("This annotation is inconsistent with the @CantBeNull annotation!", a, DatamodelPackage.Literals.ASSOCIATION__STATIC);
				}
			}
			
			if(a.isTransient() && a.isStatic()){
				if(e.getPersistentType() != PersistentType.DB){
					warning("This annotation is inconsistent with the @Static annotation!", a, DatamodelPackage.Literals.ASSOCIATION__TRANSIENT);
					warning("This annotation is inconsistent with the @Transient annotation!", a, DatamodelPackage.Literals.ASSOCIATION__STATIC);
				}
			}
			
			if(a.isContainment() && a.isStatic()){
				error("This annotation is inconsistent with the @Static annotation!", a, DatamodelPackage.Literals.ASSOCIATION__CONTAINMENT);
				error("This annotation is inconsistent with the @Containment annotation!", a, DatamodelPackage.Literals.ASSOCIATION__STATIC);
			}
			
			if(a.isContainment() && a.isTransient()){
				error("This annotation is inconsistent with the @Transient annotation!", a, DatamodelPackage.Literals.ASSOCIATION__CONTAINMENT);
				error("This annotation is inconsistent with the @Containment annotation!", a, DatamodelPackage.Literals.ASSOCIATION__TRANSIENT);
			}
			
		}
	}
	
	@Check(CheckType.FAST)
	public void checkInverseReferenceNamesFast(Entity e){
		EList<Association> as = e.getAssociation();
		for(Association a : as){
			
			if(a.isInverseNameSet()){
				ConnectionDirection conDir = a.getConnectionDir();
				if(conDir == ConnectionDirection.ONE_TO_MANY_UNI_DIRECTION || conDir == ConnectionDirection.ONE_TO_ONE_UNI_DIRECTION){
					warning("This annotation has no effect if the connection direction hasn't got an inverse reference!", a, DatamodelPackage.Literals.ASSOCIATION__INVERSE_NAME_SET);
				}
			}
			
		}
	}
	
	@Check(CheckType.NORMAL)
	public void checkInverseReferenceNamesNormal(Entity e){
		EList<Association> as = e.getAssociation();
		for(Association a : as){
			
			ConnectionDirection conDir = a.getConnectionDir();
			if(conDir == ConnectionDirection.MANY_TO_MANY || conDir == ConnectionDirection.ONE_TO_MANY_BI_DIRECTION || conDir == ConnectionDirection.ONE_TO_MANY_INVERSE_DIRECTION || conDir == ConnectionDirection.ONE_TO_ONE_BI_DIRECTION || conDir == ConnectionDirection.ONE_TO_ONE_INVERSE_DIRECTION){
				
				if(!a.isInverseNameSet()){
					error("If you use a connection direction that has inverse reference, you must set the @InverseReferenceName annotation in order to set the name of the inverse reference!", a, DatamodelPackage.Literals.ASSOCIATION__CONNECTION_DIR);
				}
				else{
					String inverseName = a.getInverseName();
					Entity connectTo = a.getConnectTo();
					
					boolean error = false;
					
					for(Property p : connectTo.getProperty()){
						if(p.getName().equals(inverseName)){
							error("Inverse name of an association must be a unique field in the referenced class!", a, DatamodelPackage.Literals.ASSOCIATION__INVERSE_NAME);
							error = true;
							break;
						}
					}
					
					if(error){
						break;
					}
					
					for(Association assoc : connectTo.getAssociation()){
						if(assoc.getName().equals(inverseName)){
							error("Inverse name of an association must be a unique field in the referenced class!", a, DatamodelPackage.Literals.ASSOCIATION__INVERSE_NAME);
							break;
						}
					}
				}
			}
		}
	}
	
	@Check(CheckType.NORMAL)
	public void checkRelationshipConnectTo(Entity e){
		EList<Association> as = e.getAssociation();
		for(Association a : as){
			
			if(e.getPersistentType() == PersistentType.DB){
				if(a.getConnectTo().getPersistentType() != PersistentType.DB && !a.isTransient()){
					error("DB persisted Entities can only hold reference to DB persisted Entities or this association must be Transient!", a, DatamodelPackage.Literals.ASSOCIATION__CONNECT_TO);
				}
			}
			
			if(e.getPersistentType() != PersistentType.DB){
				if(a.getConnectTo().getPersistentType() == PersistentType.DB){
					error("None persisted Entities can not hold references to DB persisted Entities!", a, DatamodelPackage.Literals.ASSOCIATION__CONNECT_TO);
				}
			}
			
			if(e.getPersistentType() == PersistentType.SERIALIZE){
				if(a.getConnectTo().getPersistentType() != PersistentType.SERIALIZE && !a.isTransient()){
					error("Serializable Entities can only hold reference to Serializable Entities or this association must be Transient!", a, DatamodelPackage.Literals.ASSOCIATION__CONNECT_TO);
				}
			}
			
		}
	}
	
	@Check(CheckType.NORMAL)
	public void checkAbstractEntity(Entity e){
		if(e.isAbstract()){
			ResourceSet rs = findOtherContent(e.eResource());
			TreeIterator<Notifier> content = rs.getAllContents();
			Iterator<Entity> entities = Iterators.<Entity>filter(content, Entity.class);
			
			boolean error = true;
			while(entities.hasNext()){
				Entity ent = entities.next();
				if(ent.isExtends()){
					Entity superClass = ent.getSuperClass();
					if(superClass.getName().equals(e.getName())){
						error = false;
						break;
					}
				}
			}
			
			if(error){
				warning("This abstract class hasn't got any child class!", DatamodelPackage.Literals.ENTITY__ABSTRACT);
			}
			
		}
	}
	
	@Check(CheckType.FAST)
	public void checkElementNames(Element e){
		ResourceSet rs = findOtherContent(e.eResource());
		TreeIterator<Notifier> content = rs.getAllContents();
		Iterator<Element> elements = Iterators.<Element>filter(content, Element.class);
		
		boolean error = false;
		while(elements.hasNext()){
			if(elements.next().getName().equals(e.getName())){
				error = true;
				break;
			}
		}
		
		if(error){
			error("An other Entity has the same name, change one, because Entitiy name must be unique!", DatamodelPackage.Literals.ELEMENT__NAME);
		}
		
	}
	
	@Check(CheckType.FAST)
	public void checkURIs(Content c){
		ResourceSet rs = findOtherContent(c.eResource());
		TreeIterator<Notifier> content = rs.getAllContents();
		Iterator<Content> contents = Iterators.<Content>filter(content, Content.class);
		
		boolean error = false;
		while(contents.hasNext()){
			if(contents.next().getName().equals(c.getName())){
				error = true;
				break;
			}
		}
		
		if(error){
			error("An other Entity has the same URI, change one, because File URIs must be unique!", DatamodelPackage.Literals.CONTENT__NAME);
		}
		
	}
	
	private ResourceSet findOtherContent(Resource resource) {
		ResourceSet rs = new ResourceSetImpl();
		IResourceDescriptions index = resourceDescriptionsProvider.createResourceDescriptions();
		IResourceDescription resDesc = resourceServiceProvider.getResourceDescriptionManager().getResourceDescription(resource);
		List<org.eclipse.xtext.resource.IContainer> visibleContainers = resourceServiceProvider.getContainerManager().getVisibleContainers(resDesc, index);
		for (org.eclipse.xtext.resource.IContainer c : visibleContainers) {
			for (IResourceDescription rd : c.getResourceDescriptions()) {
				if(!rd.getURI().equals(resource.getURI())){
					rs.getResource(rd.getURI(), true);
				}
			}
		}
		return rs;
	}
	
}