package hu.bme.mit.mobilegen.datamodel.generator.database.ormlite;

import com.google.common.base.Objects;
import hu.bme.mit.mobilegen.datamodel.datamodel.Entity;
import hu.bme.mit.mobilegen.datamodel.generator.GeneratorConstants;
import hu.bme.mit.mobilegen.datamodel.generator.GeneratorData;
import hu.bme.mit.mobilegen.datamodel.helper.GeneratorHelper;
import hu.bme.mit.mobilegen.datamodel.helper.WrappedEntity;
import hu.bme.mit.mobilegen.datamodel.logger.Log;
import java.util.ArrayList;
import java.util.List;
import org.eclipse.xtend2.lib.StringConcatenation;
import org.eclipse.xtext.generator.IFileSystemAccess;
import org.eclipse.xtext.xbase.lib.StringExtensions;

@SuppressWarnings("all")
public class ORMLiteDAOFactoryGenerator {
  private List<WrappedEntity> wes;
  
  private List<String> imports;
  
  public void generateDAOFactory(final List<WrappedEntity> entities, final IFileSystemAccess fsa) {
    this.wes = entities;
    this.collectImports();
    Log.info("Generating DAOFactory.java");
    CharSequence _generateDAOFactory = this.generateDAOFactory();
    fsa.generateFile("DAOFactory.java", GeneratorConstants.SRC_GEN_DB, _generateDAOFactory);
  }
  
  private boolean collectImports() {
    boolean _xblockexpression = false;
    {
      ArrayList<String> _arrayList = new ArrayList<String>();
      this.imports = _arrayList;
      this.imports.add("android.content.Context");
      boolean _xifexpression = false;
      boolean _isCache = GeneratorData.isCache();
      if (_isCache) {
        _xifexpression = this.imports.add("com.j256.ormlite.dao.Dao");
      }
      _xblockexpression = _xifexpression;
    }
    return _xblockexpression;
  }
  
  private CharSequence generateDAOFactory() {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _classHeader = this.classHeader();
    _builder.append(_classHeader, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    CharSequence _packageDeclaration = this.packageDeclaration();
    _builder.append(_packageDeclaration, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    CharSequence _classImports = this.classImports();
    _builder.append(_classImports, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("//");
    _builder.newLine();
    CharSequence _classDeclaration = this.classDeclaration();
    _builder.append(_classDeclaration, "");
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  private CharSequence classHeader() {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("//------------------------- Geneated Java Class File For DAOFactory Class ---------------------------------");
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence packageDeclaration() {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("package ");
    String _packageName = GeneratorData.getPackageName();
    _builder.append(_packageName, "");
    _builder.append(".");
    _builder.append(GeneratorConstants.SRC_GEN_DB, "");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  private CharSequence classImports() {
    StringConcatenation _builder = new StringConcatenation();
    {
      for(final String f : this.imports) {
        _builder.append("import ");
        _builder.append(f, "");
        _builder.append(";");
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder;
  }
  
  private CharSequence classJavaDoc() {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("/**");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <!-- begin-user-doc -->");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* This Class contains the representation of the model object \'<em><b></b></em>\'.");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* For each Entity the generator creates an EntityGEN.java file,");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* which contains EntityGEN class with every Property and Association for the Entity.");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* This Java file will be override, with each generation.");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* The generator creates an Entity.java file, with the Entity class,");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* which extends form EntityGEN. This file normally (if the Entity is not deleted)");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* won\'t be overrided, so this file can be modified between the generations.");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* ");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <p>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* This is The EntityGEN.java file. It must contain only the generated code.");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* Do not edit it, or it will be overrided with each generation!");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* </p>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <!-- end-user-doc -->");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <p>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* The following features are supported:");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <ul>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*   <li>{@link hu.bme.hellga.onlab.Entity#getCache <em>Cache</em>}</li>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*   <li>{@link hu.bme.hellga.onlab.Entity#getPersistentType <em>Persistent Type</em>}</li>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*   <li>{@link hu.bme.hellga.onlab.Entity#isAbstract <em>Abstract</em>}</li>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*   <li>{@link hu.bme.hellga.onlab.Entity#getName <em>Name</em>}</li>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*   <li>{@link hu.bme.hellga.onlab.Entity#getSuperType <em>Super Type</em>}</li>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*   <li>{@link hu.bme.hellga.onlab.Entity#getRelationship <em>Relationship</em>}</li>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*   <li>{@link hu.bme.hellga.onlab.Entity#getProperty <em>Property</em>}</li>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* </ul>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* </p>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* @see hu.bme.hellga.onlab.OnlabPackage#getEntity()");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* @model");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* @generated");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*/");
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence classDeclaration() {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("public class DAOFactory {");
    _builder.newLine();
    _builder.append("\t");
    _builder.newLine();
    _builder.append("\t");
    CharSequence _classBody = this.classBody();
    _builder.append(_classBody, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence classBody() {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _properties = this.properties();
    _builder.append(_properties, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    CharSequence _initDB = this.initDB();
    _builder.append(_initDB, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    CharSequence _closeDB = this.closeDB();
    _builder.append(_closeDB, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    CharSequence _dbName = this.dbName();
    _builder.append(_dbName, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    CharSequence _dbVersion = this.dbVersion();
    _builder.append(_dbVersion, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    CharSequence _daoGetters = this.daoGetters();
    _builder.append(_daoGetters, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    CharSequence _genericDAO = this.genericDAO();
    _builder.append(_genericDAO, "");
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  private CharSequence properties() {
    StringConcatenation _builder = new StringConcatenation();
    {
      for(final WrappedEntity f : this.wes) {
        _builder.append("//");
        _builder.newLine();
        CharSequence _propertyDeclaration = this.propertyDeclaration(f);
        _builder.append(_propertyDeclaration, "");
        _builder.newLineIfNotEmpty();
        _builder.newLine();
      }
    }
    {
      boolean _isCache = GeneratorData.isCache();
      if (_isCache) {
        _builder.append("//");
        _builder.newLine();
        _builder.append("private static GenericDAOInterface genericDI;");
        _builder.newLine();
      }
    }
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence propertyJavaDoc(final WrappedEntity we) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("/**");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <p>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* JavaDoc");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* </p>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <!-- end-user-doc -->");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <p>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* The following features are supported:");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* @see hu.bme.hellga.onlab.OnlabPackage#getEntity()");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* @model");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* @generated");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*/");
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence propertyDeclaration(final WrappedEntity we) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("private static ");
    Entity _entity = we.getEntity();
    String _name = _entity.getName();
    String _firstUpper = StringExtensions.toFirstUpper(_name);
    _builder.append(_firstUpper, "");
    _builder.append("DAOInterface ");
    Entity _entity_1 = we.getEntity();
    String _name_1 = _entity_1.getName();
    String _firstLower = StringExtensions.toFirstLower(_name_1);
    _builder.append(_firstLower, "");
    _builder.append("DI;");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence initDB() {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("//");
    _builder.newLine();
    _builder.append("public static void initDatabase(Context c){");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("DatabaseFactory.initDatabase(c);");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence closeDB() {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("//");
    _builder.newLine();
    _builder.append("public static void closeDatabase(){");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("DatabaseFactory.colseDatabase();");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence dbName() {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("//");
    _builder.newLine();
    _builder.append("public static void getDatabaseName(){");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("DatabaseFactory.getDBName();");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence dbVersion() {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("//");
    _builder.newLine();
    _builder.append("public static void getDatabaseVersion(){");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("DatabaseFactory.getDBVersion();");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence daoGetters() {
    StringConcatenation _builder = new StringConcatenation();
    {
      for(final WrappedEntity f : this.wes) {
        _builder.append("//");
        _builder.newLine();
        CharSequence _daoGetter = this.daoGetter(f);
        _builder.append(_daoGetter, "");
        _builder.newLineIfNotEmpty();
        _builder.newLine();
      }
    }
    return _builder;
  }
  
  private CharSequence daoGetterJavaDoc(final WrappedEntity we) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("/**");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <p>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* JavaDoc");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* </p>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <!-- end-user-doc -->");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <p>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* The following features are supported:");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* @see hu.bme.hellga.onlab.OnlabPackage#getEntity()");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* @model");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* @generated");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*/");
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence daoGetter(final WrappedEntity we) {
    StringConcatenation _builder = new StringConcatenation();
    {
      GeneratorHelper _genHelp = we.getGenHelp();
      boolean _equals = Objects.equal(_genHelp, GeneratorHelper.JoinClass);
      boolean _not = (!_equals);
      if (_not) {
        _builder.append("public ");
      }
    }
    _builder.append("static ");
    Entity _entity = we.getEntity();
    String _name = _entity.getName();
    String _firstUpper = StringExtensions.toFirstUpper(_name);
    _builder.append(_firstUpper, "");
    _builder.append("DAOInterface get");
    Entity _entity_1 = we.getEntity();
    String _name_1 = _entity_1.getName();
    String _firstUpper_1 = StringExtensions.toFirstUpper(_name_1);
    _builder.append(_firstUpper_1, "");
    _builder.append("DAO(){");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("if(");
    Entity _entity_2 = we.getEntity();
    String _name_2 = _entity_2.getName();
    String _firstLower = StringExtensions.toFirstLower(_name_2);
    _builder.append(_firstLower, "\t");
    _builder.append("DI == null){");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t");
    Entity _entity_3 = we.getEntity();
    String _name_3 = _entity_3.getName();
    String _firstLower_1 = StringExtensions.toFirstLower(_name_3);
    _builder.append(_firstLower_1, "\t\t");
    _builder.append("DI = new ");
    Entity _entity_4 = we.getEntity();
    String _name_4 = _entity_4.getName();
    String _firstUpper_2 = StringExtensions.toFirstUpper(_name_4);
    _builder.append(_firstUpper_2, "\t\t");
    _builder.append("DAO();");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("return ");
    Entity _entity_5 = we.getEntity();
    String _name_5 = _entity_5.getName();
    String _firstLower_2 = StringExtensions.toFirstLower(_name_5);
    _builder.append(_firstLower_2, "\t");
    _builder.append("DI;");
    _builder.newLineIfNotEmpty();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence genericDAO() {
    StringConcatenation _builder = new StringConcatenation();
    {
      boolean _isCache = GeneratorData.isCache();
      if (_isCache) {
        _builder.append("static <T> Dao<T, Integer> getGenericDAO(Class<T> t){");
        _builder.newLine();
        _builder.append("\t");
        _builder.append("if(genericDI ==  null){");
        _builder.newLine();
        _builder.append("\t\t");
        _builder.append("genericDI = new GenericDAO();");
        _builder.newLine();
        _builder.append("\t");
        _builder.append("}");
        _builder.newLine();
        _builder.append("\t");
        _builder.append("return genericDI.getGenericDAO(t);");
        _builder.newLine();
        _builder.append("}");
        _builder.newLine();
      }
    }
    return _builder;
  }
}
