package hu.bme.mit.mobilegen.datamodel.generator.entity;

import com.google.common.base.Objects;
import hu.bme.mit.mobilegen.datamodel.datamodel.Entity;
import hu.bme.mit.mobilegen.datamodel.datamodel.Enumeration;
import hu.bme.mit.mobilegen.datamodel.datamodel.Property;
import hu.bme.mit.mobilegen.datamodel.datamodel.PropertyType;
import hu.bme.mit.mobilegen.datamodel.generator.GeneratorConstants;
import hu.bme.mit.mobilegen.datamodel.generator.GeneratorData;
import hu.bme.mit.mobilegen.datamodel.helper.FormatPropertyType;
import hu.bme.mit.mobilegen.datamodel.helper.GeneratorHelper;
import hu.bme.mit.mobilegen.datamodel.helper.WrappedEntity;
import hu.bme.mit.mobilegen.datamodel.logger.Log;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import org.eclipse.xtend2.lib.StringConcatenation;
import org.eclipse.xtext.generator.IFileSystemAccess;
import org.eclipse.xtext.xbase.lib.StringExtensions;

@SuppressWarnings("all")
public class EntityFactoryGenerator {
  private List<WrappedEntity> wes;
  
  private List<Property> rp;
  
  public void generateEntityFactory(final List<WrappedEntity> entities, final IFileSystemAccess fsa) {
    ArrayList<WrappedEntity> _arrayList = new ArrayList<WrappedEntity>();
    this.wes = _arrayList;
    for (final WrappedEntity we : entities) {
      GeneratorHelper _genHelp = we.getGenHelp();
      boolean _equals = Objects.equal(_genHelp, GeneratorHelper.JoinClass);
      boolean _not = (!_equals);
      if (_not) {
        this.wes.add(we);
      }
    }
    Log.info("Generating EntityFactory.java");
    CharSequence _generateEntityFactory = this.generateEntityFactory();
    fsa.generateFile("EntityFactory.java", GeneratorConstants.SRC_GEN_ENTITIES, _generateEntityFactory);
  }
  
  private void setReqProps(final WrappedEntity we) {
    Map<String, Property> reqProps = new HashMap<String, Property>();
    WrappedEntity w = we;
    while ((!Objects.equal(w, null))) {
      {
        List<Property> _requiredProperties = w.getRequiredProperties();
        for (final Property p : _requiredProperties) {
          String _name = p.getName();
          boolean _containsKey = reqProps.containsKey(_name);
          boolean _not = (!_containsKey);
          if (_not) {
            String _name_1 = p.getName();
            reqProps.put(_name_1, p);
          }
        }
        WrappedEntity _superClass = w.getSuperClass();
        w = _superClass;
      }
    }
    Collection<Property> _values = reqProps.values();
    ArrayList<Property> _arrayList = new ArrayList<Property>(_values);
    this.rp = _arrayList;
  }
  
  private CharSequence generateEntityFactory() {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _classHeader = this.classHeader();
    _builder.append(_classHeader, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    CharSequence _packageDeclaration = this.packageDeclaration();
    _builder.append(_packageDeclaration, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    CharSequence _classImports = this.classImports();
    _builder.append(_classImports, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("//");
    _builder.newLine();
    CharSequence _classDeclaration = this.classDeclaration();
    _builder.append(_classDeclaration, "");
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  private CharSequence classHeader() {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("//------------------------- Geneated Java Class File For EntityFactory ---------------------------------");
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence packageDeclaration() {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("package ");
    String _packageName = GeneratorData.getPackageName();
    _builder.append(_packageName, "");
    _builder.append(".");
    _builder.append(GeneratorConstants.SRC_GEN_ENTITIES, "");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  private CharSequence classImports() {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("//import");
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence classJavaDoc() {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("/**");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <!-- begin-user-doc -->");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* This Class contains the representation of the model object \'<em><b></b></em>\'.");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* For each Entity the generator creates an EntityGEN.java file,");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* which contains EntityGEN class with every Property and Association for the Entity.");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* This Java file will be override, with each generation.");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* The generator creates an Entity.java file, with the Entity class,");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* which extends form EntityGEN. This file normally (if the Entity is not deleted)");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* won\'t be overrided, so this file can be modified between the generations.");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* ");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <p>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* This is The Entity.java file. It is created at the first generation, after that,");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* it can be modified, it won\'t be overrided, until the Entity is renamed, or the");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* Entity is deleted.");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* </p>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <!-- end-user-doc -->");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <p>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* The following features are supported:");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <ul>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*   <li>{@link hu.bme.hellga.onlab.Entity#getCache <em>Cache</em>}</li>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*   <li>{@link hu.bme.hellga.onlab.Entity#getPersistentType <em>Persistent Type</em>}</li>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*   <li>{@link hu.bme.hellga.onlab.Entity#isAbstract <em>Abstract</em>}</li>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*   <li>{@link hu.bme.hellga.onlab.Entity#getName <em>Name</em>}</li>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*   <li>{@link hu.bme.hellga.onlab.Entity#getSuperType <em>Super Type</em>}</li>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*   <li>{@link hu.bme.hellga.onlab.Entity#getRelationship <em>Relationship</em>}</li>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*   <li>{@link hu.bme.hellga.onlab.Entity#getProperty <em>Property</em>}</li>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* </ul>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* </p>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* @see hu.bme.hellga.onlab.OnlabPackage#getEntity()");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* @model");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* @generated");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*/");
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence classDeclaration() {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("public class EntityFactory {");
    _builder.newLine();
    _builder.append("\t");
    _builder.newLine();
    _builder.append("\t");
    CharSequence _factories = this.factories();
    _builder.append(_factories, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence factories() {
    StringConcatenation _builder = new StringConcatenation();
    {
      for(final WrappedEntity f : this.wes) {
        this.setReqProps(f);
        _builder.newLineIfNotEmpty();
        {
          Entity _entity = f.getEntity();
          boolean _isAbstract = _entity.isAbstract();
          boolean _not = (!_isAbstract);
          if (_not) {
            _builder.append("//");
            _builder.newLine();
            CharSequence _factory = this.factory(f);
            _builder.append(_factory, "");
            _builder.newLineIfNotEmpty();
            _builder.newLine();
            _builder.append("//");
            _builder.newLine();
            CharSequence _factoryCopy = this.factoryCopy(f);
            _builder.append(_factoryCopy, "");
            _builder.newLineIfNotEmpty();
          }
        }
      }
    }
    return _builder;
  }
  
  private CharSequence factoryJavaDoc(final WrappedEntity we) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("/**");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <p>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*  FactoryMethod");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* </p>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <!-- end-user-doc -->");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* @see hu.bme.hellga.onlab.OnlabPackage#getEntity()");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* @model");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* @generated");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*/");
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence factory(final WrappedEntity we) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("public static ");
    Entity _entity = we.getEntity();
    String _name = _entity.getName();
    String _firstUpper = StringExtensions.toFirstUpper(_name);
    _builder.append(_firstUpper, "");
    _builder.append(" create");
    Entity _entity_1 = we.getEntity();
    String _name_1 = _entity_1.getName();
    String _firstUpper_1 = StringExtensions.toFirstUpper(_name_1);
    _builder.append(_firstUpper_1, "");
    _builder.append("(");
    {
      boolean _hasElements = false;
      for(final Property f : this.rp) {
        if (!_hasElements) {
          _hasElements = true;
        } else {
          _builder.appendImmediate(", ", "");
        }
        {
          Enumeration _enumeration = f.getEnumeration();
          boolean _notEquals = (!Objects.equal(_enumeration, null));
          if (_notEquals) {
            Enumeration _enumeration_1 = f.getEnumeration();
            String _name_2 = _enumeration_1.getName();
            String _firstUpper_2 = StringExtensions.toFirstUpper(_name_2);
            _builder.append(_firstUpper_2, "");
          } else {
            PropertyType _type = f.getType();
            String _formatPrimitive = FormatPropertyType.formatPrimitive(_type);
            _builder.append(_formatPrimitive, "");
          }
        }
        _builder.append(" ");
        String _name_3 = f.getName();
        String _firstLower = StringExtensions.toFirstLower(_name_3);
        _builder.append(_firstLower, "");
      }
    }
    _builder.append("){");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    Entity _entity_2 = we.getEntity();
    String _name_4 = _entity_2.getName();
    String _firstUpper_3 = StringExtensions.toFirstUpper(_name_4);
    _builder.append(_firstUpper_3, "\t");
    _builder.append(" ");
    Entity _entity_3 = we.getEntity();
    String _name_5 = _entity_3.getName();
    String _substring = _name_5.substring(0, 1);
    String _firstLower_1 = StringExtensions.toFirstLower(_substring);
    _builder.append(_firstLower_1, "\t");
    _builder.append(" = new ");
    Entity _entity_4 = we.getEntity();
    String _name_6 = _entity_4.getName();
    String _firstUpper_4 = StringExtensions.toFirstUpper(_name_6);
    _builder.append(_firstUpper_4, "\t");
    _builder.append("();");
    _builder.newLineIfNotEmpty();
    {
      for(final Property f_1 : this.rp) {
        _builder.append("\t");
        Entity _entity_5 = we.getEntity();
        String _name_7 = _entity_5.getName();
        String _substring_1 = _name_7.substring(0, 1);
        String _firstLower_2 = StringExtensions.toFirstLower(_substring_1);
        _builder.append(_firstLower_2, "\t");
        _builder.append(".set");
        String _name_8 = f_1.getName();
        String _firstUpper_5 = StringExtensions.toFirstUpper(_name_8);
        _builder.append(_firstUpper_5, "\t");
        _builder.append("(");
        String _name_9 = f_1.getName();
        String _firstLower_3 = StringExtensions.toFirstLower(_name_9);
        _builder.append(_firstLower_3, "\t");
        _builder.append(");");
        _builder.newLineIfNotEmpty();
      }
    }
    _builder.append("\t");
    _builder.append("return ");
    Entity _entity_6 = we.getEntity();
    String _name_10 = _entity_6.getName();
    String _substring_2 = _name_10.substring(0, 1);
    String _firstLower_4 = StringExtensions.toFirstLower(_substring_2);
    _builder.append(_firstLower_4, "\t");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence factoryCopy(final WrappedEntity we) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("public static ");
    Entity _entity = we.getEntity();
    String _name = _entity.getName();
    String _firstUpper = StringExtensions.toFirstUpper(_name);
    _builder.append(_firstUpper, "");
    _builder.append(" create");
    Entity _entity_1 = we.getEntity();
    String _name_1 = _entity_1.getName();
    String _firstUpper_1 = StringExtensions.toFirstUpper(_name_1);
    _builder.append(_firstUpper_1, "");
    _builder.append("(");
    Entity _entity_2 = we.getEntity();
    String _name_2 = _entity_2.getName();
    String _firstUpper_2 = StringExtensions.toFirstUpper(_name_2);
    _builder.append(_firstUpper_2, "");
    _builder.append(" ");
    Entity _entity_3 = we.getEntity();
    String _name_3 = _entity_3.getName();
    String _firstLower = StringExtensions.toFirstLower(_name_3);
    _builder.append(_firstLower, "");
    _builder.append("){");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    Entity _entity_4 = we.getEntity();
    String _name_4 = _entity_4.getName();
    String _firstUpper_3 = StringExtensions.toFirstUpper(_name_4);
    _builder.append(_firstUpper_3, "\t");
    _builder.append(" ");
    Entity _entity_5 = we.getEntity();
    String _name_5 = _entity_5.getName();
    String _substring = _name_5.substring(0, 1);
    String _firstLower_1 = StringExtensions.toFirstLower(_substring);
    _builder.append(_firstLower_1, "\t");
    _builder.append(" = new ");
    Entity _entity_6 = we.getEntity();
    String _name_6 = _entity_6.getName();
    String _firstUpper_4 = StringExtensions.toFirstUpper(_name_6);
    _builder.append(_firstUpper_4, "\t");
    _builder.append("(");
    Entity _entity_7 = we.getEntity();
    String _name_7 = _entity_7.getName();
    String _firstLower_2 = StringExtensions.toFirstLower(_name_7);
    _builder.append(_firstLower_2, "\t");
    _builder.append(");");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("return ");
    Entity _entity_8 = we.getEntity();
    String _name_8 = _entity_8.getName();
    String _substring_1 = _name_8.substring(0, 1);
    String _firstLower_3 = StringExtensions.toFirstLower(_substring_1);
    _builder.append(_firstLower_3, "\t");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
}
