package hu.bme.mit.mobilegen.datamodel.generator.entity;

import com.google.common.base.Objects;
import com.google.inject.Inject;
import hu.bme.mit.mobilegen.datamodel.datamodel.Association;
import hu.bme.mit.mobilegen.datamodel.datamodel.ConnectionType;
import hu.bme.mit.mobilegen.datamodel.datamodel.Entity;
import hu.bme.mit.mobilegen.datamodel.datamodel.Enumeration;
import hu.bme.mit.mobilegen.datamodel.datamodel.PersistentType;
import hu.bme.mit.mobilegen.datamodel.datamodel.Property;
import hu.bme.mit.mobilegen.datamodel.datamodel.PropertyType;
import hu.bme.mit.mobilegen.datamodel.generator.GeneratorConstants;
import hu.bme.mit.mobilegen.datamodel.generator.GeneratorData;
import hu.bme.mit.mobilegen.datamodel.generator.database.IDatabaseGeneratorUtility;
import hu.bme.mit.mobilegen.datamodel.helper.FormatPropertyType;
import hu.bme.mit.mobilegen.datamodel.helper.GeneratorHelper;
import hu.bme.mit.mobilegen.datamodel.helper.WrappedEntity;
import hu.bme.mit.mobilegen.datamodel.logger.Log;
import java.util.ArrayList;
import java.util.List;
import org.eclipse.emf.common.util.EList;
import org.eclipse.xtend2.lib.StringConcatenation;
import org.eclipse.xtext.generator.IFileSystemAccess;
import org.eclipse.xtext.xbase.lib.StringExtensions;

@SuppressWarnings("all")
public class EntityGENGenerator {
  @Inject
  private IDatabaseGeneratorUtility dbGen;
  
  private WrappedEntity wrappedEntity;
  
  private Entity entity;
  
  private List<String> imports;
  
  private boolean only = false;
  
  public void generateEntityGEN(final WrappedEntity we, final IFileSystemAccess fsa) {
    this.wrappedEntity = we;
    Entity _entity = we.getEntity();
    this.entity = _entity;
    this.only = false;
    this.collectImports();
    PersistentType _persistentType = this.entity.getPersistentType();
    boolean _equals = Objects.equal(_persistentType, PersistentType.DB);
    if (_equals) {
      this.dbGen.collectEntityGENImports(this.imports, this.wrappedEntity);
    }
    String _name = this.entity.getName();
    String _firstUpper = StringExtensions.toFirstUpper(_name);
    String _plus = ("Generating " + _firstUpper);
    String _plus_1 = (_plus + "GEN.java");
    Log.info(_plus_1);
    String _name_1 = this.entity.getName();
    String _firstUpper_1 = StringExtensions.toFirstUpper(_name_1);
    String _plus_2 = (_firstUpper_1 + "GEN.java");
    CharSequence _generateEntityGEN = this.generateEntityGEN(this.entity);
    fsa.generateFile(_plus_2, GeneratorConstants.SRC_GEN_ENTITIES, _generateEntityGEN);
  }
  
  public void generateEntityGENOnly(final WrappedEntity we, final IFileSystemAccess fsa) {
    this.wrappedEntity = we;
    Entity _entity = we.getEntity();
    this.entity = _entity;
    this.only = true;
    this.collectImports();
    PersistentType _persistentType = this.entity.getPersistentType();
    boolean _equals = Objects.equal(_persistentType, PersistentType.DB);
    if (_equals) {
      this.dbGen.collectEntityGENImports(this.imports, this.wrappedEntity);
      this.dbGen.collectEntityImports(this.imports, this.wrappedEntity);
    }
    String _name = this.entity.getName();
    String _firstUpper = StringExtensions.toFirstUpper(_name);
    String _plus = ("Generating " + _firstUpper);
    String _plus_1 = (_plus + ".java");
    Log.info(_plus_1);
    String _name_1 = this.entity.getName();
    String _firstUpper_1 = StringExtensions.toFirstUpper(_name_1);
    String _plus_2 = (_firstUpper_1 + ".java");
    CharSequence _generateEntityGENOnly = this.generateEntityGENOnly(this.entity);
    fsa.generateFile(_plus_2, GeneratorConstants.SRC_GEN_ENTITIES, _generateEntityGENOnly);
  }
  
  private boolean collectImports() {
    boolean _xblockexpression = false;
    {
      ArrayList<String> _arrayList = new ArrayList<String>();
      this.imports = _arrayList;
      boolean list = false;
      EList<Property> _property = this.entity.getProperty();
      for (final Property p : _property) {
        ConnectionType _connectionType = p.getConnectionType();
        boolean _equals = Objects.equal(_connectionType, ConnectionType.MANY);
        if (_equals) {
          list = true;
        }
      }
      EList<Association> _association = this.entity.getAssociation();
      for (final Association a : _association) {
        boolean _and = false;
        ConnectionType _connectionType_1 = a.getConnectionType();
        boolean _equals_1 = Objects.equal(_connectionType_1, ConnectionType.MANY);
        if (!_equals_1) {
          _and = false;
        } else {
          boolean _or = false;
          boolean _or_1 = false;
          PersistentType _persistentType = this.entity.getPersistentType();
          boolean _equals_2 = _persistentType.equals(PersistentType.DB);
          boolean _not = (!_equals_2);
          if (_not) {
            _or_1 = true;
          } else {
            boolean _isStatic = a.isStatic();
            _or_1 = _isStatic;
          }
          if (_or_1) {
            _or = true;
          } else {
            boolean _isTransient = a.isTransient();
            _or = _isTransient;
          }
          _and = _or;
        }
        if (_and) {
          list = true;
        }
      }
      if (list) {
        this.imports.add("java.util.List");
      }
      boolean date = false;
      EList<Property> _property_1 = this.entity.getProperty();
      for (final Property p_1 : _property_1) {
        PropertyType _type = p_1.getType();
        boolean _equals_3 = Objects.equal(_type, PropertyType.DATE);
        if (_equals_3) {
          date = true;
        }
      }
      if (date) {
        this.imports.add("java.util.Date");
      }
      PersistentType _persistentType_1 = this.entity.getPersistentType();
      boolean _equals_4 = Objects.equal(_persistentType_1, PersistentType.SERIALIZE);
      if (_equals_4) {
        this.imports.add("java.io.Serializable");
      }
      this.imports.add("android.os.Parcel");
      _xblockexpression = this.imports.add("android.os.Parcelable");
    }
    return _xblockexpression;
  }
  
  private CharSequence generateEntityGEN(final Entity e) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _classHeader = this.classHeader(e);
    _builder.append(_classHeader, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    CharSequence _packageDeclaration = this.packageDeclaration();
    _builder.append(_packageDeclaration, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    CharSequence _classImports = this.classImports(e);
    _builder.append(_classImports, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("//");
    _builder.newLine();
    CharSequence _classDeclaration = this.classDeclaration(e);
    _builder.append(_classDeclaration, "");
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  private CharSequence generateEntityGENOnly(final Entity e) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _classHeader = this.classHeader(e);
    _builder.append(_classHeader, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    CharSequence _packageDeclaration = this.packageDeclaration();
    _builder.append(_packageDeclaration, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    CharSequence _classImports = this.classImports(e);
    _builder.append(_classImports, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("//");
    _builder.newLine();
    CharSequence _classAnnotations = this.classAnnotations(e);
    _builder.append(_classAnnotations, "");
    _builder.newLineIfNotEmpty();
    CharSequence _classDeclarationOnlyGEN = this.classDeclarationOnlyGEN(e);
    _builder.append(_classDeclarationOnlyGEN, "");
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  private CharSequence classHeader(final Entity e) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("//------------------------- Geneated Java Class File For Entity - ");
    String _name = e.getName();
    _builder.append(_name, "");
    _builder.append(" ---------------------------------");
    _builder.newLineIfNotEmpty();
    _builder.append("//------------------------- Model Implementation In ");
    String _resourceURI = this.wrappedEntity.getResourceURI();
    _builder.append(_resourceURI, "");
    _builder.append(" Resource File --------------");
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  private CharSequence packageDeclaration() {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("package ");
    String _packageName = GeneratorData.getPackageName();
    _builder.append(_packageName, "");
    _builder.append(".");
    _builder.append(GeneratorConstants.SRC_GEN_ENTITIES, "");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  private CharSequence classImports(final Entity e) {
    StringConcatenation _builder = new StringConcatenation();
    {
      for(final String f : this.imports) {
        _builder.append("import ");
        _builder.append(f, "");
        _builder.append(";");
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder;
  }
  
  private CharSequence classJavaDoc(final Entity e) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("/**");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <!-- begin-user-doc -->");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* This Class contains the representation of the model object \'<em><b");
    String _name = e.getName();
    _builder.append(_name, " ");
    _builder.append("</b></em>\'.");
    _builder.newLineIfNotEmpty();
    _builder.append(" ");
    _builder.append("* For each Entity the generator creates an EntityGEN.java file,");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* which contains EntityGEN class with every Property and Association for the Entity.");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* This Java file will be override, with each generation.");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* The generator creates an Entity.java file, with the Entity class,");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* which extends form EntityGEN. This file normally (if the Entity is not deleted)");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* won\'t be overrided, so this file can be modified between the generations.");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* ");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <p>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* This is The EntityGEN.java file. It must contain only the generated code.");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* Do not edit it, or it will be overrided with each generation!");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* </p>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <!-- end-user-doc -->");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <p>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* The following features are supported:");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <ul>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*   <li>{@link hu.bme.hellga.onlab.Entity#getCache <em>Cache</em>}</li>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*   <li>{@link hu.bme.hellga.onlab.Entity#getPersistentType <em>Persistent Type</em>}</li>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*   <li>{@link hu.bme.hellga.onlab.Entity#isAbstract <em>Abstract</em>}</li>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*   <li>{@link hu.bme.hellga.onlab.Entity#getName <em>Name</em>}</li>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*   <li>{@link hu.bme.hellga.onlab.Entity#getSuperType <em>Super Type</em>}</li>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*   <li>{@link hu.bme.hellga.onlab.Entity#getRelationship <em>Relationship</em>}</li>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*   <li>{@link hu.bme.hellga.onlab.Entity#getProperty <em>Property</em>}</li>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* </ul>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* </p>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* @see hu.bme.hellga.onlab.OnlabPackage#getEntity()");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* @model");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* @generated");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*/");
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence classAnnotations(final Entity e) {
    StringConcatenation _builder = new StringConcatenation();
    {
      PersistentType _persistentType = this.entity.getPersistentType();
      boolean _equals = _persistentType.equals(PersistentType.DB);
      if (_equals) {
        CharSequence _classAnnotations = this.dbGen.classAnnotations(e);
        _builder.append(_classAnnotations, "");
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder;
  }
  
  private CharSequence classDeclaration(final Entity e) {
    StringConcatenation _builder = new StringConcatenation();
    {
      boolean _isAbstract = e.isAbstract();
      if (_isAbstract) {
        _builder.append("abstract ");
      }
    }
    _builder.append("class ");
    String _name = e.getName();
    String _firstUpper = StringExtensions.toFirstUpper(_name);
    _builder.append(_firstUpper, "");
    _builder.append("GEN ");
    {
      Entity _superClass = e.getSuperClass();
      boolean _notEquals = (!Objects.equal(_superClass, null));
      if (_notEquals) {
        _builder.append("extends ");
        Entity _superClass_1 = e.getSuperClass();
        String _name_1 = _superClass_1.getName();
        String _firstUpper_1 = StringExtensions.toFirstUpper(_name_1);
        _builder.append(_firstUpper_1, "");
        _builder.append("GEN ");
      }
    }
    _builder.append("implements Parcelable");
    {
      PersistentType _persistentType = e.getPersistentType();
      boolean _equals = Objects.equal(_persistentType, PersistentType.SERIALIZE);
      if (_equals) {
        _builder.append(", Serializable");
      }
    }
    _builder.append(" {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.newLine();
    _builder.append("\t");
    CharSequence _classBody = this.classBody(e);
    _builder.append(_classBody, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence classDeclarationOnlyGEN(final Entity e) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("public ");
    {
      boolean _isAbstract = e.isAbstract();
      if (_isAbstract) {
        _builder.append("abstract ");
      }
    }
    _builder.append("class ");
    String _name = e.getName();
    String _firstUpper = StringExtensions.toFirstUpper(_name);
    _builder.append(_firstUpper, "");
    _builder.append(" ");
    {
      Entity _superClass = e.getSuperClass();
      boolean _notEquals = (!Objects.equal(_superClass, null));
      if (_notEquals) {
        _builder.append("extends ");
        Entity _superClass_1 = e.getSuperClass();
        String _name_1 = _superClass_1.getName();
        String _firstUpper_1 = StringExtensions.toFirstUpper(_name_1);
        _builder.append(_firstUpper_1, "");
        {
          if ((!this.only)) {
            _builder.append("GEN ");
          }
        }
      }
    }
    _builder.append("implements Parcelable");
    {
      PersistentType _persistentType = e.getPersistentType();
      boolean _equals = Objects.equal(_persistentType, PersistentType.SERIALIZE);
      if (_equals) {
        _builder.append(", Serializable");
      }
    }
    _builder.append(" {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.newLine();
    _builder.append("\t");
    CharSequence _classBodyOnlyGEN = this.classBodyOnlyGEN(e);
    _builder.append(_classBodyOnlyGEN, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence classBody(final Entity e) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("// Property declarations");
    _builder.newLine();
    _builder.newLine();
    CharSequence _properties = this.properties(e);
    _builder.append(_properties, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("// Association declarations");
    _builder.newLine();
    _builder.newLine();
    CharSequence _associations = this.associations(e);
    _builder.append(_associations, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("// Parcelable creator");
    _builder.newLine();
    _builder.newLine();
    CharSequence _parcelableCreator = this.parcelableCreator(e);
    _builder.append(_parcelableCreator, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("// Constructor");
    _builder.newLine();
    _builder.newLine();
    CharSequence _constructor = this.constructor(e);
    _builder.append(_constructor, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("// Parcel methods");
    _builder.newLine();
    _builder.newLine();
    CharSequence _parcelMethods = this.parcelMethods(e);
    _builder.append(_parcelMethods, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("// Getter-setters");
    _builder.newLine();
    _builder.newLine();
    CharSequence _terSetters = this.getterSetters(e);
    _builder.append(_terSetters, "");
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  private CharSequence classBodyOnlyGEN(final Entity e) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("// Property declarations");
    _builder.newLine();
    _builder.newLine();
    CharSequence _properties = this.properties(e);
    _builder.append(_properties, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("// Association declarations");
    _builder.newLine();
    _builder.newLine();
    CharSequence _associations = this.associations(e);
    _builder.append(_associations, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("// Parcelable creator");
    _builder.newLine();
    _builder.newLine();
    CharSequence _parcelableCreator = this.parcelableCreator(e);
    _builder.append(_parcelableCreator, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("// Constructor");
    _builder.newLine();
    _builder.newLine();
    CharSequence _constructorOnlyGEN = this.constructorOnlyGEN(e);
    _builder.append(_constructorOnlyGEN, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("// Parcel methods");
    _builder.newLine();
    _builder.newLine();
    CharSequence _parcelMethods = this.parcelMethods(e);
    _builder.append(_parcelMethods, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("// Getter-setters");
    _builder.newLine();
    _builder.newLine();
    CharSequence _terSetters = this.getterSetters(e);
    _builder.append(_terSetters, "");
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  private CharSequence properties(final Entity e) {
    StringConcatenation _builder = new StringConcatenation();
    {
      PersistentType _persistentType = e.getPersistentType();
      boolean _equals = _persistentType.equals(PersistentType.SERIALIZE);
      if (_equals) {
        CharSequence _serializeProperty = this.serializeProperty();
        _builder.append(_serializeProperty, "");
        _builder.newLineIfNotEmpty();
        _builder.newLine();
      }
    }
    {
      PersistentType _persistentType_1 = e.getPersistentType();
      boolean _equals_1 = _persistentType_1.equals(PersistentType.DB);
      if (_equals_1) {
        CharSequence _fieldNames = this.dbGen.fieldNames(e);
        _builder.append(_fieldNames, "");
        _builder.newLineIfNotEmpty();
        CharSequence _propertyID = this.dbGen.propertyID(e);
        _builder.append(_propertyID, "");
        _builder.newLineIfNotEmpty();
        _builder.newLine();
      }
    }
    {
      EList<Property> _property = e.getProperty();
      for(final Property f : _property) {
        CharSequence _property_1 = this.property(f);
        _builder.append(_property_1, "");
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder;
  }
  
  private CharSequence serializeProperty() {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("/**");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* ");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*/");
    _builder.newLine();
    _builder.append("private static final long serialVersionUID = 1L;");
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence property(final Property p) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("//");
    _builder.newLine();
    CharSequence _propertyAnnotations = this.propertyAnnotations(p);
    _builder.append(_propertyAnnotations, "");
    _builder.newLineIfNotEmpty();
    CharSequence _propertyDeclaration = this.propertyDeclaration(p);
    _builder.append(_propertyDeclaration, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence propertyJavaDoc(final Property p) {
    StringConcatenation _builder = new StringConcatenation();
    {
      ConnectionType _connectionType = p.getConnectionType();
      boolean _equals = _connectionType.equals(ConnectionType.ONE);
      if (_equals) {
        CharSequence _simplePropertyJavaDoc = this.simplePropertyJavaDoc(p);
        _builder.append(_simplePropertyJavaDoc, "");
        _builder.newLineIfNotEmpty();
      } else {
        CharSequence _listPropertyJavaDoc = this.listPropertyJavaDoc(p);
        _builder.append(_listPropertyJavaDoc, "");
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder;
  }
  
  private CharSequence simplePropertyJavaDoc(final Property p) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("/**");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <p>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* ");
    String _name = p.getName();
    _builder.append(_name, " ");
    _builder.append(" sproperty of ");
    String _name_1 = this.entity.getName();
    _builder.append(_name_1, " ");
    _builder.append(" entitiy.");
    _builder.newLineIfNotEmpty();
    _builder.append(" ");
    _builder.append("* </p>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <!-- end-user-doc -->");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <p>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* The following features are supported:");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* @see hu.bme.hellga.onlab.OnlabPackage#getEntity()");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* @model");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* @generated");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*/");
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence listPropertyJavaDoc(final Property p) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("/**");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <p>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* ");
    String _name = p.getName();
    _builder.append(_name, " ");
    _builder.append(" lproperty of ");
    String _name_1 = this.entity.getName();
    _builder.append(_name_1, " ");
    _builder.append(" entitiy.");
    _builder.newLineIfNotEmpty();
    _builder.append(" ");
    _builder.append("* </p>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <!-- end-user-doc -->");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <p>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* The following features are supported:");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* @see hu.bme.hellga.onlab.OnlabPackage#getEntity()");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* @model");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* @generated");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*/");
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence propertyAnnotations(final Property p) {
    StringConcatenation _builder = new StringConcatenation();
    {
      PersistentType _persistentType = this.entity.getPersistentType();
      boolean _equals = _persistentType.equals(PersistentType.DB);
      if (_equals) {
        CharSequence _propertyAnnotations = this.dbGen.propertyAnnotations(p);
        _builder.append(_propertyAnnotations, "");
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder;
  }
  
  private CharSequence propertyDeclaration(final Property p) {
    StringConcatenation _builder = new StringConcatenation();
    {
      ConnectionType _connectionType = p.getConnectionType();
      boolean _equals = _connectionType.equals(ConnectionType.ONE);
      if (_equals) {
        CharSequence _simpleProperty = this.simpleProperty(p);
        _builder.append(_simpleProperty, "");
        _builder.newLineIfNotEmpty();
      } else {
        CharSequence _listProperty = this.listProperty(p);
        _builder.append(_listProperty, "");
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder;
  }
  
  private CharSequence simpleProperty(final Property p) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("protected ");
    {
      boolean _isTransient = p.isTransient();
      if (_isTransient) {
        _builder.append("transient ");
      }
    }
    {
      boolean _isStatic = p.isStatic();
      if (_isStatic) {
        _builder.append("static ");
      }
    }
    {
      Enumeration _enumeration = p.getEnumeration();
      boolean _notEquals = (!Objects.equal(_enumeration, null));
      if (_notEquals) {
        Enumeration _enumeration_1 = p.getEnumeration();
        String _name = _enumeration_1.getName();
        String _firstUpper = StringExtensions.toFirstUpper(_name);
        _builder.append(_firstUpper, "");
      } else {
        PropertyType _type = p.getType();
        String _formatPrimitive = FormatPropertyType.formatPrimitive(_type);
        _builder.append(_formatPrimitive, "");
      }
    }
    _builder.append(" ");
    String _name_1 = p.getName();
    String _firstLower = StringExtensions.toFirstLower(_name_1);
    _builder.append(_firstLower, "");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  private CharSequence listProperty(final Property p) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("protected ");
    {
      boolean _isTransient = p.isTransient();
      if (_isTransient) {
        _builder.append("transient ");
      }
    }
    {
      boolean _isStatic = p.isStatic();
      if (_isStatic) {
        _builder.append("static ");
      }
    }
    _builder.append("List<");
    {
      Enumeration _enumeration = p.getEnumeration();
      boolean _notEquals = (!Objects.equal(_enumeration, null));
      if (_notEquals) {
        Enumeration _enumeration_1 = p.getEnumeration();
        String _name = _enumeration_1.getName();
        String _firstUpper = StringExtensions.toFirstUpper(_name);
        _builder.append(_firstUpper, "");
      } else {
        PropertyType _type = p.getType();
        String _formatClass = FormatPropertyType.formatClass(_type);
        _builder.append(_formatClass, "");
      }
    }
    _builder.append("> ");
    String _name_1 = p.getName();
    String _firstLower = StringExtensions.toFirstLower(_name_1);
    _builder.append(_firstLower, "");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  private CharSequence associations(final Entity e) {
    StringConcatenation _builder = new StringConcatenation();
    {
      EList<Association> _association = e.getAssociation();
      for(final Association f : _association) {
        CharSequence _association_1 = this.association(f);
        _builder.append(_association_1, "");
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder;
  }
  
  private CharSequence association(final Association a) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("//");
    _builder.newLine();
    CharSequence _associationAnnotations = this.associationAnnotations(a);
    _builder.append(_associationAnnotations, "");
    _builder.newLineIfNotEmpty();
    CharSequence _associationDeclaration = this.associationDeclaration(a);
    _builder.append(_associationDeclaration, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence associationJavaDoc(final Association a) {
    StringConcatenation _builder = new StringConcatenation();
    {
      ConnectionType _connectionType = a.getConnectionType();
      boolean _equals = _connectionType.equals(ConnectionType.ONE);
      if (_equals) {
        CharSequence _simpleAssociationJavaDoc = this.simpleAssociationJavaDoc(a);
        _builder.append(_simpleAssociationJavaDoc, "");
        _builder.newLineIfNotEmpty();
      } else {
        CharSequence _listAssociationJavaDoc = this.listAssociationJavaDoc(a);
        _builder.append(_listAssociationJavaDoc, "");
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder;
  }
  
  private CharSequence simpleAssociationJavaDoc(final Association a) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("/**");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <p>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* ");
    String _name = a.getName();
    _builder.append(_name, " ");
    _builder.append(" sassociation of ");
    String _name_1 = this.entity.getName();
    _builder.append(_name_1, " ");
    _builder.append(" entitiy.");
    _builder.newLineIfNotEmpty();
    _builder.append(" ");
    _builder.append("* </p>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <!-- end-user-doc -->");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <p>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* The following features are supported:");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* @see hu.bme.hellga.onlab.OnlabPackage#getEntity()");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* @model");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* @generated");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*/");
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence listAssociationJavaDoc(final Association a) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("/**");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <p>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* ");
    String _name = a.getName();
    _builder.append(_name, " ");
    _builder.append(" lassociation of ");
    String _name_1 = this.entity.getName();
    _builder.append(_name_1, " ");
    _builder.append(" entitiy.");
    _builder.newLineIfNotEmpty();
    _builder.append(" ");
    _builder.append("* </p>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <!-- end-user-doc -->");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <p>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* The following features are supported:");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* @see hu.bme.hellga.onlab.OnlabPackage#getEntity()");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* @model");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* @generated");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*/");
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence associationAnnotations(final Association a) {
    StringConcatenation _builder = new StringConcatenation();
    {
      PersistentType _persistentType = this.entity.getPersistentType();
      boolean _equals = _persistentType.equals(PersistentType.DB);
      if (_equals) {
        CharSequence _associationAnnotations = this.dbGen.associationAnnotations(a);
        _builder.append(_associationAnnotations, "");
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder;
  }
  
  private CharSequence associationDeclaration(final Association a) {
    StringConcatenation _builder = new StringConcatenation();
    {
      ConnectionType _connectionType = a.getConnectionType();
      boolean _equals = _connectionType.equals(ConnectionType.ONE);
      if (_equals) {
        CharSequence _simpleAssociation = this.simpleAssociation(a);
        _builder.append(_simpleAssociation, "");
        _builder.newLineIfNotEmpty();
      } else {
        {
          boolean _and = false;
          boolean _and_1 = false;
          PersistentType _persistentType = this.entity.getPersistentType();
          boolean _equals_1 = _persistentType.equals(PersistentType.DB);
          if (!_equals_1) {
            _and_1 = false;
          } else {
            boolean _isTransient = a.isTransient();
            boolean _not = (!_isTransient);
            _and_1 = _not;
          }
          if (!_and_1) {
            _and = false;
          } else {
            boolean _isStatic = a.isStatic();
            boolean _not_1 = (!_isStatic);
            _and = _not_1;
          }
          if (_and) {
            CharSequence _associationListDeclaration = this.dbGen.associationListDeclaration(a);
            _builder.append(_associationListDeclaration, "");
            _builder.newLineIfNotEmpty();
          } else {
            CharSequence _listAssociation = this.listAssociation(a);
            _builder.append(_listAssociation, "");
            _builder.newLineIfNotEmpty();
          }
        }
      }
    }
    return _builder;
  }
  
  private CharSequence simpleAssociation(final Association a) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("protected ");
    {
      boolean _isTransient = a.isTransient();
      if (_isTransient) {
        _builder.append("transient ");
      }
    }
    {
      boolean _isStatic = a.isStatic();
      if (_isStatic) {
        _builder.append("static ");
      }
    }
    Entity _connectTo = a.getConnectTo();
    String _name = _connectTo.getName();
    _builder.append(_name, "");
    _builder.append(" ");
    String _name_1 = a.getName();
    String _firstLower = StringExtensions.toFirstLower(_name_1);
    _builder.append(_firstLower, "");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  private CharSequence listAssociation(final Association a) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("protected ");
    {
      boolean _isTransient = a.isTransient();
      if (_isTransient) {
        _builder.append("transient ");
      }
    }
    {
      boolean _isStatic = a.isStatic();
      if (_isStatic) {
        _builder.append("static ");
      }
    }
    _builder.append("List<");
    Entity _connectTo = a.getConnectTo();
    String _name = _connectTo.getName();
    _builder.append(_name, "");
    _builder.append("> ");
    String _name_1 = a.getName();
    String _firstLower = StringExtensions.toFirstLower(_name_1);
    _builder.append(_firstLower, "");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  private CharSequence parcelableCreator(final Entity e) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("//");
    _builder.newLine();
    _builder.append("public static final Parcelable.Creator<");
    String _name = e.getName();
    String _firstUpper = StringExtensions.toFirstUpper(_name);
    _builder.append(_firstUpper, "");
    _builder.append("> CREATOR = new Parcelable.Creator<");
    String _name_1 = e.getName();
    String _firstUpper_1 = StringExtensions.toFirstUpper(_name_1);
    _builder.append(_firstUpper_1, "");
    _builder.append(">() {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("public ");
    String _name_2 = e.getName();
    String _firstUpper_2 = StringExtensions.toFirstUpper(_name_2);
    _builder.append(_firstUpper_2, "\t");
    _builder.append(" createFromParcel(Parcel pc) {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t");
    _builder.append("return new ");
    String _name_3 = e.getName();
    String _firstUpper_3 = StringExtensions.toFirstUpper(_name_3);
    _builder.append(_firstUpper_3, "\t\t");
    _builder.append("(pc);");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("public ");
    String _name_4 = e.getName();
    String _firstUpper_4 = StringExtensions.toFirstUpper(_name_4);
    _builder.append(_firstUpper_4, "\t");
    _builder.append("[] newArray(int size) {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t");
    _builder.append("return new ");
    String _name_5 = e.getName();
    String _firstUpper_5 = StringExtensions.toFirstUpper(_name_5);
    _builder.append(_firstUpper_5, "\t\t");
    _builder.append("[size];");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("};");
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence constructor(final Entity e) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("//");
    _builder.newLine();
    CharSequence _emptyConstructor = this.emptyConstructor(e);
    _builder.append(_emptyConstructor, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    CharSequence _copyConstructor = this.copyConstructor(e);
    _builder.append(_copyConstructor, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    CharSequence _parcelConstructor = this.parcelConstructor(e);
    _builder.append(_parcelConstructor, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence constructorOnlyGEN(final Entity e) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("//");
    _builder.newLine();
    CharSequence _emptyConstructorOnlyGEN = this.emptyConstructorOnlyGEN(e);
    _builder.append(_emptyConstructorOnlyGEN, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    CharSequence _copyConstructorOnlyGEN = this.copyConstructorOnlyGEN(e);
    _builder.append(_copyConstructorOnlyGEN, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    CharSequence _parcelConstructorOnlyGEN = this.parcelConstructorOnlyGEN(e);
    _builder.append(_parcelConstructorOnlyGEN, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence emptyConstructorJavaDoc(final Entity e) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("/**");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <p>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* Empty constructor of ");
    String _name = e.getName();
    _builder.append(_name, " ");
    _builder.append(" entity class.");
    _builder.newLineIfNotEmpty();
    _builder.append(" ");
    _builder.append("* </p>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <!-- end-user-doc -->");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* @see hu.bme.hellga.onlab.OnlabPackage#getEntity()");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* @model");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* @generated");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*/");
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence emptyConstructor(final Entity e) {
    StringConcatenation _builder = new StringConcatenation();
    {
      GeneratorHelper _genHelp = this.wrappedEntity.getGenHelp();
      boolean _equals = _genHelp.equals(GeneratorHelper.JoinClass);
      if (_equals) {
        _builder.append("public ");
      } else {
        _builder.append("protected ");
      }
    }
    String _name = e.getName();
    String _firstUpper = StringExtensions.toFirstUpper(_name);
    _builder.append(_firstUpper, "");
    _builder.append("GEN(){");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("super();");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence emptyConstructorOnlyGEN(final Entity e) {
    StringConcatenation _builder = new StringConcatenation();
    {
      GeneratorHelper _genHelp = this.wrappedEntity.getGenHelp();
      boolean _equals = _genHelp.equals(GeneratorHelper.JoinClass);
      if (_equals) {
        _builder.append("public ");
      } else {
        _builder.append("protected ");
      }
    }
    String _name = e.getName();
    String _firstUpper = StringExtensions.toFirstUpper(_name);
    _builder.append(_firstUpper, "");
    _builder.append("(){");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("super();");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence copyConstructor(final Entity e) {
    StringConcatenation _builder = new StringConcatenation();
    {
      GeneratorHelper _genHelp = this.wrappedEntity.getGenHelp();
      boolean _equals = _genHelp.equals(GeneratorHelper.JoinClass);
      if (_equals) {
        _builder.append("public ");
      } else {
        _builder.append("protected ");
      }
    }
    _builder.append(" ");
    String _name = e.getName();
    String _firstUpper = StringExtensions.toFirstUpper(_name);
    _builder.append(_firstUpper, "");
    _builder.append("GEN(");
    String _name_1 = e.getName();
    String _firstUpper_1 = StringExtensions.toFirstUpper(_name_1);
    _builder.append(_firstUpper_1, "");
    _builder.append(" ");
    String _name_2 = e.getName();
    String _firstLower = StringExtensions.toFirstLower(_name_2);
    _builder.append(_firstLower, "");
    _builder.append("){");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("super();");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence copyConstructorOnlyGEN(final Entity e) {
    StringConcatenation _builder = new StringConcatenation();
    {
      GeneratorHelper _genHelp = this.wrappedEntity.getGenHelp();
      boolean _equals = _genHelp.equals(GeneratorHelper.JoinClass);
      if (_equals) {
        _builder.append("public ");
      } else {
        _builder.append("protected ");
      }
    }
    String _name = e.getName();
    String _firstUpper = StringExtensions.toFirstUpper(_name);
    _builder.append(_firstUpper, "");
    _builder.append("(");
    String _name_1 = e.getName();
    String _firstUpper_1 = StringExtensions.toFirstUpper(_name_1);
    _builder.append(_firstUpper_1, "");
    _builder.append(" ");
    String _name_2 = e.getName();
    String _firstLower = StringExtensions.toFirstLower(_name_2);
    _builder.append(_firstLower, "");
    _builder.append("){");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("super();");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence parcelConstructor(final Entity e) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("public ");
    String _name = e.getName();
    String _firstUpper = StringExtensions.toFirstUpper(_name);
    _builder.append(_firstUpper, "");
    _builder.append("GEN(Parcel pc){");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("super();");
    _builder.newLine();
    _builder.append("\t");
    _builder.newLine();
    {
      EList<Property> _property = e.getProperty();
      for(final Property f : _property) {
        _builder.append("\t");
        CharSequence _parcelProperty = this.parcelProperty(f);
        _builder.append(_parcelProperty, "\t");
        _builder.newLineIfNotEmpty();
      }
    }
    {
      EList<Association> _association = e.getAssociation();
      for(final Association f_1 : _association) {
        _builder.append("\t");
        CharSequence _parcelAssociation = this.parcelAssociation(f_1);
        _builder.append(_parcelAssociation, "\t");
        _builder.newLineIfNotEmpty();
      }
    }
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence parcelConstructorOnlyGEN(final Entity e) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("public ");
    String _name = e.getName();
    String _firstUpper = StringExtensions.toFirstUpper(_name);
    _builder.append(_firstUpper, "");
    _builder.append("(Parcel pc){");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("super();");
    _builder.newLine();
    _builder.append("\t");
    _builder.newLine();
    {
      EList<Property> _property = e.getProperty();
      for(final Property f : _property) {
        _builder.append("\t");
        CharSequence _parcelPropertyBack = this.parcelPropertyBack(f);
        _builder.append(_parcelPropertyBack, "\t");
        _builder.newLineIfNotEmpty();
      }
    }
    {
      EList<Association> _association = e.getAssociation();
      for(final Association f_1 : _association) {
        _builder.append("\t");
        CharSequence _parcelAssociationBack = this.parcelAssociationBack(f_1);
        _builder.append(_parcelAssociationBack, "\t");
        _builder.newLineIfNotEmpty();
      }
    }
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence parcelPropertyBack(final Property p) {
    StringConcatenation _builder = new StringConcatenation();
    {
      Enumeration _enumeration = p.getEnumeration();
      boolean _notEquals = (!Objects.equal(_enumeration, null));
      if (_notEquals) {
        String _name = p.getName();
        String _firstLower = StringExtensions.toFirstLower(_name);
        _builder.append(_firstLower, "");
        _builder.append(" = (");
        Enumeration _enumeration_1 = p.getEnumeration();
        String _name_1 = _enumeration_1.getName();
        String _firstUpper = StringExtensions.toFirstUpper(_name_1);
        _builder.append(_firstUpper, "");
        _builder.append(") pc.readValue(");
        Enumeration _enumeration_2 = p.getEnumeration();
        String _name_2 = _enumeration_2.getName();
        String _firstUpper_1 = StringExtensions.toFirstUpper(_name_2);
        _builder.append(_firstUpper_1, "");
        _builder.append(".class.getClassLoader());");
      } else {
        PropertyType _type = p.getType();
        boolean _equals = _type.equals(PropertyType.DATE);
        if (_equals) {
          String _name_3 = p.getName();
          String _firstLower_1 = StringExtensions.toFirstLower(_name_3);
          _builder.append(_firstLower_1, "");
          _builder.append(" = (");
          PropertyType _type_1 = p.getType();
          String _formatClass = FormatPropertyType.formatClass(_type_1);
          _builder.append(_formatClass, "");
          _builder.append(") pc.readValue(");
          PropertyType _type_2 = p.getType();
          String _formatClass_1 = FormatPropertyType.formatClass(_type_2);
          _builder.append(_formatClass_1, "");
          _builder.append(".class.getClassLoader());");
        } else {
          PropertyType _type_3 = p.getType();
          boolean _equals_1 = _type_3.equals(PropertyType.STRING);
          if (_equals_1) {
            String _name_4 = p.getName();
            String _firstLower_2 = StringExtensions.toFirstLower(_name_4);
            _builder.append(_firstLower_2, "");
            _builder.append(" = pc.readString();");
          } else {
            PropertyType _type_4 = p.getType();
            boolean _equals_2 = _type_4.equals(PropertyType.INTEGER);
            if (_equals_2) {
              String _name_5 = p.getName();
              String _firstLower_3 = StringExtensions.toFirstLower(_name_5);
              _builder.append(_firstLower_3, "");
              _builder.append(" = pc.readInt();");
            }
          }
        }
      }
    }
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  private CharSequence parcelAssociationBack(final Association a) {
    StringConcatenation _builder = new StringConcatenation();
    String _name = a.getName();
    String _firstLower = StringExtensions.toFirstLower(_name);
    _builder.append(_firstLower, "");
    _builder.append(" = (");
    String _name_1 = a.getName();
    String _firstUpper = StringExtensions.toFirstUpper(_name_1);
    _builder.append(_firstUpper, "");
    _builder.append(") pc.readValue(");
    String _name_2 = a.getName();
    String _firstUpper_1 = StringExtensions.toFirstUpper(_name_2);
    _builder.append(_firstUpper_1, "");
    _builder.append(".class.getClassLoader());");
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  private CharSequence parcelMethods(final Entity e) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.newLine();
    CharSequence _parcelDesribeContents = this.parcelDesribeContents();
    _builder.append(_parcelDesribeContents, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    CharSequence _parcelWriteToLocalParcel = this.parcelWriteToLocalParcel(e);
    _builder.append(_parcelWriteToLocalParcel, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence parcelDesribeContents() {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("@Override");
    _builder.newLine();
    _builder.append("public int describeContents() {");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("return 0;");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence parcelWriteToLocalParcel(final Entity e) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("@Override");
    _builder.newLine();
    _builder.append("public void writeToParcel(Parcel dest, int flags) {");
    _builder.newLine();
    {
      EList<Property> _property = e.getProperty();
      for(final Property f : _property) {
        _builder.append("\t");
        CharSequence _parcelProperty = this.parcelProperty(f);
        _builder.append(_parcelProperty, "\t");
        _builder.newLineIfNotEmpty();
      }
    }
    {
      EList<Association> _association = e.getAssociation();
      for(final Association f_1 : _association) {
        _builder.append("\t");
        CharSequence _parcelAssociation = this.parcelAssociation(f_1);
        _builder.append(_parcelAssociation, "\t");
        _builder.newLineIfNotEmpty();
      }
    }
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence parcelProperty(final Property p) {
    StringConcatenation _builder = new StringConcatenation();
    {
      boolean _or = false;
      Enumeration _enumeration = p.getEnumeration();
      boolean _notEquals = (!Objects.equal(_enumeration, null));
      if (_notEquals) {
        _or = true;
      } else {
        PropertyType _type = p.getType();
        boolean _equals = _type.equals(PropertyType.DATE);
        _or = _equals;
      }
      if (_or) {
        _builder.append("dest.writeValue(");
        String _name = p.getName();
        String _firstLower = StringExtensions.toFirstLower(_name);
        _builder.append(_firstLower, "");
        _builder.append(");");
      } else {
        PropertyType _type_1 = p.getType();
        boolean _equals_1 = _type_1.equals(PropertyType.STRING);
        if (_equals_1) {
          _builder.append("dest.writeString(");
          String _name_1 = p.getName();
          String _firstLower_1 = StringExtensions.toFirstLower(_name_1);
          _builder.append(_firstLower_1, "");
          _builder.append(");");
        } else {
          PropertyType _type_2 = p.getType();
          boolean _equals_2 = _type_2.equals(PropertyType.INTEGER);
          if (_equals_2) {
            _builder.append("dest.writeInt(");
            String _name_2 = p.getName();
            String _firstLower_2 = StringExtensions.toFirstLower(_name_2);
            _builder.append(_firstLower_2, "");
            _builder.append(");");
          }
        }
      }
    }
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  private CharSequence parcelAssociation(final Association a) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("dest.writeValue(");
    String _name = a.getName();
    String _firstLower = StringExtensions.toFirstLower(_name);
    _builder.append(_firstLower, "");
    _builder.append(");");
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  private CharSequence getterSetters(final Entity e) {
    StringConcatenation _builder = new StringConcatenation();
    {
      PersistentType _persistentType = e.getPersistentType();
      boolean _equals = _persistentType.equals(PersistentType.DB);
      if (_equals) {
        CharSequence _idGetter = this.dbGen.idGetter(e);
        _builder.append(_idGetter, "");
        _builder.newLineIfNotEmpty();
        _builder.newLine();
      }
    }
    CharSequence _propertyGettersSetters = this.propertyGettersSetters(e);
    _builder.append(_propertyGettersSetters, "");
    _builder.newLineIfNotEmpty();
    CharSequence _associationGettersSetters = this.associationGettersSetters(e);
    _builder.append(_associationGettersSetters, "");
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  private CharSequence propertyGettersSetters(final Entity e) {
    StringConcatenation _builder = new StringConcatenation();
    {
      EList<Property> _property = e.getProperty();
      for(final Property f : _property) {
        _builder.append("//");
        _builder.newLine();
        CharSequence _propertyGetter = this.propertyGetter(f);
        _builder.append(_propertyGetter, "");
        _builder.newLineIfNotEmpty();
        _builder.newLine();
        _builder.append("//");
        _builder.newLine();
        CharSequence _propertySetter = this.propertySetter(f);
        _builder.append(_propertySetter, "");
        _builder.newLineIfNotEmpty();
        _builder.newLine();
      }
    }
    return _builder;
  }
  
  private CharSequence propertyGetterJavaDoc(final Property p) {
    StringConcatenation _builder = new StringConcatenation();
    {
      ConnectionType _connectionType = p.getConnectionType();
      boolean _equals = _connectionType.equals(ConnectionType.ONE);
      if (_equals) {
        CharSequence _simplePropertyGetterJavaDoc = this.simplePropertyGetterJavaDoc(p);
        _builder.append(_simplePropertyGetterJavaDoc, "");
        _builder.newLineIfNotEmpty();
      } else {
        CharSequence _listPropertyGetterJavaDoc = this.listPropertyGetterJavaDoc(p);
        _builder.append(_listPropertyGetterJavaDoc, "");
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder;
  }
  
  private CharSequence simplePropertyGetterJavaDoc(final Property p) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("/**");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <p>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* ");
    String _name = p.getName();
    _builder.append(_name, " ");
    _builder.append(" sproperty of ");
    String _name_1 = this.entity.getName();
    _builder.append(_name_1, " ");
    _builder.append(" entitiy.");
    _builder.newLineIfNotEmpty();
    _builder.append(" ");
    _builder.append("* </p>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <!-- end-user-doc -->");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <p>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* The following features are supported:");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* @see hu.bme.hellga.onlab.OnlabPackage#getEntity()");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* @model");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* @generated");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*/");
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence listPropertyGetterJavaDoc(final Property p) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("/**");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <p>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* ");
    String _name = p.getName();
    _builder.append(_name, " ");
    _builder.append(" lproperty of ");
    String _name_1 = this.entity.getName();
    _builder.append(_name_1, " ");
    _builder.append(" entitiy.");
    _builder.newLineIfNotEmpty();
    _builder.append(" ");
    _builder.append("* </p>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <!-- end-user-doc -->");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <p>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* The following features are supported:");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* @see hu.bme.hellga.onlab.OnlabPackage#getEntity()");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* @model");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* @generated");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*/");
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence propertyGetter(final Property p) {
    StringConcatenation _builder = new StringConcatenation();
    {
      ConnectionType _connectionType = p.getConnectionType();
      boolean _equals = _connectionType.equals(ConnectionType.ONE);
      if (_equals) {
        CharSequence _simplePropertyGetter = this.simplePropertyGetter(p);
        _builder.append(_simplePropertyGetter, "");
        _builder.newLineIfNotEmpty();
      } else {
        CharSequence _listPropertyGetter = this.listPropertyGetter(p);
        _builder.append(_listPropertyGetter, "");
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder;
  }
  
  private CharSequence simplePropertyGetter(final Property p) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("public ");
    {
      boolean _isStatic = p.isStatic();
      if (_isStatic) {
        _builder.append("static ");
      }
    }
    {
      Enumeration _enumeration = p.getEnumeration();
      boolean _notEquals = (!Objects.equal(_enumeration, null));
      if (_notEquals) {
        Enumeration _enumeration_1 = p.getEnumeration();
        String _name = _enumeration_1.getName();
        String _firstUpper = StringExtensions.toFirstUpper(_name);
        _builder.append(_firstUpper, "");
      } else {
        PropertyType _type = p.getType();
        String _formatPrimitive = FormatPropertyType.formatPrimitive(_type);
        _builder.append(_formatPrimitive, "");
      }
    }
    _builder.append(" get");
    String _name_1 = p.getName();
    String _firstUpper_1 = StringExtensions.toFirstUpper(_name_1);
    _builder.append(_firstUpper_1, "");
    _builder.append("(){");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("return ");
    {
      boolean _isStatic_1 = p.isStatic();
      boolean _not = (!_isStatic_1);
      if (_not) {
        _builder.append("this.");
      }
    }
    String _name_2 = p.getName();
    String _firstLower = StringExtensions.toFirstLower(_name_2);
    _builder.append(_firstLower, "\t");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence listPropertyGetter(final Property p) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("public ");
    {
      boolean _isStatic = p.isStatic();
      if (_isStatic) {
        _builder.append("static ");
      }
    }
    _builder.append("List<");
    {
      Enumeration _enumeration = p.getEnumeration();
      boolean _notEquals = (!Objects.equal(_enumeration, null));
      if (_notEquals) {
        Enumeration _enumeration_1 = p.getEnumeration();
        String _name = _enumeration_1.getName();
        String _firstUpper = StringExtensions.toFirstUpper(_name);
        _builder.append(_firstUpper, "");
      } else {
        PropertyType _type = p.getType();
        String _formatClass = FormatPropertyType.formatClass(_type);
        _builder.append(_formatClass, "");
      }
    }
    _builder.append("> get");
    String _name_1 = p.getName();
    String _firstUpper_1 = StringExtensions.toFirstUpper(_name_1);
    _builder.append(_firstUpper_1, "");
    _builder.append("(){");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("return ");
    {
      boolean _isStatic_1 = p.isStatic();
      boolean _not = (!_isStatic_1);
      if (_not) {
        _builder.append("this.");
      }
    }
    String _name_2 = p.getName();
    String _firstLower = StringExtensions.toFirstLower(_name_2);
    _builder.append(_firstLower, "\t");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence propertySetterJavaDoc(final Property p) {
    StringConcatenation _builder = new StringConcatenation();
    {
      ConnectionType _connectionType = p.getConnectionType();
      boolean _equals = _connectionType.equals(ConnectionType.ONE);
      if (_equals) {
        CharSequence _simplePropertySetterJavaDoc = this.simplePropertySetterJavaDoc(p);
        _builder.append(_simplePropertySetterJavaDoc, "");
        _builder.newLineIfNotEmpty();
      } else {
        CharSequence _listPropertySetterJavaDoc = this.listPropertySetterJavaDoc(p);
        _builder.append(_listPropertySetterJavaDoc, "");
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder;
  }
  
  private CharSequence simplePropertySetterJavaDoc(final Property p) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("/**");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <p>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* ");
    String _name = p.getName();
    _builder.append(_name, " ");
    _builder.append(" sproperty of ");
    String _name_1 = this.entity.getName();
    _builder.append(_name_1, " ");
    _builder.append(" entitiy.");
    _builder.newLineIfNotEmpty();
    _builder.append(" ");
    _builder.append("* </p>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <!-- end-user-doc -->");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <p>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* The following features are supported:");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* @see hu.bme.hellga.onlab.OnlabPackage#getEntity()");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* @model");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* @generated");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*/");
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence listPropertySetterJavaDoc(final Property p) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("/**");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <p>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* ");
    String _name = p.getName();
    _builder.append(_name, " ");
    _builder.append(" lproperty of ");
    String _name_1 = this.entity.getName();
    _builder.append(_name_1, " ");
    _builder.append(" entitiy.");
    _builder.newLineIfNotEmpty();
    _builder.append(" ");
    _builder.append("* </p>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <!-- end-user-doc -->");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <p>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* The following features are supported:");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* @see hu.bme.hellga.onlab.OnlabPackage#getEntity()");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* @model");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* @generated");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*/");
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence propertySetter(final Property p) {
    StringConcatenation _builder = new StringConcatenation();
    {
      ConnectionType _connectionType = p.getConnectionType();
      boolean _equals = _connectionType.equals(ConnectionType.ONE);
      if (_equals) {
        CharSequence _simplePropertySetter = this.simplePropertySetter(p);
        _builder.append(_simplePropertySetter, "");
        _builder.newLineIfNotEmpty();
      } else {
        CharSequence _listPropertySetter = this.listPropertySetter(p);
        _builder.append(_listPropertySetter, "");
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder;
  }
  
  private CharSequence simplePropertySetter(final Property p) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("public ");
    {
      boolean _isStatic = p.isStatic();
      if (_isStatic) {
        _builder.append("static ");
      }
    }
    _builder.append("void set");
    String _name = p.getName();
    String _firstUpper = StringExtensions.toFirstUpper(_name);
    _builder.append(_firstUpper, "");
    _builder.append("(");
    {
      Enumeration _enumeration = p.getEnumeration();
      boolean _notEquals = (!Objects.equal(_enumeration, null));
      if (_notEquals) {
        Enumeration _enumeration_1 = p.getEnumeration();
        String _name_1 = _enumeration_1.getName();
        String _firstUpper_1 = StringExtensions.toFirstUpper(_name_1);
        _builder.append(_firstUpper_1, "");
      } else {
        PropertyType _type = p.getType();
        String _formatPrimitive = FormatPropertyType.formatPrimitive(_type);
        _builder.append(_formatPrimitive, "");
      }
    }
    _builder.append(" ");
    String _name_2 = p.getName();
    String _firstLower = StringExtensions.toFirstLower(_name_2);
    _builder.append(_firstLower, "");
    _builder.append("){");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    {
      boolean _isStatic_1 = p.isStatic();
      boolean _not = (!_isStatic_1);
      if (_not) {
        _builder.append("this.");
      } else {
        String _name_3 = this.entity.getName();
        String _firstUpper_2 = StringExtensions.toFirstUpper(_name_3);
        _builder.append(_firstUpper_2, "\t");
        {
          if ((!this.only)) {
            _builder.append("GEN");
          }
        }
        _builder.append(".");
      }
    }
    String _name_4 = p.getName();
    String _firstLower_1 = StringExtensions.toFirstLower(_name_4);
    _builder.append(_firstLower_1, "\t");
    _builder.append(" = ");
    String _name_5 = p.getName();
    String _firstLower_2 = StringExtensions.toFirstLower(_name_5);
    _builder.append(_firstLower_2, "\t");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence listPropertySetter(final Property p) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("public ");
    {
      boolean _isStatic = p.isStatic();
      if (_isStatic) {
        _builder.append("static ");
      }
    }
    _builder.append("void set");
    String _name = p.getName();
    String _firstUpper = StringExtensions.toFirstUpper(_name);
    _builder.append(_firstUpper, "");
    _builder.append("(List<");
    {
      Enumeration _enumeration = p.getEnumeration();
      boolean _notEquals = (!Objects.equal(_enumeration, null));
      if (_notEquals) {
        Enumeration _enumeration_1 = p.getEnumeration();
        String _name_1 = _enumeration_1.getName();
        String _firstUpper_1 = StringExtensions.toFirstUpper(_name_1);
        _builder.append(_firstUpper_1, "");
      } else {
        PropertyType _type = p.getType();
        String _formatClass = FormatPropertyType.formatClass(_type);
        _builder.append(_formatClass, "");
      }
    }
    _builder.append("> ");
    String _name_2 = p.getName();
    String _firstLower = StringExtensions.toFirstLower(_name_2);
    _builder.append(_firstLower, "");
    _builder.append("){");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    {
      boolean _isStatic_1 = p.isStatic();
      boolean _not = (!_isStatic_1);
      if (_not) {
        _builder.append("this.");
      } else {
        String _name_3 = this.entity.getName();
        String _firstUpper_2 = StringExtensions.toFirstUpper(_name_3);
        _builder.append(_firstUpper_2, "\t");
        {
          if ((!this.only)) {
            _builder.append("GEN");
          }
        }
        _builder.append(".");
      }
    }
    String _name_4 = p.getName();
    String _firstLower_1 = StringExtensions.toFirstLower(_name_4);
    _builder.append(_firstLower_1, "\t");
    _builder.append(" = ");
    String _name_5 = p.getName();
    String _firstLower_2 = StringExtensions.toFirstLower(_name_5);
    _builder.append(_firstLower_2, "\t");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence associationGettersSetters(final Entity e) {
    StringConcatenation _builder = new StringConcatenation();
    {
      EList<Association> _association = e.getAssociation();
      for(final Association f : _association) {
        _builder.append("//");
        _builder.newLine();
        CharSequence _associationGetter = this.associationGetter(f);
        _builder.append(_associationGetter, "");
        _builder.newLineIfNotEmpty();
        _builder.newLine();
        _builder.append("//");
        _builder.newLine();
        CharSequence _associationSetter = this.associationSetter(f);
        _builder.append(_associationSetter, "");
        _builder.newLineIfNotEmpty();
        _builder.newLine();
      }
    }
    return _builder;
  }
  
  private CharSequence associationGetterJavaDoc(final Association a) {
    StringConcatenation _builder = new StringConcatenation();
    {
      ConnectionType _connectionType = a.getConnectionType();
      boolean _equals = _connectionType.equals(ConnectionType.ONE);
      if (_equals) {
        CharSequence _simpleAssociationGetterJavaDoc = this.simpleAssociationGetterJavaDoc(a);
        _builder.append(_simpleAssociationGetterJavaDoc, "");
        _builder.newLineIfNotEmpty();
      } else {
        CharSequence _listAssociationGetterJavaDoc = this.listAssociationGetterJavaDoc(a);
        _builder.append(_listAssociationGetterJavaDoc, "");
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder;
  }
  
  private CharSequence simpleAssociationGetterJavaDoc(final Association a) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("/**");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <p>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* ");
    String _name = a.getName();
    _builder.append(_name, " ");
    _builder.append(" sassociation of ");
    String _name_1 = this.entity.getName();
    _builder.append(_name_1, " ");
    _builder.append(" entitiy.");
    _builder.newLineIfNotEmpty();
    _builder.append(" ");
    _builder.append("* </p>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <!-- end-user-doc -->");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <p>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* The following features are supported:");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* @see hu.bme.hellga.onlab.OnlabPackage#getEntity()");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* @model");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* @generated");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*/");
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence listAssociationGetterJavaDoc(final Association a) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("/**");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <p>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* ");
    String _name = a.getName();
    _builder.append(_name, " ");
    _builder.append(" lassociation of ");
    String _name_1 = this.entity.getName();
    _builder.append(_name_1, " ");
    _builder.append(" entitiy.");
    _builder.newLineIfNotEmpty();
    _builder.append(" ");
    _builder.append("* </p>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <!-- end-user-doc -->");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <p>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* The following features are supported:");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* @see hu.bme.hellga.onlab.OnlabPackage#getEntity()");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* @model");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* @generated");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*/");
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence associationGetter(final Association a) {
    StringConcatenation _builder = new StringConcatenation();
    {
      ConnectionType _connectionType = a.getConnectionType();
      boolean _equals = _connectionType.equals(ConnectionType.ONE);
      if (_equals) {
        CharSequence _simpleAssociationGetter = this.simpleAssociationGetter(a);
        _builder.append(_simpleAssociationGetter, "");
        _builder.newLineIfNotEmpty();
      } else {
        {
          boolean _and = false;
          boolean _and_1 = false;
          PersistentType _persistentType = this.entity.getPersistentType();
          boolean _equals_1 = _persistentType.equals(PersistentType.DB);
          if (!_equals_1) {
            _and_1 = false;
          } else {
            boolean _isStatic = a.isStatic();
            boolean _not = (!_isStatic);
            _and_1 = _not;
          }
          if (!_and_1) {
            _and = false;
          } else {
            boolean _isTransient = a.isTransient();
            boolean _not_1 = (!_isTransient);
            _and = _not_1;
          }
          if (_and) {
            CharSequence _associationListGetter = this.dbGen.associationListGetter(a);
            _builder.append(_associationListGetter, "");
            _builder.newLineIfNotEmpty();
          } else {
            CharSequence _listAssociationGetter = this.listAssociationGetter(a);
            _builder.append(_listAssociationGetter, "");
            _builder.newLineIfNotEmpty();
          }
        }
      }
    }
    return _builder;
  }
  
  private CharSequence simpleAssociationGetter(final Association a) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("public ");
    {
      boolean _isStatic = a.isStatic();
      if (_isStatic) {
        _builder.append("static ");
      }
    }
    Entity _connectTo = a.getConnectTo();
    String _name = _connectTo.getName();
    _builder.append(_name, "");
    _builder.append(" get");
    String _name_1 = a.getName();
    String _firstUpper = StringExtensions.toFirstUpper(_name_1);
    _builder.append(_firstUpper, "");
    _builder.append("(){");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("return ");
    {
      boolean _isStatic_1 = a.isStatic();
      boolean _not = (!_isStatic_1);
      if (_not) {
        _builder.append("this.");
      }
    }
    String _name_2 = a.getName();
    String _firstLower = StringExtensions.toFirstLower(_name_2);
    _builder.append(_firstLower, "\t");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence listAssociationGetter(final Association a) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("public ");
    {
      boolean _isStatic = a.isStatic();
      if (_isStatic) {
        _builder.append("static ");
      }
    }
    _builder.append("List<");
    Entity _connectTo = a.getConnectTo();
    String _name = _connectTo.getName();
    _builder.append(_name, "");
    _builder.append("> get");
    String _name_1 = a.getName();
    String _firstUpper = StringExtensions.toFirstUpper(_name_1);
    _builder.append(_firstUpper, "");
    _builder.append("(){");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("return ");
    {
      boolean _isStatic_1 = a.isStatic();
      boolean _not = (!_isStatic_1);
      if (_not) {
        _builder.append("this.");
      }
    }
    String _name_2 = a.getName();
    String _firstLower = StringExtensions.toFirstLower(_name_2);
    _builder.append(_firstLower, "\t");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence associationSetterJavaDoc(final Association a) {
    StringConcatenation _builder = new StringConcatenation();
    {
      ConnectionType _connectionType = a.getConnectionType();
      boolean _equals = _connectionType.equals(ConnectionType.ONE);
      if (_equals) {
        CharSequence _simpleAssociationSetterJavaDoc = this.simpleAssociationSetterJavaDoc(a);
        _builder.append(_simpleAssociationSetterJavaDoc, "");
        _builder.newLineIfNotEmpty();
      } else {
        CharSequence _listAssociationSetterJavaDoc = this.listAssociationSetterJavaDoc(a);
        _builder.append(_listAssociationSetterJavaDoc, "");
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder;
  }
  
  private CharSequence simpleAssociationSetterJavaDoc(final Association a) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("/**");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <p>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* ");
    String _name = a.getName();
    _builder.append(_name, " ");
    _builder.append(" sassociation of ");
    String _name_1 = this.entity.getName();
    _builder.append(_name_1, " ");
    _builder.append(" entitiy.");
    _builder.newLineIfNotEmpty();
    _builder.append(" ");
    _builder.append("* </p>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <!-- end-user-doc -->");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <p>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* The following features are supported:");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* @see hu.bme.hellga.onlab.OnlabPackage#getEntity()");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* @model");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* @generated");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*/");
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence listAssociationSetterJavaDoc(final Association a) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("/**");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <p>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* ");
    String _name = a.getName();
    _builder.append(_name, " ");
    _builder.append(" lassociation of ");
    String _name_1 = this.entity.getName();
    _builder.append(_name_1, " ");
    _builder.append(" entitiy.");
    _builder.newLineIfNotEmpty();
    _builder.append(" ");
    _builder.append("* </p>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <!-- end-user-doc -->");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <p>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* The following features are supported:");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* @see hu.bme.hellga.onlab.OnlabPackage#getEntity()");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* @model");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* @generated");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*/");
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence associationSetter(final Association a) {
    StringConcatenation _builder = new StringConcatenation();
    {
      ConnectionType _connectionType = a.getConnectionType();
      boolean _equals = _connectionType.equals(ConnectionType.ONE);
      if (_equals) {
        CharSequence _simpleAssociationSetter = this.simpleAssociationSetter(a);
        _builder.append(_simpleAssociationSetter, "");
        _builder.newLineIfNotEmpty();
      } else {
        {
          boolean _and = false;
          boolean _and_1 = false;
          PersistentType _persistentType = this.entity.getPersistentType();
          boolean _equals_1 = _persistentType.equals(PersistentType.DB);
          if (!_equals_1) {
            _and_1 = false;
          } else {
            boolean _isStatic = a.isStatic();
            boolean _not = (!_isStatic);
            _and_1 = _not;
          }
          if (!_and_1) {
            _and = false;
          } else {
            boolean _isTransient = a.isTransient();
            boolean _not_1 = (!_isTransient);
            _and = _not_1;
          }
          if (_and) {
            CharSequence _associationListSetter = this.dbGen.associationListSetter(a);
            _builder.append(_associationListSetter, "");
            _builder.newLineIfNotEmpty();
          } else {
            CharSequence _listAssociationSetter = this.listAssociationSetter(a);
            _builder.append(_listAssociationSetter, "");
            _builder.newLineIfNotEmpty();
          }
        }
      }
    }
    return _builder;
  }
  
  private CharSequence simpleAssociationSetter(final Association a) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("public ");
    {
      boolean _isStatic = a.isStatic();
      if (_isStatic) {
        _builder.append("static ");
      }
    }
    _builder.append("void set");
    String _name = a.getName();
    String _firstUpper = StringExtensions.toFirstUpper(_name);
    _builder.append(_firstUpper, "");
    _builder.append("(");
    Entity _connectTo = a.getConnectTo();
    String _name_1 = _connectTo.getName();
    _builder.append(_name_1, "");
    _builder.append(" ");
    String _name_2 = a.getName();
    String _firstLower = StringExtensions.toFirstLower(_name_2);
    _builder.append(_firstLower, "");
    _builder.append("){");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    {
      boolean _isStatic_1 = a.isStatic();
      boolean _not = (!_isStatic_1);
      if (_not) {
        _builder.append("this.");
      } else {
        String _name_3 = this.entity.getName();
        String _firstUpper_1 = StringExtensions.toFirstUpper(_name_3);
        _builder.append(_firstUpper_1, "\t");
        {
          if ((!this.only)) {
            _builder.append("GEN");
          }
        }
        _builder.append(".");
      }
    }
    String _name_4 = a.getName();
    String _firstLower_1 = StringExtensions.toFirstLower(_name_4);
    _builder.append(_firstLower_1, "\t");
    _builder.append(" = ");
    String _name_5 = a.getName();
    String _firstLower_2 = StringExtensions.toFirstLower(_name_5);
    _builder.append(_firstLower_2, "\t");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence listAssociationSetter(final Association a) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("public ");
    {
      boolean _isStatic = a.isStatic();
      if (_isStatic) {
        _builder.append("static ");
      }
    }
    _builder.append("void set");
    String _name = a.getName();
    String _firstUpper = StringExtensions.toFirstUpper(_name);
    _builder.append(_firstUpper, "");
    _builder.append("(List<");
    Entity _connectTo = a.getConnectTo();
    String _name_1 = _connectTo.getName();
    _builder.append(_name_1, "");
    _builder.append("> ");
    String _name_2 = a.getName();
    String _firstLower = StringExtensions.toFirstLower(_name_2);
    _builder.append(_firstLower, "");
    _builder.append("){");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    {
      boolean _isStatic_1 = a.isStatic();
      boolean _not = (!_isStatic_1);
      if (_not) {
        _builder.append("this.");
      } else {
        String _name_3 = this.entity.getName();
        String _firstUpper_1 = StringExtensions.toFirstUpper(_name_3);
        _builder.append(_firstUpper_1, "\t");
        {
          if ((!this.only)) {
            _builder.append("GEN");
          }
        }
        _builder.append(".");
      }
    }
    String _name_4 = a.getName();
    String _firstLower_1 = StringExtensions.toFirstLower(_name_4);
    _builder.append(_firstLower_1, "\t");
    _builder.append(" = ");
    String _name_5 = a.getName();
    String _firstLower_2 = StringExtensions.toFirstLower(_name_5);
    _builder.append(_firstLower_2, "\t");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
}
