package hu.bme.mit.mobilegen.datamodel.generator.entity;

import com.google.common.base.Objects;
import com.google.inject.Inject;
import hu.bme.mit.mobilegen.datamodel.datamodel.Entity;
import hu.bme.mit.mobilegen.datamodel.datamodel.PersistentType;
import hu.bme.mit.mobilegen.datamodel.generator.GeneratorConstants;
import hu.bme.mit.mobilegen.datamodel.generator.GeneratorData;
import hu.bme.mit.mobilegen.datamodel.generator.database.IDatabaseGeneratorUtility;
import hu.bme.mit.mobilegen.datamodel.helper.WrappedEntity;
import hu.bme.mit.mobilegen.datamodel.logger.Log;
import java.util.ArrayList;
import java.util.List;
import org.eclipse.xtend2.lib.StringConcatenation;
import org.eclipse.xtext.generator.IFileSystemAccess;
import org.eclipse.xtext.xbase.lib.StringExtensions;

@SuppressWarnings("all")
public class EntityGenerator {
  @Inject
  private IDatabaseGeneratorUtility dbGen;
  
  private WrappedEntity wrappedEntity;
  
  private Entity entity;
  
  private List<String> imports;
  
  public void generateEntity(final WrappedEntity we, final IFileSystemAccess fsa) {
    this.wrappedEntity = we;
    Entity _entity = we.getEntity();
    this.entity = _entity;
    this.collectImports();
    PersistentType _persistentType = this.entity.getPersistentType();
    boolean _equals = Objects.equal(_persistentType, PersistentType.DB);
    if (_equals) {
      this.dbGen.collectEntityImports(this.imports, this.wrappedEntity);
    }
    String _name = this.entity.getName();
    String _firstUpper = StringExtensions.toFirstUpper(_name);
    String _plus = ("Generating " + _firstUpper);
    String _plus_1 = (_plus + ".java");
    Log.info(_plus_1);
    String _name_1 = this.entity.getName();
    String _firstUpper_1 = StringExtensions.toFirstUpper(_name_1);
    String _plus_2 = (_firstUpper_1 + ".java");
    CharSequence _generateEntity = this.generateEntity(this.entity);
    fsa.generateFile(_plus_2, GeneratorConstants.SRC_ENTITIES, _generateEntity);
  }
  
  private List<String> collectImports() {
    ArrayList<String> _arrayList = new ArrayList<String>();
    return this.imports = _arrayList;
  }
  
  private CharSequence generateEntity(final Entity e) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _classHeader = this.classHeader(e);
    _builder.append(_classHeader, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    CharSequence _packageDeclaration = this.packageDeclaration();
    _builder.append(_packageDeclaration, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    CharSequence _classImports = this.classImports(e);
    _builder.append(_classImports, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("//");
    _builder.newLine();
    CharSequence _classAnnotations = this.classAnnotations(e);
    _builder.append(_classAnnotations, "");
    _builder.newLineIfNotEmpty();
    CharSequence _classDeclaration = this.classDeclaration(e);
    _builder.append(_classDeclaration, "");
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  private CharSequence classHeader(final Entity e) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("//------------------------- Geneated Java Class File For Entity - ");
    String _name = e.getName();
    _builder.append(_name, "");
    _builder.append(" ---------------------------------");
    _builder.newLineIfNotEmpty();
    _builder.append("//------------------------- Model Implementation In ");
    String _resourceURI = this.wrappedEntity.getResourceURI();
    _builder.append(_resourceURI, "");
    _builder.append(" Resource File ----------------------------");
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  private CharSequence packageDeclaration() {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("package ");
    String _packageName = GeneratorData.getPackageName();
    _builder.append(_packageName, "");
    _builder.append(".");
    _builder.append(GeneratorConstants.SRC_GEN_ENTITIES, "");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  private CharSequence classImports(final Entity e) {
    StringConcatenation _builder = new StringConcatenation();
    {
      for(final String f : this.imports) {
        _builder.append("import ");
        _builder.append(f, "");
        _builder.append(";");
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder;
  }
  
  private CharSequence classJavaDoc(final Entity e) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("/**");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <!-- begin-user-doc -->");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* This Class contains the representation of the model object \'<em><b");
    String _name = e.getName();
    _builder.append(_name, " ");
    _builder.append("</b></em>\'.");
    _builder.newLineIfNotEmpty();
    _builder.append(" ");
    _builder.append("* For each Entity the generator creates an EntityGEN.java file,");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* which contains EntityGEN class with every Property and Association for the Entity.");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* This Java file will be override, with each generation.");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* The generator creates an Entity.java file, with the Entity class,");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* which extends form EntityGEN. This file normally (if the Entity is not deleted)");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* won\'t be overrided, so this file can be modified between the generations.");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* ");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <p>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* This is The Entity.java file. It is created at the first generation, after that,");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* it can be modified, it won\'t be overrided, until the Entity is renamed, or the");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* Entity is deleted.");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* </p>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <!-- end-user-doc -->");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <p>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* The following features are supported:");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* <ul>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*   <li>{@link hu.bme.hellga.onlab.Entity#getCache <em>Cache</em>}</li>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*   <li>{@link hu.bme.hellga.onlab.Entity#getPersistentType <em>Persistent Type</em>}</li>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*   <li>{@link hu.bme.hellga.onlab.Entity#isAbstract <em>Abstract</em>}</li>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*   <li>{@link hu.bme.hellga.onlab.Entity#getName <em>Name</em>}</li>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*   <li>{@link hu.bme.hellga.onlab.Entity#getSuperType <em>Super Type</em>}</li>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*   <li>{@link hu.bme.hellga.onlab.Entity#getRelationship <em>Relationship</em>}</li>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*   <li>{@link hu.bme.hellga.onlab.Entity#getProperty <em>Property</em>}</li>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* </ul>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* </p>");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* @see hu.bme.hellga.onlab.OnlabPackage#getEntity()");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* @model");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* @generated");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*/");
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence classAnnotations(final Entity e) {
    StringConcatenation _builder = new StringConcatenation();
    {
      PersistentType _persistentType = this.entity.getPersistentType();
      boolean _equals = _persistentType.equals(PersistentType.DB);
      if (_equals) {
        CharSequence _classAnnotations = this.dbGen.classAnnotations(e);
        _builder.append(_classAnnotations, "");
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder;
  }
  
  private CharSequence classDeclaration(final Entity e) {
    StringConcatenation _builder = new StringConcatenation();
    {
      boolean _isAbstract = e.isAbstract();
      if (_isAbstract) {
        _builder.append("abstract ");
      }
    }
    _builder.append("public class ");
    String _name = e.getName();
    String _firstUpper = StringExtensions.toFirstUpper(_name);
    _builder.append(_firstUpper, "");
    _builder.append(" extends ");
    String _name_1 = e.getName();
    String _firstUpper_1 = StringExtensions.toFirstUpper(_name_1);
    _builder.append(_firstUpper_1, "");
    _builder.append("GEN{");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.newLine();
    _builder.append("\t");
    CharSequence _classBody = this.classBody(e);
    _builder.append(_classBody, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence classBody(final Entity e) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("// Class body, that can be edited");
    _builder.newLine();
    _builder.newLine();
    _builder.append("// Constructor");
    _builder.newLine();
    _builder.newLine();
    CharSequence _constructor = this.constructor(e);
    _builder.append(_constructor, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    CharSequence _constructorCopy = this.constructorCopy(e);
    _builder.append(_constructorCopy, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("//");
    _builder.newLine();
    _builder.newLine();
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence constructor(final Entity e) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("protected ");
    String _name = e.getName();
    String _firstUpper = StringExtensions.toFirstUpper(_name);
    _builder.append(_firstUpper, "");
    _builder.append("(){");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("super();");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence constructorCopy(final Entity e) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("protected ");
    String _name = e.getName();
    String _firstUpper = StringExtensions.toFirstUpper(_name);
    _builder.append(_firstUpper, "");
    _builder.append("(");
    String _name_1 = e.getName();
    String _firstUpper_1 = StringExtensions.toFirstUpper(_name_1);
    _builder.append(_firstUpper_1, "");
    _builder.append(" ");
    String _name_2 = e.getName();
    String _firstLower = StringExtensions.toFirstLower(_name_2);
    _builder.append(_firstLower, "");
    _builder.append("){");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("super();");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
}
